#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: domain_backup.cgi                                                 #
#                                                                             #
# Description: CGI needed to display and modify backup/restore configurations.#
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use POSIX qw(strftime);
use LanguageTime;
use LanguageResource;
use DisplayMain;
use Authorize;
use Daemon;
use Backup;
use Safe;
use CGI;

###############################################################################
# RenderDefault
###############################################################################

sub RenderDefault
{
	my ($cgi, $DisplayDataRef) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_backup.lrc', 'common.lrc' ]);
	my $TimeResource		= new LanguageTime($PreferredLanguage, '/usr/ins/intel/gui/locale');
	my %_R					= ( 'IDT:6700'					=> '',
								'BACKUP'					=> '',
								'CHOOSE_FILES_TO_BACKUP'	=> '',
								'ENTIRE_DOMAIN'				=> '',
								'PARTIAL_DOMAIN'			=> '',
								'WEB_SITE'					=> '',
								'DOMAIN_E_MAIL'				=> '',
								'DOMAIN_USER_DIRECTORIES'	=> '',
								'VIEW_FILE_LISTING'			=> '',
								'START_BACKUP'				=> '',  	
								'SPECIFY_RESTORE_LOCATION'	=> '',
								'LOCAL_FILE_PATH'			=> '',
								'RESTORE'					=> '',
								'ERROR_EMPTY_PATH'			=> '' );
	my @FilesToBackup		= $cgi->param('files_to_backup');
	my %FormData			= ( 'entire_domain'			=> '',
								'partial_system'   		=> 'checked',
								'domain_web_site'  		=> '',
								'domain_email'	   		=> '',
								'domain_user_home_dirs'	=> 'checked' );
	my $SuggestedFileName;
	my %Parameters;
	my $Level;
	my $Feedback;
	my $Result;
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Get the suggested file name...
	$SuggestedFileName = $TimeResource->FormatTime($StringResource->_S('BACKUP_HOME_TIME_FORMAT'), localtime());

	# Set up the configuration parameters.
	$Parameters{'caller'}			= 'BackupDomain';
	$Parameters{'q'}				= $cgi;
	$Parameters{'backup_medium'}	= 'web';
	$Parameters{'backup_filename'}	= $SuggestedFileName; 

	# Get the domain and level of the user.
	($Parameters{'domain'}, $Level) = Authorize::getTargetDomain();

	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }

	# If applying a change...
	if (defined($cgi->param('start_backup')))
	{
		# If there is file selection data...
		if (('entire_domain' eq $cgi->param('system_to_backup')) ||
			(('partial_system' eq $cgi->param('system_to_backup')) &&
			 (defined($cgi->param('files_to_backup')))))
		{
			# Trim the whitespace.
			$Parameters{'backup_filename'} =~ s/^\s*(.*?)\s*$/$1/;
			
			# Validate the file name...
			if ($Parameters{'backup_filename'} =~ /^[\w|_|-|\.]+$/)
			{
				# Start the backup sequence.
				Backup->initialize_backup_sequence(\%Parameters); 
			}
			else
			{
				$Result = 670001;
			}
		}
		else
		{
			$Result = 670000;
		}
	} 
	elsif (defined($cgi->param('restore')))
	{
		# Start the restore process.
		Backup->restore_from_web(\%Parameters);
		undef($DisplayDataRef);
		return 0;
	}
	elsif (defined($cgi->param('view_file_list')))
	{
		# If there is file selection data...
		if (('entire_domain' eq $cgi->param('system_to_backup')) ||
			(('partial_system' eq $cgi->param('system_to_backup')) &&
			 (defined($cgi->param('files_to_backup')))))
		{
			# Redirect to view the file list.
			return 'view_file_list';
		}
		else
		{
			$Result = 670000;
		}
	}
	
	# If there is an action...
	if ((defined($cgi->param('start_backup'))) ||
		(defined($cgi->param('view_file_list'))))
	{
		my $FileSet;
		
		# Set up the radio buttons and checkboxes.
		$FormData{'entire_domain'}			= ('entire_domain' eq $cgi->param('system_to_backup')) ? 'checked' : '';
		$FormData{'partial_system'} 		= ('partial_system' eq $cgi->param('system_to_backup')) ? 'checked' : '';
		$FormData{'domain_web_site'}		= '';
		$FormData{'domain_email'}			= '';
		$FormData{'domain_user_home_dirs'}	= '';

		# Run through each of the files...
		foreach $FileSet (@FilesToBackup)
		{
			SWITCH:
			{
				# Add the file path into the file source list...
				if ('domain_web_site' eq $FileSet)			{ $FormData{'domain_web_site'} = 'checked'; last SWITCH; }
				if ('domain_email' eq $FileSet)				{ $FormData{'domain_email'} = 'checked'; last SWITCH; }
				if ('domain_user_home_dirs' eq $FileSet)	{ $FormData{'domain_user_home_dirs'} = 'checked'; last SWITCH; }
			}
		}
	}
	
	# Get the data sizes.
	Backup->calculate_disk_usage(\%Parameters);

	# If there was an error...
	if ($Result)
	{
		# Get the error string.
		$Feedback = $StringResource->_S($Result);
	}

	$html = <<HERE;
<script src=/utilities.js language=javascript></script>
<script language=javascript>
function selectCorrespondingRadio()
{
	if ((document.backup.files_to_backup[0].checked) || 
		(document.backup.files_to_backup[1].checked) || 
		(document.backup.files_to_backup[2].checked))
	{
		document.backup.system_to_backup[1].checked = "checked";
	}
}

function deselectCheckboxes()
{
	document.backup.files_to_backup[0].checked = '';
	document.backup.files_to_backup[1].checked = '';
	document.backup.files_to_backup[2].checked = '';
}

function validateRestorePath()
{
	if ((isEmpty(document.restore.upload.value)) || 
		(isWhitespace(document.restore.upload.value)))
	{
		return warnEmpty(document.restore.upload, '$_R{'ERROR_EMPTY_PATH'}');
	}
}
</script>
<form method=post name=backup action=domain_backup.cgi enctype=multipart/form-data>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td width=359 valign=top align=left colspan=4><b>$_R{'BACKUP'}</b></td>
			</tr>
			<tr>
				<td width=359 colspan=4>$_R{'CHOOSE_FILES_TO_BACKUP'}</td>
			</tr>
			<tr>
				<td width=1><input type=radio name=system_to_backup value=entire_domain onClick='deselectCheckboxes()' $FormData{'entire_domain'}></td>
				<td width=228 colspan=2><b>$_R{'ENTIRE_DOMAIN'}</b></td>
				<td width=96>($Parameters{'domain_usage'})</td>
			</tr>
			<tr>
				<td width=1><input type=radio name=system_to_backup value=partial_system $FormData{'partial_system'}></td>
				<td width=228 colspan=2><b>$_R{'PARTIAL_DOMAIN'}</b></td>
				<td width=96></td>
			</tr>
			<tr>
				<td width=1></td>
				<td width=1><input type=checkbox name=files_to_backup value=domain_web_site onClick='selectCorrespondingRadio()' $FormData{'domain_web_site'}></td>
				<td width=268><b>$_R{'WEB_SITE'}</b></td>
				<td width=96>($Parameters{'domain_web_usage'})</td>
			</tr>
			<tr>
				<td width=1></td>
				<td width=1><input type=checkbox name=files_to_backup value=domain_email onClick='selectCorrespondingRadio()' $FormData{'domain_email'}></td>
				<td width=268><b>$_R{'DOMAIN_E_MAIL'}</b></td>
				<td width=96>($Parameters{'domain_mail_usage'})</td>
			</tr>
			<tr>
				<td width=1></td>
				<td width=1><input type=checkbox name=files_to_backup value=domain_user_home_dirs onClick='selectCorrespondingRadio()' $FormData{'domain_user_home_dirs'}></td>
				<td width=268><b>$_R{'DOMAIN_USER_DIRECTORIES'}</b></td>
				<td width=96>($Parameters{'domain_users_usage'})</td>
			</tr>
			<tr>
				<td width=1></td>
				<td width=269 colspan=3>&nbsp;<input type=submit class=clsButton value='$_R{'VIEW_FILE_LISTING'}' name=view_file_list></td>
			</tr>
			<tr>
				<td width=359 colspan=4>&nbsp;</td>
			</tr>
			<tr>
				<td colspan=4 align=right width=100%>
					<input type=submit class=clsButton value='$_R{'START_BACKUP'}' name=start_backup>
				</td>
            </tr>
			</table>
		</td>
	</tr>
	</table>
</form>
<form method=post name=restore action=domain_backup.cgi enctype=multipart/form-data>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td width=353 colspan=2><b>$_R{'RESTORE'}</b></td>
			</tr>
			<tr>
				<td width=353 colspan=2>$_R{'SPECIFY_RESTORE_LOCATION'}</td>
			</tr>
			<tr>
				<td width=353 colspan=2>&nbsp;<b>$_R{'LOCAL_FILE_PATH'}<br>&nbsp;<input type=file name=upload size=27 maxlength=80 class=clsButtonfile></td>
			</tr>
			<tr>
				<td width=353 colspan=2 align=right>&nbsp;</td>
			</tr>
			<tr>
			<td width=100% colspan=2 align=right>
				<input type=submit class=clsButton value='$_R{'RESTORE'}' name=restore onClick='return validateRestorePath()'></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE

	# Overriding result...
	$Result		= $DisplayDataRef->{'result'} unless(defined($Result));
	$Feedback	= $DisplayDataRef->{'feedback'} unless(defined($Feedback));

	# Fill out the display information.
	$DisplayDataRef->{'q'}			= $cgi;
	$DisplayDataRef->{'html'}		= $html;
	$DisplayDataRef->{'class'}		= 'domain_backup';
	$DisplayDataRef->{'id'}			= 6700;
	$DisplayDataRef->{'title'}		= $_R{'IDT:6700'};
	$DisplayDataRef->{'root-link'}	= 'domain_backup';
	$DisplayDataRef->{'result'}		= $Result;
	$DisplayDataRef->{'feedback'}	= $Feedback;

	return 0;
}

###############################################################################
# RenderViewFileList
###############################################################################

sub RenderViewFileList
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_backup.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:6700'				=> '',
								'IDT:6800'				=> '' );
	my $TargetSystem		= $cgi->param('system_to_backup');
	my $Domain;
	my $Level;
	my @FileSources;
	my $UserMail			= 0;
	my $File;
	my $Return;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Get the domain and level of the user.
	($Domain, $Level) = Authorize::getTargetDomain();

	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }

	# Do not use the conventional display.
	undef($DisplayDataRef);

	# Check the backup group...
	if ('partial_system' eq $TargetSystem)
	{
		my @FilesToBackup = $cgi->param('files_to_backup');
		
		# Run through each of the files...
		foreach $File (@FilesToBackup)
		{
			SWITCH:
			{
				# Add the file path into the file source list...
				if ('domain_email' eq $File) { $UserMail = 1; last SWITCH; }
				if ('domain_web_site' eq $File) { push(@FileSources, 'web'); last SWITCH; }
				if ('domain_user_home_dirs' eq $File) { push(@FileSources, 'users'); last SWITCH; }
			}
		}
	}
	else
	{
		# Show root.
		push(@FileSources, "/home/domain/$Domain/")
	}

	print <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
	<head>
		<link rel=stylesheet href=/style/style.css>
	</head>
	<script>
		selected_item = "domain_backup";
		parent.navbar.selectitem("domain_backup");
	</script>
<body bgcolor=white marginheight=2 marginwidth=2 leftmargin=0 topmargin=2>
	<table align=left cellspacing=0 cellpadding=0 border=0 bgcolor=white width=100%>
	<tr align=left valign=top>
		<td align=left colspan=3>
			<b><font face=arial size=1><u><a href=domain_backup.cgi>&lt;&nbsp;$_R{'IDT:6700'}</a></u>&nbsp;</font></b>
			<p class=clsTitlePage>$_R{'IDT:6800'}</p>
			<tr align=left valign=top>
				<td valign=top>
				<pre><hr noshade><br>
HERE
		
	# If there are any files...
	if (scalar(@FileSources)) 
	{
		# Get the file lists.
		my $File	= Safe::pipe_read("(cd /home/domain/$Domain; /usr/bin/find @FileSources -printf \"%s\\t%TD %TT\\t%p\\n\")");
		my $Buffer;

		# Read in the results...
		while (sysread $File, $Buffer, 1024) 
		{
			# Output the line.
			syswrite STDOUT, $Buffer;
		}
		
		# Close the pipe.
		Safe::pipe_close($File);
	}
	
	# If listing user mail too...
	if ($UserMail)
	{
		my $MailBoxPathsRef			= Backup::get_user_home_dirs('domain', $Domain);
		my $FileModificationTime;
		my $FileSize;

		# Run through the list of paths...
		foreach (@$MailBoxPathsRef) 
		{
			# Continue if the time and size of the file is not available...
			next unless (($FileSize, $FileModificationTime) = (stat("/home/domain/$Domain/$_/mbox"))[7,9]);

			# Format the output data.
			$FileModificationTime	= strftime("%D %T", localtime($FileModificationTime));
			$FileSize				= "0" unless ($FileSize);
			
			# Output the line.
			print STDOUT "$FileSize\t$FileModificationTime\t$_/mbox\n";
		}
	}
	
	print <<HERE;
				<hr noshade>
				</td>
			</tr>
		</td>
	</tr>
	</table>
</body>
</html>
HERE

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi			= shift;
	my %DisplayData;
	my $FormType	= $cgi->param('type');
	my $Redirect	= 0;

	# Safe default.
	$FormType = 'default' unless(defined($FormType));
	
	SWITCH:
	{
		if ('default'          eq $FormType) { $FormType = RenderDefault($cgi, \%DisplayData);						last SWITCH; }
		if ('view_file_list'   eq $FormType) { $FormType = RenderViewFileList($cgi, \%DisplayData, $Redirect);		last SWITCH; }
		$FormType = 0;
	}

	# If there is still some form data...
	if ($FormType)
	{
		# Set the redirect.
		$Redirect = 1;
		goto SWITCH;
	}
	elsif (defined(%DisplayData))
	{
		# Display the page.
		DisplayMain->printDocument(\%DisplayData);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
