#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: firstboot2.cgi                                                    #
#                                                                             #
# Description: CGI needed to display the first boot system setup page.        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use CookieData;
use Chronos;
use Zone;
use CGI;

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
		else
		{
			# Set an empty value.
			$FormDataRef->{$Key} = '';
		}
	}
	
	# Validate the data...
	if ($FormDataRef->{'name_host'} !~ /^(\w|-|_)+$/)
	{
		# Set the return code.
		$Return = 1030002;
	}
	elsif ($FormDataRef->{'name_domain'} !~ /^(\w|-|\.|_)+$/)
	{
		# Set the return code.
		$Return = 1030003;
	}
	elsif (($FormDataRef->{'dns_server_address_primary'}) &&
		   (($FormDataRef->{'dns_server_address_primary'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ||
		    (1 > $1) || (255 < $1) || (0 > $2) || (255 < $2) ||
		    (0 > $3) || (255 < $3) || (0 > $4) || (255 < $4)))
	{
		# Set the return code.
		$Return = 1030004;
	}
	elsif (($FormDataRef->{'dns_server_address_primary'}) &&
		   ($FormDataRef->{'dns_server_address_primary'} eq $FormDataRef->{'dns_server_address_secondary'}))
	{
		# Set the return code.
		$Return = 1030005;
	}
	elsif (($FormDataRef->{'dns_server_address_secondary'}) &&
		   (($FormDataRef->{'dns_server_address_secondary'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ||
		    (1 > $1) || (255 < $1) || (0 > $2) || (255 < $2) ||
		    (0 > $3) || (255 < $3) || (0 > $4) || (255 < $4)))
	{
		# Set the return code.
		$Return = 1030006;
	}
	elsif (($FormDataRef->{'chronos_ntp_active'}) &&
		   (!(($FormDataRef->{'chronos_ntp_server'} =~ /^(\w|-|\.|_)+$/) || 
		      (!(($FormDataRef->{'chronos_ntp_server'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ||
		         (1 > $1) || (255 < $1) || (0 > $2) || (255 < $2) ||
		         (0 > $3) || (255 < $3) || (0 > $4) || (255 < $4))))))
	{
		# Set the return code.
		$Return = 1030007;
	}
	elsif (!$FormDataRef->{'chronos_ntp_active'})
	{
		# If the time format is preserved...
		if ($FormDataRef->{'chronos_time'} =~ /^\d{1,2}:\d{1,2}:\d{1,2}$/)
		{
			my ($Hour, $Minute, $Second) = split(/:/, $FormDataRef->{'chronos_time'});
	
			# Validate the fields.
			if ((24 > $Hour) &&
				(60 > $Minute) &&
				(60 > $Second))
			{
				my %MonthDayHash	= ( '01' => 31,
										'02' => 29, 
										'03' => 31,
										'04' => 30,
										'05' => 31,
										'06' => 30,
										'07' => 31,
										'08' => 31,
										'09' => 30,
										'10' => 31,
										'11' => 30,
										'12' => 31 );
	
				# Check for a valid date other than February.
				if ($FormDataRef->{'chronos_date'} <= $MonthDayHash{$FormDataRef->{'chronos_month'}})
				{
					# Validate for February.
					if ((2 == $FormDataRef->{'chronos_month'}) &&
						($FormDataRef->{'chronos_date'} > (((0 == ($FormDataRef->{'chronos_year'} % 4)) && ((!(0 == ($FormDataRef->{'chronos_year'} % 100))) || ($FormDataRef->{'chronos_year'} % 400 == 0))) ? 29 : 28)))
					{
						# Set the return.
						$Return = 1030001;
					}
				}
				else
				{
					# Set the return.
					$Return = 1030001;
				}
			}
			else
			{
				# Set the return.
				$Return = 1030000;
			}
		}
		else
		{
			# Set the return.
			$Return = 1030000;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $TimeData			= new Chronos($PreferredLanguage, '/usr/ins/intel/gui/locale');
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'firstboot2.lrc', 'server_date.lrc', 'common.lrc' ]);
	my $Cookie				= new CookieData(0);
	my %_R					= ( 'IDT:10300'						=> '',
								'INTRODUCTION'					=> '',
								'STEP_1_OF_3'					=> '',
								'HOSTNAME'						=> '',
								'HOSTNAME_EXAMPLE'				=> '',
								'DOMAINNAME'					=> '',
								'DOMAINNAME_EXAMPLE'			=> '',
								'DNS_SERVER_ADDRESS_PRIMARY'	=> '',
								'DNS_SERVER_ADDRESS_SECONDARY'	=> '',
								'TIME'							=> '',
								'DATE'							=> '',
								'TIME_ZONE'						=> '',
								'NETWORK_TIME_PROTOCOL'			=> '',
								'SYNCHRONIZE_WITH_NTP'			=> '',
								'NTP_SERVER_ADDRESS'			=> '',
								'JAN'							=> '',
								'FEB'							=> '',
								'MAR'							=> '',
								'APR'							=> '',
								'MAY'							=> '',
								'JUN'							=> '',
								'JUL'							=> '',
								'AUG'							=> '',
								'SEP'							=> '',
								'OCT'							=> '',
								'NOV'							=> '',
								'DEC'							=> '',
								'BACK'							=> '',
								'NEXT'							=> '',
								'REQUIRED_FIELD_NOTE'			=> '' );
	my %FormData			= ( 'name_host'						=> '',
								'name_domain'					=> '',
								'dns_server_address_primary'	=> '',
								'dns_server_address_secondary'	=> '',
								'chronos_time'					=> '',
								'chronos_month'					=> '',
								'chronos_date'					=> '',
								'chronos_year'					=> '',
								'chronos_timezone'				=> '',
								'chronos_ntp_active'			=> '',
								'chronos_ntp_server'			=> '' );
	my @TimeZones			= Zone::tzone;
	my %NumberMonthHash;
	my %CurrentTime;
	my %Arrow;
	my $CookieFile			= '/usr/ins/intel/gui/cookies/firstboot';
	my $Selection;
	my $SelectionMonth;
	my $SelectionDate;
	my $SelectionYear;
	my $SelectionTimeZone;
	my $Item;
	my $Key;
	my $Result;
	my $Feedback;
	my $htmlTitle;
	my $html;
	
	# If a button is pressed...
	if (defined($cgi->param('back')))
	{
		# Redirect to the previous form.
		print $cgi->redirect( -url => 'firstboot_license.cgi' );
		return;
	}
	elsif (defined($cgi->param('next')))
	{
		# Get the form data from the form...
		foreach $Key (keys(%FormData))
		{
			# Get the data...
			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
		}

		# Validate the form data...
		if (0 == ($Result = ValidateFormData(\%FormData)))
		{
			# Attach to the cookie file.
			$Cookie->Attach($CookieFile);

			# Run through all of the keys...
			foreach $Key (keys(%FormData))
			{
				# Save the data.
				$Cookie->SetValueKeyed($Key, $FormData{$Key});
			}

			# Detach from the cookie.
			$Cookie->Detach();
			
			# Redirect to the next form.
			print $cgi->redirect( -url => 'firstboot3.cgi' );
			return;
		}
		else
		{
			# Get the error code.
			$Feedback = $StringResource->_S($Result);
		}
	}
	elsif (defined($cgi->param('fresh')))
	{
		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);

		# Delete the file.
		$Cookie->Delete();
	}
	elsif ( -e $CookieFile )
	{
		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);

		# Run through all of the keys...
		foreach $Key (keys(%FormData))
		{
			# Fetch the data.
			$FormData{$Key} = $Cookie->GetValueKeyed($Key);
		}

		# Detach from the cookie.
		$Cookie->Detach();
	}

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Based on the user agent...
	if (-1 < (index($cgi->user_agent(), 'MSIE')))
	{
		# Set the Microsoft arrows.
		$Arrow{'left'}	=  '&larr;';
		$Arrow{'right'}	=  '&rarr;';
	}
	else
	{
		# Set the other arrows.
		$Arrow{'left'}	=  '<-';
		$Arrow{'right'}	=  '->';
	}

	# Fill in the hash.
	%NumberMonthHash				= ( '01' => $_R{'JAN'}, 
									    '02' => $_R{'FEB'},
									    '03' => $_R{'MAR'},
									    '04' => $_R{'APR'},
									    '05' => $_R{'MAY'},
									    '06' => $_R{'JUN'},
									    '07' => $_R{'JUL'},
									    '08' => $_R{'AUG'},
									    '09' => $_R{'SEP'},
									    '10' => $_R{'OCT'},
									    '11' => $_R{'NOV'},
									    '12' => $_R{'DEC'} );

	# Get the current time.
	$TimeData->GetTime(\%CurrentTime);
    
	# Default values.
	$FormData{'chronos_time'}		= sprintf("%02d:%02d:%02d", $CurrentTime{'hour'}, $CurrentTime{'minute'}, $CurrentTime{'second'}) unless('' ne $FormData{'chronos_time'});
	$FormData{'chronos_month'}		= sprintf("%02d", $CurrentTime{'month'}) unless('' ne $FormData{'chronos_month'});
	$FormData{'chronos_date'}		= $CurrentTime{'date'} unless('' ne $FormData{'chronos_date'});
	$FormData{'chronos_year'}		= $CurrentTime{'year'} unless('' ne $FormData{'chronos_year'});
	$FormData{'chronos_timezone'}	= $TimeData->GetTimeZone() unless('' ne $FormData{'chronos_timezone'});

	# Run through all of the months...
	foreach $Item (sort(keys(%NumberMonthHash)))
	{
		$Selection		 = ($Item eq $FormData{'chronos_month'}) ? 'selected' : '';
		$SelectionMonth	.= "<option value=$Item $Selection>$NumberMonthHash{$Item}\n";
	} 

	# Run through all of the dates...
	foreach $Item (1..31) 
	{
		$Selection		 = ($Item eq $FormData{'chronos_date'}) ? 'selected' : '';
		$SelectionDate	.= "<option value=$Item $Selection>$Item\n"; 
	}

	# Run through all of the years...
	foreach $Item (1970 .. 2037) 
	{
		$Selection		 = ($Item eq $FormData{'chronos_year'}) ? 'selected' : '';
		$SelectionYear	.= "<option value=$Item $Selection>$Item\n"; 
	}

	# Run through all of the timezones...	
    foreach $Item (@TimeZones) 
	{
		$Selection			 = ($Item eq $FormData{'chronos_timezone'}) ? 'selected' : '';
		$SelectionTimeZone	.= "<option value=$Item $Selection>$Item\n";
	}

	$htmlTitle = <<HERE;
	<table width=548 border=0 cellspacing=0 cellpadding=0>
	<tr>
		<td align=left><p class=clstitlepage><b><img align=absmiddle border=0 width=24 src=/images/1.gif height=22>&nbsp;$_R{'IDT:10300'}</b></p></td>
		<td align=right><b>$_R{'STEP_1_OF_3'}</b></td>
	</tr>
	</table>
HERE

	$html = <<HERE;
<table border=0 width=550 bgcolor=white>
<tr>
	<td>$_R{'INTRODUCTION'}</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td bgcolor=#eeeeee>
		<table border=0 width=100% bgcolor=#eeeeee>
		<tr>
			<td>&nbsp;
				<table cellspacing=1 border=0 width=100% bgcolor=#eeeeee>
				<form method=post action=firstboot2.cgi enctype=application/x-www-form-urlencoded>
				<tr>
					<td width=1% valign=top><font size=3 color=red><sup>*</sup></font></td>
					<td width=40% nowrap><b>$_R{'HOSTNAME'}</b><br><nobr>&nbsp;$_R{'HOSTNAME_EXAMPLE'}</nobr></td>
					<td width=60%><input type=text name=name_host size=25 maxlength=32 value='$FormData{'name_host'}'></td>
				</tr>
				<tr>
					<td width=1% valign=top><font size=3 color=red><sup>*</sup></font></td>
					<td width=40% nowrap><b>$_R{'DOMAINNAME'}</b><br><nobr>&nbsp;$_R{'DOMAINNAME_EXAMPLE'}</nobr></td>
					<td width=60%><input type=text name=name_domain size=25 maxlength=32 value='$FormData{'name_domain'}'></td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td width=40% nowrap><b>$_R{'DNS_SERVER_ADDRESS_PRIMARY'}</b></td>
					<td width=60%><input type=text name=dns_server_address_primary size=25 maxlength=15 value='$FormData{'dns_server_address_primary'}'></td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td width=40% nowrap><b>$_R{'DNS_SERVER_ADDRESS_SECONDARY'}</b></td>
					<td width=60%><input type=text name=dns_server_address_secondary size=25 maxlength=15 value='$FormData{'dns_server_address_secondary'}'></td>
				</tr>
				<tr>
					<td colspan=3>&nbsp;</td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td width=40%><b>$_R{'TIME'}</b></td>
					<td width=60%><input type=text name=chronos_time size=11 maxlength=8 value='$FormData{'chronos_time'}'></td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td width=40%><b>$_R{'DATE'}</b></td>
					<td width=60%>
						<select size=1 name=chronos_month>$SelectionMonth</select>
						<select size=1 name=chronos_date>$SelectionDate</select>
						<select size=1 name=chronos_year>$SelectionYear</select>
					</td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td width=40%><b>$_R{'TIME_ZONE'}</b></td>
					<td width=60%><select size=1 name=chronos_timezone>$SelectionTimeZone</select></td>
				</tr>
				<tr>
					<td colspan=3>&nbsp;</td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td colspan=2><b>$_R{'NETWORK_TIME_PROTOCOL'}</b></td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td width=40%><b>$_R{'NTP_SERVER_ADDRESS'}</b></td>
					<td width=60%><input type=text name=chronos_ntp_server size=25 maxlength=32 value='$FormData{'chronos_ntp_server'}'></td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td width=100% colspan=2 align=left>
						<input type=checkbox name=chronos_ntp_active value=checked $FormData{'chronos_ntp_active'}>&nbsp;<font color=black><b>$_R{'SYNCHRONIZE_WITH_NTP'}</b></font>
					</td>
				</tr>
				<tr>
					<td colspan=3>&nbsp;</td>
				</tr>
				<tr>
					<td nowrap colspan=3 align=right width=532>
						<input type=submit class=clsButton value='$Arrow{'left'}&nbsp;$_R{'BACK'}' name=back>
						<input type=submit class=clsButton value='$_R{'NEXT'}&nbsp;$Arrow{'right'}' name=next>
					</td>
				</tr>
				</form>
				</table>
			</td>
		</tr>
		</table>
	</td>
</tr>
<tr>
	<td>$_R{'REQUIRED_FIELD_NOTE'}</td>
</tr>
</table>
HERE

	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'title-html' => $htmlTitle, 'margin-left' => 3, 
								 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
