#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_disk.cgi                                                   #
#                                                                             #
# Description: Provides status information and configuration support for      #
#              the appliance disk monitoring.                                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageTime;
use LanguageResource;
use POSIX qw(strftime);
use IO::File;
use DisplayMain;
use Backup;
use Safe;
use CGI;
use ProductInformation;


###############################################################################
# ConfigFileRead
###############################################################################

sub ConfigFileRead
{
	my ($ConfigFileName) = @_;
	my $ConfigFile = new IO::File("< $ConfigFileName");
	my @ConfigFileContents;

	# If the file is not open...
	unless ($ConfigFile)
	{
		exit(0);
	}

	# Snag the contents.
	@ConfigFileContents = <$ConfigFile>;
	
	# Close the file.
	$ConfigFile->close;

	return @ConfigFileContents;
}

###############################################################################
# ConfigFileWrite
###############################################################################

sub ConfigFileWrite
{
	my ($ConfigFileName, @ConfigFileContents) = @_;
    my $ConfigFile = new IO::File("> $ConfigFileName");

	# If the file is not open...
	unless ($ConfigFile)
	{
		exit(0);
	}
	else
	{
		print $ConfigFile @ConfigFileContents;
	}
	
	# Close the file.
	$ConfigFile->close;

	return 1;
}

###############################################################################
# GetConfigData
###############################################################################

sub GetConfigData
{
	my ($Label, @ConfigFileContents) = @_;
    my $Value;
	my $Line;

	# Loop through all of the lines in the file.
	foreach $Line (@ConfigFileContents)
	{
		# Continue if there is a comment.
		next if $Line =~ /^#/;

		# Continue if the label is not found.
		next unless $Line =~ /$Label/;
		
		# Parse out the value.
		$Value = $Line;
		$Value =~ s/.*$Label=(.*)$/$1/;
		chomp $Value;
		
		return $Value;
	}

	return '';
}

###############################################################################
# GetConfigDataMulti
###############################################################################

sub GetConfigDataMulti
{
	my ($Label, $Tag, @ConfigFileContents) = @_;
    my @Values;
	my $Data;
	my $Line;
	my ($Device, $Type);

	$Values[0] = "";
	$Values[1] = "";

	# Loop through all of the lines in the file.
	foreach $Line (@ConfigFileContents)
	{
		# Continue if there is a comment.
		next if $Line =~ /^#/;

		# Continue if the label is not found.
		next unless ($Line =~ /$Label/) and ($Line =~ /$Tag/);
		
		# Parse out the value.
		$Data = $Line;
		$Data =~ s/.*$Label=(.*)$/$1/;
		chomp $Data;
		
		# Get the data.
		($Device, $Tag, $Type, $Values[0], $Values[1]) = split(/:/, $Data);

		# Exit the loop.
		last;
	}

	return @Values;
}

###############################################################################
# SetConfigData
###############################################################################

sub SetConfigData
{
	my ($Label, $Value, $ConfigFileContents) = @_;
	my $Line;

	# Loop through the file and rewrite it to disk.
    foreach $Line (@$ConfigFileContents)
	{
		# Check to see if the line can be rerendered to file...
		unless ($Line !~ /$Label/ or $Line =~ /^#/)
		{
			# Set the data.
			$Line =~ s/$Label=(.*)/$Label=$Value/s;
		}
	}
	
	return 0;
}

###############################################################################
# SetConfigDataMutli
###############################################################################

sub SetConfigDataMutli
{
	my ($Label, $Tag, $Value1, $Value2, $ConfigFileContents) = @_;
	my $Line;
	
	# Loop through the file and rewrite it to disk.
    foreach $Line (@$ConfigFileContents)
	{
		# Check to see if the line can be rerendered to file...
		unless ($Line !~ /$Label/ or $Line =~ /^#/)
		{
			# If the tag exists...
			if ($Line =~ /:$Tag:/)
			{
				# Set the data.
                $Line =~ s/^.*$Label=(.*):$Tag:(.*):\S*:\S*$/$Label=$1:$Tag:$2:$Value1:$Value2/;

				# Bail out.
				last;
			}
		}
	}
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my ($html1, $html2, $html3, $html4, $html5, $html6);
	my ($ViewMode, $ViewLabel, $ViewLink, $ViewHidden);
	my ($DiskPercentUsage, $DiskUsage, $DiskLabel, $DiskDevice, $DiskSize, $DiskUsed, $DiskFree);
	my ($ObjectNameWarning, $ObjectNameCritical);
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_disk.lrc', 'server_status.lrc', 'server_alerts.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:900'				=> '',
								'DRIVE_SPACE'			=> '',
								'ALERT_THRESHOLDS'		=> '',
								'VIEW_PERCENT'			=> '',
								'VIEW_MB'				=> '',
								'FREE'					=> '',
								'USED'					=> '',
								'TOTAL'					=> '',
								'WARNING_LABEL'			=> '',
								'CRITICAL_LABEL'		=> '',
								'MB'					=> '',
								'GB'					=> '',
								'ADD_DRIVE'				=> '',
								'SECOND_DRIVE_STATUS'	=> '',
								'IDT:600'				=> '',
								'APPLY'					=> '',
								'CONFIGURE_ALERTS'		=> '' );
	my $UpdateConfiguration = 0;
	my $ConfigurationASM	= '/etc/asm/asm.conf';
	my $ConfigurationDC		= '/usr/ins/intel/bin/dc.conf';
	my $DriveStatusLabel	= '';
	my $DriveStatusText		= '';
	my $DiskInformation 	= Safe::backtick('/bin/df -h');
	my $TextAlignment		= '',
	my @ConfigFileContents;
	my $SecondDriveStatus;
	my $DiskSizeMB;
	my $Leftover;
	my @Values;
	my $Label;
	my $i					= 0;
	my $Feedback;
	my $Result;
	my $ProductInformation	= new ProductInformation();

	# Get the string resources.

	$StringResource->GetStrings(\%_R);
    
	# Set the edit control text alignment.
	$TextAlignment = "style='text-align: right'" if($ENV{'HTTP_USER_AGENT'} =~ /MSIE/);
	
	# Set the viewmode.
    if ('percent' eq $cgi->param('view_mode'))
	{
		$ViewMode	= 2;
		$ViewLabel	= '%&nbsp;&nbsp;';
		$ViewLink	= "<a href=server_disk.cgi?view_mode=size>$_R{'VIEW_MB'}</a>";
		$ViewHidden	= "percent";
	}
	else
	{
		$ViewMode	= 1;
		$ViewLabel	= $_R{'MB'};
		$ViewLink	= "<a href=server_disk.cgi?view_mode=percent>$_R{'VIEW_PERCENT'}</a>";
		$ViewHidden = "size";
	}

	$html1 = <<HERE;
<form method=post action=server_disk.cgi enctype="application/x-www-form-urlencoded">
	<input type=hidden name=view_mode value=$ViewHidden>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td colspan=5 width=100%><b>$_R{'DRIVE_SPACE'}</b></td>
			</tr>
			<tr>
				<td colspan=3 width=221></td>
				<td width=161 colspan=2><b>$_R{'ALERT_THRESHOLDS'}</b></td>
			</tr>
			<tr>
				<td colspan=3 width=221><b>&nbsp;</b></td>
				<td width=161 colspan=2>$ViewLink</td>
			</tr>
HERE

	$_ = $DiskInformation;
	
	# Read in the configuration file.
	@ConfigFileContents = ConfigFileRead($ConfigurationASM);

	# Run a loop to capture output from df.
	while (/\/dev\/\S+\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)/g) 
	{
		$DiskLabel			= $5;
		$DiskPercentUsage	= $4;
		$DiskSize			= $1;
		$DiskUsed			= $2;
		$DiskFree			= $3;

		# Only show non-mounted partitions...
		unless (($DiskLabel =~ /^\/mnt/) || ($DiskLabel eq '/emergency'))
		{
			#Set the object name.
			$ObjectNameWarning	= "warnlimit" . $i;
			$ObjectNameCritical	= "critlimit" . $i;
			++$i;

			# Cache the values.
			$Values[0] = $cgi->param($ObjectNameWarning);
			$Values[1] = $cgi->param($ObjectNameCritical);

			# Patch up the label.
			($DiskSize, $Label) = split(/(G|M)/, $DiskSize);

			# Normalize the disk size to MB.
			$DiskSizeMB = ($Label eq 'G') ? $DiskSize * 1024 : $DiskSize;

			# Add the label back on.
			$DiskSize .= ($Label eq 'G') ? " $_R{'GB'}" : " $_R{'MB'}";

			# If there is data to set.
			if (($cgi->param('apply')) &&
				(($Values[0]) or
				 ($Values[1])))
			{
				# If there is not an error.
				if (!$Feedback)
				{
					# Get the data length..
					my $ValueW;
					my $ValueC;

					# Make sure that the data is valid.
					unless ((($Values[0] !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
					         (!($1 || ('0' eq $1) || $2 || ('0' eq $2)))) ||
					        (($Values[1] !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
							 (!($1 || ('0' eq $1) || $2 || ('0' eq $2)))))
					{
						# Get the normalized value.
						$ValueW = (2 == $ViewMode) ? sprintf("%.3f", ($DiskSizeMB * ($Values[0] / 100))) : $Values[0];
						$ValueC = (2 == $ViewMode) ? sprintf("%.3f", ($DiskSizeMB * ($Values[1] / 100))) : $Values[1];

						# Check the sizes...
						if (($ValueW > $DiskSizeMB) or
							($ValueC > $DiskSizeMB))
						{
							$Result					= 90001;
							$UpdateConfiguration	= 0;
						}
						elsif ($ValueW <= $ValueC)
						{
							$Result					= 90003;
							$UpdateConfiguration	= 0;
						}
						else
						{
							$ValueW *= 1024;
							$ValueC *= 1024;

							SetConfigDataMutli('DISKS', $DiskLabel, $ValueW, $ValueC, \@ConfigFileContents);
							$UpdateConfiguration = 1;
						}
					}
					else
					{
						$Result					= 90002;
						$UpdateConfiguration 	= 0;
					}

					# If there was an error.
					if (0 == $UpdateConfiguration)
					{
						$Feedback = $StringResource->_S($Result) . " $DiskLabel";
					}
				}
			}
			else
			{
				# Fetch the thresholds.
				@Values = GetConfigDataMulti('DISKS', $DiskLabel, @ConfigFileContents);

				# Convert to MB.
				$Values[0] = sprintf("%.1f", ($Values[0] / 1024));
				$Values[1] = sprintf("%.1f", ($Values[1] / 1024));

				# Check to see if we are dealing with percentages...
				if (2 == $ViewMode)
				{
					# Get the disk percentage value.
					$Values[0] = sprintf("%.1f", (($Values[0] / $DiskSizeMB) * 100));
					$Values[1] = sprintf("%.1f", (($Values[1] / $DiskSizeMB) * 100));
				}

				# Check for a trailing zero...
				if ($Values[0] =~ /^\d*?\.(0)*$/)
				{
					# Remove the trailing zero.
					$Values[0] = sprintf("%d", $Values[0]);
				}

				# Check for a trailing zero...
				if ($Values[1] =~ /^\d*?\.(0)*$/)
				{
					# Remove the trailing zero.
					$Values[1] = sprintf("%d", $Values[1]);
				}
			}

			# Patch up the label.
			($DiskUsed, $Label) = split(/(G|M|k)/, $DiskUsed);

			# Check for a really small amount.
			if ($Label eq 'k')
			{
				$DiskUsed = "0 $_R{'MB'}";
			}
			else
			{
				$DiskUsed .= ($Label eq 'G') ? " $_R{'GB'}" : " $_R{'MB'}";
			}

			# Patch up the label.
			($DiskFree, $Label) = split(/(G|M|k)/, $DiskFree);

			# Check for a really small amount.
			if ($Label eq 'k')
			{
				$DiskFree = "0 $_R{'MB'}";
			}
			else
			{
				$DiskFree .= ($Label eq 'G') ? " $_R{'GB'}" : " $_R{'MB'}";
			}

			# Get the leftover percentage.
			$Leftover  = $DiskPercentUsage;
			$Leftover =~ s/%//;
			$Leftover  = (100 - $Leftover);
			$Leftover .= "%";

			$html2 .= <<HERE;
			<tr>
				<td width=100% colspan=5><b>$DiskLabel</b></td>
			</tr>
			<tr>
				<td width=115 rowspan=2>
					<table border=1 cellpadding=0 cellspacing=0 width=100>
					<tr>
						<td width=95% bgcolor=#ffffff>
							<table border=0 width=100% cellspacing=0 cellpadding=0>
							<tr>
								<td width=$DiskPercentUsage bgcolor=#008000>&nbsp;</td>
								<td width=$Leftover></td>
							</tr>
							</table>
						</td>
					</tr>
					</table>
				</td>
				<td width=93 colspan=2 rowspan=2 nowrap>
					<b>&nbsp;$_R{'FREE'}</b> $DiskFree<br>
					<b>&nbsp;$_R{'USED'}</b> $DiskUsed<br>
					<b>&nbsp;$_R{'TOTAL'}</b> $DiskSize
				</td>
				<td width=73 nowrap>$_R{'WARNING_LABEL'}&nbsp;</td>
				<td width=54><input type=text name=$ObjectNameWarning size=4 maxlength=6 value="$Values[0]" $TextAlignment>$ViewLabel</td>
			</tr>
			<tr>
				<td width=73 nowrap>$_R{'CRITICAL_LABEL'}&nbsp;</td>
				<td width=54><input type=text name=$ObjectNameCritical size=4 maxlength=6 value="$Values[1]" $TextAlignment>$ViewLabel</td>
			</tr>
HERE
		}
	}

	# See if the configuration needs to be updated...
	if ($UpdateConfiguration)
	{
		# Write out the configuration file.
		ConfigFileWrite($ConfigurationASM, @ConfigFileContents);

		# Reset ERA.
		Safe::backtick("/usr/ins/intel/bin/asm/bakAsm set_era");
		$Result	= 0;
	}
    
    $html3 .= <<HERE;
			<tr>
				<td width=100% colspan=5>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% align=right colspan=5><input class=clsButton name=apply type=submit value="$_R{'APPLY'}"></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
	<br><a href=server_alerts.cgi>$_R{'CONFIGURE_ALERTS'}</a>
</form>
HERE

	# Read in the configuration file.
	@ConfigFileContents = ConfigFileRead($ConfigurationDC);

	# Check for a post...
	if ($cgi->param('set_drive_config'))
	{
		my $SecondDriveStatusNew = GetConfigData('CID_AUXILIARY_DRIVE_STATUS', @ConfigFileContents);
		
		# Get the user's request.
		$SecondDriveStatus = ($cgi->param('aux_drive_config') eq 'mirror') ? 2 : 3;

		# Make sure everything is still good...
		if (($SecondDriveStatus != $SecondDriveStatusNew) &&
			(1 == $SecondDriveStatusNew))
		{
			#Set the new status.
			SetConfigData('CID_AUXILIARY_DRIVE_STATUS', $SecondDriveStatus, \@ConfigFileContents);
			
			# Write out the configuration file.
			ConfigFileWrite($ConfigurationDC, @ConfigFileContents);
	
			# Tell FPIC to remove it's message.
			Safe::backtick('/usr/bin/killall -HUP fpicd');

			# Determine which script to call.
			if ('mirror' eq $cgi->param('aux_drive_config'))
			{
				# Call the mirroring configuration script.
				Safe::backtick('/usr/ins/intel/bin/raidmirror.pl');
			}
			else
			{
				# Call the extension configuration script.
				Safe::backtick('/usr/ins/intel/bin/raidextend.pl');
			}

			$Result = 0;
		}
		else
		{
			# Use the new status.
			$SecondDriveStatus = $SecondDriveStatusNew;

			# Set the feedback.
			$Result		= 90000;
			$Feedback	= $StringResource->_S($Result);
		}
	}
	elsif ($cgi->param('start_backup'))
	{
		my $TimeResource	= new LanguageTime($PreferredLanguage, '/usr/ins/intel/gui/locale');
		my %Parameters		= ( 'q'					=> new CGI,
								'caller'			=> 'BackupServer',
								'backup_medium'		=> $cgi->param('backup_medium'),
								'backup_filename'	=> $TimeResource->FormatTime($StringResource->_S('BACKUP_HOME_TIME_FORMAT'), localtime()) );

		# Initialize backup sequence.
		Backup->initialize_backup_sequence(\%Parameters); 
	}
	else
	{
		$SecondDriveStatus = GetConfigData('CID_AUXILIARY_DRIVE_STATUS', @ConfigFileContents);
	}

	# Safe default.
    $html5 = '';
	
	# Branch based on the drive status...
	if (0 == $SecondDriveStatus)
	{
		my %_RE	=	( 'NOTE'					=> '',
					  'NOT_INSTALLED'			=> '',                       
					  'SECOND_DRIVE_ADD_NOTE'	=> '' );

		# Get the string resources.
		$StringResource->GetStrings(\%_RE);

		# Set the status message.
		$DriveStatusText = $_RE{'NOT_INSTALLED'};
		
		$html5 .= <<HERE;
	<table border=0 cellPadding=0 cellSpacing=1 width=375>
	<tr>
		<td width=100%><b><br>$_RE{'NOTE'}</b>&nbsp;$_RE{'SECOND_DRIVE_ADD_NOTE'}</td>
	</tr>
	</table>
HERE
	}
	elsif (1 == $SecondDriveStatus)
	{
		my %_RE	=	( 'NOTE'						=> '',
					  'WARNING'						=> '',
					  'BACKUP_HOME_PARTITION'		=> '',
					  'INSTALLED_NOT_CONFIGURED'	=> '',
					  'SECOND_DRIVE_DETECTED'		=> '',
					  'SELECT_CONFIGURATION_OPTION'	=> '',
					  'MIRROR_EXISTING_DRIVE'		=> '',
					  'EXTEND_HOME_PARTITION'		=> '',
					  'DRIVE_EXTENSION_WARNING'		=> '',
					  'DRIVE_CONFIGURATION_NOTE'	=> '' );
		
		# Get the string resources.
		$StringResource->GetStrings(\%_RE);
		
		# Set the status message.
		$DriveStatusText = $_RE{'INSTALLED_NOT_CONFIGURED'};
		
		$html5 .= <<HERE;
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td width=100%>
			<table border=0 width=100%>
			<tr>
				<td colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td colspan=2>$_RE{'SECOND_DRIVE_DETECTED'}&nbsp;$_RE{'SELECT_CONFIGURATION_OPTION'}</td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=2><input type=radio name=aux_drive_config value=mirror checked></td>
				<td width=100%><b>$_RE{'MIRROR_EXISTING_DRIVE'}</b></td>
			</tr>
			<tr>
				<td width=2><input type=radio name=aux_drive_config value=extend></td>
				<td width=100%><b>$_RE{'EXTEND_HOME_PARTITION'}</b></td>
			</tr>
			<tr>
				<td width=2>&nbsp;</td>
				<td width=100%><b><font color=red>$_RE{'WARNING'}</font></b>&nbsp;$_RE{'DRIVE_EXTENSION_WARNING'}</td>
			</tr>
			<tr>
				<td width=2>&nbsp;</td>
				<td width=100%>
					<input type=hidden name=system_to_backup value=partial_system checked>
					<input type=hidden name=files_to_backup value=home_paritition checked>
					<input type=hidden name=backup_medium value=web checked>
					<input type=submit name=start_backup value='$_RE{'BACKUP_HOME_PARTITION'}' class=clsButton>
				</td>
			</tr>
			<tr>
				<td colspan=2><br><b>$_RE{'NOTE'}</b>&nbsp;$_RE{'DRIVE_CONFIGURATION_NOTE'}</td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;</td>
			</tr>
			<tr>
HERE

		# Look for a storefront configuration...
		if (2 == $ProductInformation->getClassification())
		{
			# Run this as a separate Perl program...
			eval
			{
				require IS4Control;
			
				my $is4stopped = IS4Control->IS4Stopped();
				my $is4javascript = "javascript: if (!$is4stopped) { alert('" . ${\($StringResource->_S('ECOMM_RUNNING'))} . "'); return false; } else { return true; }";

				# Throw an alert if the Intershop processes are still running. 
				$html5 .= <<HERE;
				<td colspan=2 align=right><input class=clsButton name=set_drive_config type=submit ONCLICK="$is4javascript" value="$_R{'APPLY'}"></td>
HERE
			};
		}
		else
		{
			$html5 .= <<HERE;
			<td colspan=2 align=right><input class=clsButton name=set_drive_config type=submit value="$_R{'APPLY'}"></td>
HERE
		}

		$html5 .= <<HERE;

			</tr>
			</table>
		</td>
	</tr>
	</table>
HERE
	}
	elsif ((2 == $SecondDriveStatus) or 
		   (3 == $SecondDriveStatus))
	{
		my %_RE	=	( 'NOTE'										=> '',
					  'WARNING'										=> '',
					  'FAILED'										=> '',
					  'NORMAL'										=> '',
					  'PARTITION'									=> '',
					  'STATUS'										=> '',
					  'FIRST_DRIVE_STATUS'							=> '',
					  'INSTALLED_AND_CONFIGURED'					=> '',
					  'MIRRORING_CONFIGURATION_IN_PROGRESS'			=> '',
					  'MIRRORING_CONFIGURATION_IN_PROGRESS_WARNING'	=> '',
					  'MIRRORING_IN_PROGRESS'						=> '',
					  'MIRRORING_IN_PROGRESS_FIRST_WARNING'			=> '',
					  'MIRRORING_IN_PROGRESS_SECOND_WARNING'		=> '',
					  'MIRRORING_FAILURE_DETECTED'					=> '',
					  'MIRRORING_FAILURE_DETECTED_FIRST_NOTE'		=> '',
					  'MIRRORING_FAILURE_DETECTED_SECOND_NOTE'		=> '',
					  'MIRRORING_CONFIGURATION_NOTE'				=> '',
					  'EXTENSION_CONFIGURATION_IN_PROGRESS'			=> '',
					  'EXTENSION_CONFIGURATION_IN_PROGRESS_WARNING'	=> '',
					  'EXTENSION_CONFIGURATION_NOTE'				=> '' );
		
		# Get the string resources.
		$StringResource->GetStrings(\%_RE);
		
		# Branch based on the installation type...
		if (2 == $SecondDriveStatus)
		{
			# Check for configuration activity...
			if (Safe::backtick('/bin/ps -ax | grep raidmirror') =~ /raidmirror\.pl/m)
			{
				# The configuration phase is in progress.
				$DriveStatusText = $_RE{'MIRRORING_CONFIGURATION_IN_PROGRESS'};
				
				$html5 .= <<HERE;
	<table border=0 cellpadding=0 cellspacing=1 width=375>
	<tr>
		<td width=100%><b><br><font color=red>$_RE{'WARNING'}</font></b>&nbsp;$_RE{'MIRRORING_CONFIGURATION_IN_PROGRESS_WARNING'}</b></td>
	</tr>
	</table>
HERE
			}
			elsif (open(MDSTAT, '/proc/mdstat'))
			{
				my @MirrorStatus = <MDSTAT>;

				# Make sure that mirroring is set...
				if ($MirrorStatus[0] =~ /raid1/)
				{
					my @FailedPartitions;
					my @NormalPartitions;
					my $RecoveringPartition	= '';
					my $PercentMirrored		= '';
					my $FailedDrive			= 0;
					my $FailedPartition		= 0;
					my $Mapping;
					my $Buffer				= '';
					my $i					= 2;

					# Loop through all of the devices...
					while ($MirrorStatus[$i])
					{
						# Check for a mirrored partition...
						if ($MirrorStatus[$i] =~ /^md/)
						{
							# Check for a recovery status indicator...
							if ($MirrorStatus[$i] =~ /=\d+%/)
							{
								# Recovering detected.
								$Buffer = $PercentMirrored = $MirrorStatus[$i];

								# Get the partition and percent complete.
								$PercentMirrored	=~ s/^.*=(\d+%).*$/$1/;
								$Buffer				=~ s/^(md\d)\s*:.*$/$1/;

								# Take off the newlines.
								chomp($PercentMirrored);

								# Mark this as a failed partition.
								$FailedPartition = 1;
							}
							else
							{
								# Check for a downed partition...
								unless ($MirrorStatus[$i] =~ /\[UU]/)
								{
									# Cache the line and get just the device.
									$Buffer = $MirrorStatus[$i];
									$Buffer =~ s/^(md\d)\s*:.*$/$1/;
									
									# Mark this as a failed partition.
									$FailedPartition = 1;
								}
							}

							# If there is a failed partition...
							if ($Buffer ne '')
							{
								# Set the failed drive.
								$FailedDrive = (($MirrorStatus[$i] =~ /sda|hda/) ? 2 : 1) unless($FailedDrive != 0);
							}
							else
							{
								# Cache the line and get just the device.
								$Buffer = $MirrorStatus[$i];
								$Buffer =~ s/^(md\d)\s*:.*$/$1/;
							}

							# Run through the fstab to make a device/partition match.
							$_ = $DiskInformation;

							# Take off the newline.
							chomp($Buffer);

							# Run a loop to capture output from df.
							while (/\/dev\/(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)/g) 
							{
								# Cache the label and device..
								$DiskLabel	= $6;
								$DiskDevice	= $1;

								# If the mapping is found...
								if ($DiskDevice eq $Buffer)
								{
									# Use the partition name.
									$Buffer = $DiskLabel;
									last;
								}
                            }

                            # If this is the recovering partition...
							if (($PercentMirrored ne '') &&
								($RecoveringPartition eq ''))
							{
								# Cache the label.
								$RecoveringPartition = $Buffer;
							}
							
							# If this is a failed partition...
							if ($FailedPartition)
							{
								# Add this partition to the list.
								push(@FailedPartitions, $Buffer);
							}
							else
							{
								# Add this partition to the list.
								push(@NormalPartitions, $Buffer);
							}
							
							# Clean out the buffers.
							$Buffer	= '';

							# Reset the marker.
							$FailedPartition = 0;
						}

						# Increment the counter.
						++$i;
					}

					# If there are failed partitions...
					if (@FailedPartitions)
					{
						my $Partition;
						my $DriveStatusNote;
						my $Egg;

						$html5 = <<HERE;
	<table border=0 width=375>
	<tr>
		<td width=100%>&nbsp</td>
	</tr>
	<tr>
		<td width=100%>
			<table border=1 cellpadding=2 cellspacing=0 width=372 bgcolor=#ffffff bordercolor=#eeeeee>
			<tr>
				<td width=50% bgcolor=#808080><b><font color=#ffffff>$_RE{'PARTITION'}</font></b></td>
				<td width=50% bgcolor=#808080 align=left><b><font color=#ffffff>$_RE{'STATUS'}</font></b></td>
			</tr>
HERE
						
						# If there is a recovering partition...
						if ($RecoveringPartition ne '')
						{
							# Set the drive status text.
							$DriveStatusText = "<a href=server_disk.cgi>$_RE{'MIRRORING_IN_PROGRESS'}</a>";

                            # Run through the completed paritions...
							foreach $Partition (@NormalPartitions)
							{
								# Append the completed partition status.
								$html5 .= "<tr><td width=50%><b>$Partition&nbsp;</b></td><td width=50% align=left>100%</td></tr>";
							}
							
							# Add the recovering partition.
							$html5 .= "<tr><td width=50%><b>$RecoveringPartition&nbsp;</b></td><td width=50% align=left>$PercentMirrored</td></tr>";
							
							# Run through the non-completed partitions...
							foreach $Partition (@FailedPartitions)
							{
								# Make sure the recovering parititon is not duplicated...
								if ($Partition ne $RecoveringPartition)
								{
									# Append the un-completed partition status.
									$html5 .= "<tr><td width=50%><b>$Partition&nbsp;</b></td><td width=50% align=left>0%</td></tr>";
								}
							}
						}
						else
						{
							# Set the drive status text.
							$DriveStatusText = $_RE{'MIRRORING_FAILURE_DETECTED'};

                            # Run through the completed paritions...
							foreach $Partition (@FailedPartitions)
							{
								# Append the completed partition status.
								$html5 .= "<tr><td width=50%><b>$Partition&nbsp;</b></td><td width=50% align=left><font color=red>$_RE{'FAILED'}</font></td></tr>";
							}
							
							# Run through the non-completed partitions...
							foreach $Partition (@NormalPartitions)
							{
								# Append the un-completed partition status.
								$html5 .= "<tr><td width=50%><b>$Partition&nbsp;</b></td><td width=50% align=left><font color=green>$_RE{'NORMAL'}</font></td></tr>";
							}
						}

						$html5 .= '</table></td></tr>';
						
						# If the first drive failed...
						if ($FailedDrive == 1)
						{
							# Move the label to the first drive.
							$DriveStatusLabel = "$_RE{'FIRST_DRIVE_STATUS'}&nbsp;";
							
							# Set the drive status note.
							$DriveStatusNote = ($RecoveringPartition ne '') ? $_RE{'MIRRORING_IN_PROGRESS_FIRST_WARNING'} : $_RE{'MIRRORING_FAILURE_DETECTED_FIRST_NOTE'};
						}
						else
						{
							# Set the drive status note.
							$DriveStatusNote = ($RecoveringPartition ne '') ? $_RE{'MIRRORING_IN_PROGRESS_SECOND_WARNING'} : $_RE{'MIRRORING_FAILURE_DETECTED_SECOND_NOTE'};
						}
						
						# If this is in recovery mode...
						if ($RecoveringPartition ne '')
						{
							# Append the correct note to the bottom.
							$html5 .= <<HERE;
	<tr>
		<td width=100%>
			<table border=0 cellpadding=0 cellspacing=1 width=375>
			<tr>
				<td width=100%><b><br><font color=red>$_RE{'WARNING'}</font></b>&nbsp;$DriveStatusNote</b></td>
			</tr>
			</table>
		</td>
	</tr>
HERE
						}
						else
						{

							# Append the correct note to the bottom.
							$html5 .= <<HERE;
	<tr>
		<td width=100%>
			<table border=0 cellpadding=0 cellspacing=1 width=375>
			<tr>
				<td width=100%><b><br>$_RE{'NOTE'}</b>&nbsp;$DriveStatusNote</td>
			</tr>
			</table>
		</td>
	</tr>
HERE
						}


						$html5 .= '</td></tr></table>';
					}
				}
			}

			# Default message...
			if ($html5 eq '')
			{
				$html5 .= <<HERE;
	<table border=0 cellpadding=0 cellspacing=1 width=375>
	<tr>
		<td width=100%><b><br>$_RE{'NOTE'}</b>&nbsp;$_RE{'MIRRORING_CONFIGURATION_NOTE'}</td>
	</tr>
	</table>
HERE
			}
		}
		else
		{
            # Check for configuration activity...
			if (Safe::backtick('/bin/ps -ax | grep raidextend') =~ /raidextend\.pl/m)
			{
				# The configuration phase is in progress.
				$DriveStatusText = $_RE{'EXTENSION_CONFIGURATION_IN_PROGRESS'};
				
				$html5 .= <<HERE;
	<table border=0 cellpadding=0 cellspacing=1 width=375>
	<tr>
		<td width=100%><b><br><font color=red>$_RE{'WARNING'}</font></b>&nbsp;$_RE{'EXTENSION_CONFIGURATION_IN_PROGRESS_WARNING'}</b></td>
	</tr>
	</table>
HERE
			}
			else
			{
				$html5 .= <<HERE;
	<table border=0 cellpadding=0 cellspacing=1 width=375>
	<tr>
		<td width=100%><b><br>$_RE{'NOTE'}</b>&nbsp;$_RE{'EXTENSION_CONFIGURATION_NOTE'}</td>
	</tr>
	</table>
HERE
			}
		}
		
		# Set the status message.
		$DriveStatusText = $_RE{'INSTALLED_AND_CONFIGURED'} unless($DriveStatusText ne '');
	}
	else
	{
		# Set the status message.
		$DriveStatusText = $StringResource->_S('DRIVE_STATUS_UNKNOWN');
	}

	# Set the label.
	$DriveStatusLabel = "$_R{'SECOND_DRIVE_STATUS'}&nbsp;" unless($DriveStatusLabel ne '');

	$html4 .= <<HERE;
<form method=post action=server_disk.cgi enctype="multipart/form-data">
<table border=0 cellPadding=0 cellSpacing=1 width=375>
	<tr>
		<td width=100% colspan=2><b>$_R{'ADD_DRIVE'}</b></td>
	</tr>
	<tr>
		<td width=100% colspan=2>&nbsp;</td>
	</tr>
	<tr>
		<td width=133><b>$DriveStatusLabel</b></td>
		<td align=left width=220>$DriveStatusText</td>
	</tr>
</table>
HERE

	$html6 .= '</form>';

	# Glom the HTML together.
	$html1 = $html1 . $html2 . $html3 . $html4 . $html5 . $html6;
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html1, 
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:600'}, 'location' => 'server_status.cgi' } ],
								 'class' => 'server_disk', 'id' => 900, 'title' => $_R{'IDT:900'},
								 'root-link' => 'server_status', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
