#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_updates.cgi                                                #
#                                                                             #
# Description: Provides status information for the installed appliance        #
#              software stack.                                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use SoftwareSupport;
use LanguageResource;
use ConfigurationData;
use DisplayMain;
use Safe;
use CGI;
use IO::File;
use ProductInformation;

###############################################################################
# GetConfigData
###############################################################################

sub GetConfigData
{
	my ($Label, @ConfigFileContents) = @_;
    my $Value;
	my $Line;

	# Loop through all of the lines in the file.
	foreach $Line (@ConfigFileContents)
	{
		# Continue if there is a comment.
		next if $Line =~ /^#/;

		# Continue if the label is not found.
		next unless $Line =~ /$Label/;
		
		# Parse out the value.
		$Value = $Line;
		$Value =~ s/.*$Label=(.*)$/$1/;
		chomp $Value;
		
		return $Value;
	}

	return '';
}

###############################################################################
# FindPartition
###############################################################################

sub FindPartition 
{
	my ($MountPoint) = @_;
	my $Return = '';
	
	# Get the file system table...
	if (open(FSTAB, "/etc/fstab"))
	{
		my @FileSystemTable = <FSTAB>;
        
		# Close the file.
		close(FSTAB);
		
		# Look through the file...
		foreach $_ (@FileSystemTable) 
		{
			# See if there is a mountpoint match...
			if (/$MountPoint\s/) 
			{
				# Set the return value.
				($Return) = /(\S*)/ ;
				
				last;
			}
		}
   }
   
   return $Return;
}

###############################################################################
# GetUpdateInformation
###############################################################################

sub GetUpdateInformation
{
	my $StringResource	= shift;
	my @Manifest;
	my $ProductInfo		= new ProductInformation();
	my $SoftwareSupport;
	my $ProductModel;
	my $ManifestName;
	my $Return;

	# Safe default.
	undef(@Manifest);

	# Get the product model and manifest.
	$ProductModel		= $ProductInfo->getModel();		
	$ManifestName		= $ProductInfo->getManufacturerID() . "_" . $ProductInfo->getFamily() . "_" . $ProductInfo->getSeries();
	$ManifestName		= lc($ManifestName);
	
	# Get available update manifest from the manufacturer support site.
	$SoftwareSupport	= new SoftwareSupport($ManifestName, $ProductInfo->getVersion());
	$SoftwareSupport->GetOfficialProductManifest(\@Manifest);
   
	# If there is a manifest...
	if (defined(@Manifest))
	{
		my %_R							= ( 'N/A'					=> '',
											'INSTALL'				=> '',
											'REINSTALL'				=> '' );
		my $InstalledMajorVersion;
		my $InstalledMinorVersion;
		my $InstalledReleaseVersion;
		my $InstalledVersion;
		my $MinorVersion;
		my $BuildVersion;
		my $MajorVersion;
		my $ButtonText;
		my $RPMName;
		my $i;		
		my $ProductMatch;
		my $Model;
		    
		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		# Run through the manifest...
		for $i (0 .. $#Manifest)
		{
			# Get the RPM name.
			$RPMName = $Manifest[$i]{'file-name'};  
			
			# Remove the .i386.rpm from the filename.
			$RPMName	=~ s/^(.*)-.*$/$1/;
			$RPMName	=~ s/^(.*)-\d.\d/$1/;
			
			# No matches wo far.
			$ProductMatch = 0;
			
			# Run through the acceptable models...
			foreach $Model (@{ $Manifest[$i]{'model'} })
			{
				# Look for a matching model number...
			    if (($Model eq $ProductModel) || 
					($Model eq '*')) 
			    {
					# A match is found.
					$ProductMatch = 1;
					last;
			    }			    
			}    	 
			       																								    
			# Validate the data...
			if (($ProductMatch) &&
				($Manifest[$i]{'display-name'}))
			{
				# Get the installed full version and major-minor version.
				$InstalledVersion			= Safe::backtick("/bin/rpm -q --queryformat '%{VERSION}-%{RELEASE}' $RPMName 2>/dev/null");
				$InstalledMajorVersion		= Safe::backtick("/bin/rpm -q --queryformat '%{VERSION}' $RPMName 2>/dev/null"); 
				
				# Break out the major and minor versions.
				($InstalledMajorVersion, $InstalledMinorVersion) = split(/\./, $InstalledMajorVersion); 
				
				# Get just the release.
				$InstalledReleaseVersion	= Safe::backtick("/bin/rpm -q --queryformat '%{RELEASE}' $RPMName 2>/dev/null");
				
				# Massage the manifest information.
				$MajorVersion = $MinorVersion = $BuildVersion = $Manifest[$i]{'version'};
				$MajorVersion =~ s/^(\d)*\.\d*-\d*$/$1/; 
				$MinorVersion =~ s/^\d*\.(\d)*-\d*$/$1/;
				$BuildVersion =~ s/^\d*\.\d*-(\d*)$/$1/;
			
				# Check version compatibilities.
				if (($MajorVersion > $InstalledMajorVersion) || 
					(($MajorVersion == $InstalledMajorVersion) && ($MinorVersion > $InstalledMinorVersion)) || 
					(($MajorVersion == $InstalledMajorVersion) && ($MinorVersion == $InstalledMinorVersion) && ($BuildVersion >= $InstalledReleaseVersion)))
				{
					# Re-join the model numbers.
					$Model = join(',', @{ $Manifest[$i]{'model'} });
					
					# Safe default.
					$InstalledVersion = $_R{'N/A'} unless($InstalledVersion);
					
					# Get the correct button text...
					$ButtonText = ($InstalledVersion eq $Manifest[$i]{'version'}) ? $_R{'REINSTALL'} : $_R{'INSTALL'};

					$Return .= <<HERE;
							<tr>
								<form method=post action=server_updates.cgi enctype="application/x-www-form-urlencoded">
									<td width=40%><b>$Manifest[$i]{'display-name'}&nbsp;</b></td>
									<td width=15%>$InstalledVersion</td>
									<td width=20%>$Manifest[$i]{'version'}</td>
									<td width=20% align=center>
										<input type=submit class=clsButton value='$ButtonText' name=install_rpm>
										<input type=hidden name=filename value='$Manifest[$i]{'file-name'}'>
										<input type=hidden name=location value='$Manifest[$i]{'location'}'>
										<input type=hidden name=model value='$Model'>
									</td>
								</form>
							</tr>            
HERE
				}
			}
		}
    }
    
	# If there is nothing to display...
	unless ($Return)
    {
         # No updates available.
		 $Return = '<tr><td><b>' . $StringResource->_S('NO_UPDATES_AVAILABLE') . '</b></td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr>';
    }
    
	return $Return;
}

###############################################################################
# InstallRPM
###############################################################################

sub InstallRPM
{
	my ($RPMLocation, $RPMFileName, $RPMModel)	= @_;
	my $ProductInfo		= new ProductInformation();
	my $ProductModel	= $ProductInfo->getModel();		
	my $ProductMatch	= 0;
	my $Model;
	my $Return			= 0;
	
	# No matches wo far.
	$ProductMatch = 0;

	# Run through the acceptable models...
	foreach $Model (split(/,/, $RPMModel))
	{
		# Look for a matching model number...
		if (($Model eq $ProductModel) || 
			($Model eq '*')) 
		{
			# A match is found.
			$ProductMatch = 1;
			last;
		}			    
	}    	 
	
	# Get the product model.
	if ($ProductMatch)
	{
		my @Arguments;
		my $RPMName			 = $RPMFileName;
		my $RPMURL			 = $RPMLocation . $RPMFileName;
		my $AvailableVersion = $RPMFileName;
		my $InstalledVersion;
		
		# Remove the .i386.rpm from the filename.
		$RPMName	=~ s/^(.*)-.*$/$1/;
		$RPMName	=~ s/^(.*)-\d.\d/$1/;

		# Get the available version and installed version.
		$AvailableVersion	= Safe::backtick("/bin/rpm -qp --queryformat '%{RELEASE}' $RPMURL");
		$InstalledVersion	= Safe::backtick("/bin/rpm -q --queryformat '%{RELEASE}' $RPMName");

		# Check for upgrade versus install...
		if ($InstalledVersion eq $AvailableVersion)
		{
			# Set the arguments for install.
			@Arguments = ( '/bin/rpm', '-i', '--force', '--replacepkgs', $RPMURL );
		}
		else
		{
			# Set the arguments for upgrade.
			@Arguments = ( '/bin/rpm', '-U', '--force', $RPMURL );
		}

		# Look for the special case involving the Administrator Web service...
		if ($RPMName =~ /Apache-admin|apache-admin/)
		{
			my $Script = '/var/tmp/extscript_rpm.sh';

			# Try to create the special script...
			if (open(SCRIPT, "> $Script"))
			{
				# Write out the script.
				print SCRIPT "#/bin/bash\n\n";
				print SCRIPT "/bin/sleep 3\n";
				print SCRIPT "exec " . @Arguments[0] . " " . @Arguments[1] . " " . @Arguments[2] . " " . @Arguments[3] . " \n";

				# Close the file.
				close(SCRIPT);

				# Set the permissions.
				chmod(0700, $Script);

				# Call the script.
				Safe::spawn_daemon($Script);

				# Wait a second.
				sleep(1);

				$Return = 240099;
			}
			else
			{
				$Return = 240001;
			}
		}
		else
		{
			$Return = (0 == Safe::system(@Arguments)) ? 0 : 240000;
		}
	}
	else
	{
		$Return = 240006;
	}

	return $Return;
}

###############################################################################
# UpgradeBIOS
###############################################################################

sub UpgradeBIOS 
{
	my ($BIOSPathRemote) = @_;
    my $BIOSPathLocal;
	my $BIOSFileLocal;
	my $Return;
	
	# Get the local file path.
	$BIOSPathRemote	=~ m/^.*(\\|\/)(.*)/;
	$BIOSPathLocal	=  "/tmp/$2";
	
	# Delete the old file.
    Safe::system("/bin/rm -f $BIOSPathLocal");
	
	# Try to create the local instance of the file...
	if ($BIOSFileLocal = new IO::File($BIOSPathLocal, O_RDWR | O_CREAT | O_EXCL))
	{
		my ($BytesRead, $Buffer);
		my $SourceDevice;
		
		# Read off the source until completed...
		while ($BytesRead = read($BIOSPathRemote, $Buffer, 1024))
		{
			# Write out to the local file.
			print $BIOSFileLocal $Buffer;
		}

		# Close the file.
		close($BIOSFileLocal);

		# If the /boot parition exists...
		if ($SourceDevice = FindPartition('/boot'))
		{
			# Prep the /emboot partition device.
			chomp($SourceDevice);
			chop($SourceDevice);
			$SourceDevice .= '3';

			# Try to push the BIOS package...
			if (0 == ($Return = Safe::system("/usr/ins/intel/bin/asm/sysBiosUpgrade", $BIOSPathLocal)))
			{
				# Set up LILO.
				Safe::system('/bin/mkdir -p /emboot');
				Safe::system("/bin/mount $SourceDevice /emboot");
				Safe::system('/sbin/lilo -R biosupgrade');
				Safe::system('/bin/umount /emboot');
				Safe::system('/bin/rmdir /emboot');
			}

            # Adjust the return value.
			$Return	/= 256;
            $Return += 240050 if($Return);
		}
		else
		{
			$Return = 240003;
		}
	}
	else
	{
		$Return = 240004;
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_updates.lrc', 'common.lrc' ]);
	my $PageSelection		= 'default';
	my $Result;
	my $Feedback;
	my $html;
	
	# If applying an new BIOS...
	if (defined($cgi->param('update_bios')))
	{
		my $BIOSPathRemote = $cgi->param('biosfile');

		# Validate the path...
		if ((defined($BIOSPathRemote)) &&
			(length($BIOSPathRemote)) &&
			($BIOSPathRemote =~ /\w+/))
		{
			# If things go well...
			if (0 == ($Result = UpgradeBIOS($BIOSPathRemote)))
			{
				# Display the "Reboot?" page.
				$PageSelection = 'reboot';
			}
		}
		else
		{
			$Result = 240002;
		}
	}
	elsif (defined($cgi->param('reboot_yes')))
	{
		# Try to shut down the system...
		unless ($Result = Safe::backtick('/sbin/init 6'))
		{
			# Force a logout.
			print $cgi->redirect( -url => '/logout.cgi?message=biosupdate' );
			exit(0);
		}
		else
		{
			$Result = 240005;
		}
	}
	
	# If things are going good and the page needs to be displayed...
	if ('default' eq $PageSelection)
	{
		my %_R				= ( 'IDT:2400'	   				=> '',
								'INSTALLED_SOFTWARE'		=> '',
								'DOWNLOAD_LATEST'			=> '',
								'SOFTWARE_PACKAGES'			=> '',
								'VERSION_INSTALLED'			=> '',
								'VERSION_AVAILABLE'			=> '',
								'MORE_SOFTWARE_INFORMATION'	=> '',
								'OS_AND_BIOS'				=> '',
								'LINUX'						=> '',
								'BIOS'						=> '',
								'SPECIFY_BIOS_LOCATION'		=> '',
								'BIOS_FILE'					=> '',
								'BROWSE'					=> '',
								'UPDATE_BIOS'				=> '' );
		my $Title;
		my $Message;
		my $ButtonName;
		my $ButtonTitle;
		my $PackageList		= '';
		my $GOOptions		= '';
		my $OSVersion;
		my @BIOSData;
		my $BIOSVersion;

		# Get the string resources.
		$StringResource->GetStrings(\%_R);

		# Get the OS version.
		$OSVersion		= Safe::backtick("/bin/cat /etc/redhat-release");
		chomp($OSVersion);

		# Get the BIOS version.
		@BIOSData		= Safe::backtick("/usr/ins/intel/bin/asm/sysSMBIOS dump bios");
		$BIOSVersion	= GetConfigData('BIOS_VERSION', @BIOSData);

		# If applying a change...
		if (defined($cgi->param('install_rpm')))
		{     
			# Install the RPM.
			$Result = InstallRPM($cgi->param('location'), $cgi->param('filename'), $cgi->param('model'));
			
			# Evaluate success/failure...
			if (240099 == $Result)
			{
				# Get the special case string...
				$Feedback = $StringResource->_S('WEB_SERVER_UPDATE');
				$Result = 0; 
			}
		}
		elsif (defined($cgi->param('install_em')))
		{
			# Install group operations.
			Safe::system('/sbin/chkconfig --level 3 wcdd on');
			Safe::system('/etc/rc.d/init.d/wcdd start');
			
			# Signal the parent document.
			$GOOptions = '<script>parent.title.location = "server_title.cgi";</script>';

			# Success.
			$Result = 0; 
		}
		elsif (defined($cgi->param('uninstall_em')))
		{
			# Uninstall group operations.
			Safe::system('/sbin/chkconfig --level 3 wcdd off');
			Safe::system('/etc/rc.d/init.d/wcdd stop');
			sleep(1) while(Safe::backtick('/bin/ps -ax | grep wcdd') =~ /\/wcdd/g);
			Safe::system('rm -f /usr/ins/intel/bin/wcdd-source/*');
			Safe::system('rm -f /usr/ins/intel/bin/wcdd-result/*');
			
			# Signal the parent document.
			$GOOptions = '<script>parent.title.location = "server_title.cgi";</script>';

			# Success.
			$Result = 0; 
		}
		
		# If group operations are available...
		if (Safe::backtick('/sbin/chkconfig --list wcdd') =~ /3:on/)
		{
			# Set up the uninstallation tokens.
			$Title			= $StringResource->_S('GROUP_OPERATIONS_UNINSTALL');
			$Message		= $StringResource->_S('GROUP_OPERATIONS_UNINSTALL_NOTE');
			$ButtonTitle	= $StringResource->_S('UNINSTALL_GROUP_OPERATIONS');
			$ButtonName		= 'uninstall_em';
		}
		else
		{
			# Set up the installation tokens.
			$Title			= $StringResource->_S('GROUP_OPERATIONS_INSTALL');
			$Message		= $StringResource->_S('GROUP_OPERATIONS_INSTALL_NOTE');
			$ButtonTitle	= $StringResource->_S('INSTALL_GROUP_OPERATIONS');
			$ButtonName		= 'install_em';
		}

		$GOOptions .= <<HERE;
<form method=post action=server_updates.cgi enctype="multipart/form-data">
	<table bgcolor=#eeeeee border=0 width=375>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td width=100%><b>$Title</b></td>
			</tr>
			<tr>
				<td width=100%>$Message</td>
			</tr>
			<tr>
				<td width=100%>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% align=right><input type=submit class=clsButton name=$ButtonName value='$ButtonTitle'></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
		
		# Get the software installation state HTML.
		$PackageList = GetUpdateInformation($StringResource);
		
		$html = <<HERE;
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td width=359 valign=top align=left><b>$_R{'INSTALLED_SOFTWARE'}</b></td>
			</tr>
			<tr>
				<td width=353>$_R{'DOWNLOAD_LATEST'}</td>
			</tr>
			<tr>
				<td width=100%>
					<table border=0 cellpadding=0 cellspacing=0 width=100% bgcolor=#eeeeee>
					<tr>
						<td width=100%>
							<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
							<tr>
								<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SOFTWARE_PACKAGES'}</font></b></td>
								<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'VERSION_INSTALLED'}</font></b></td>
								<td width=20% bgcolor=#808080><b><font color=#ffffff>$_R{'VERSION_AVAILABLE'}</font></b></td>
								<td width=20% bgcolor=#808080>&nbsp;</td>   
							</tr>
$PackageList
							</table>
						</td>
					</tr>
					</table>
					&nbsp;<a href=/cgi-bin/server/server_updates_more.cgi>$_R{'MORE_SOFTWARE_INFORMATION'}</a>
				</td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
	<form method=post action=server_updates.cgi enctype="multipart/form-data">
		<table border=0 width=375 bgcolor=#eeeeee>
		<tr>
			<td>
				<table width=100%>
				<tr>
					<td width=100%><b>$_R{'OS_AND_BIOS'}</b></td>
				</tr>
				<tr>
					<td width=100%>
                        <table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
						<tr>
							<td width=25% bgcolor=#808080><font color=#ffffff><b>&nbsp;</b></font></td>
							<td width=70% bgcolor=#808080><font color=#ffffff><b>$_R{'VERSION_INSTALLED'}</b></font></td>
						</tr>
						<tr>
							<td width=25%><b>$_R{'LINUX'}</b></td>
							<td width=70%>$OSVersion</td>
						</tr>
						<tr>
							<td width=25%><b>$_R{'BIOS'}</b></td>
							<td width=70%>$BIOSVersion</td>
						</tr>
						</table>
					</td>
				</tr>
				<tr>
					<td width=100%>&nbsp;</td>
				</tr>
				<tr>
					<td width=100%>$_R{'SPECIFY_BIOS_LOCATION'}</td>
				</tr>
				<tr>
					<td width=100%><b>$_R{'BIOS_FILE'}&nbsp;&nbsp;<input class=clsButtonFile type=file name=biosfile size=29></b></td>
				</tr>
				<tr>            
					<td width=100% align=right>&nbsp;</td>
				</tr>
				<tr>
					<td width=100% align=right><input class=clsbutton type=submit name=update_bios value='$_R{'UPDATE_BIOS'}'></td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
	</form>
$GOOptions
HERE

		# If there was an error...
		if ($Result)
		{
			# Get the error string.
			$Feedback = $StringResource->_S($Result);
		}
		
		# Display the page.
		DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html,
									 'class' => 'server_updates', 'id' => 2400, 'title' => $_R{'IDT:2400'},
									 'root-link' => 'server_updates', 'result' => $Result, 'feedback' => $Feedback });
	}
	elsif ('reboot' eq $PageSelection)
	{
		my %_R				= ( 'IDT:2401'				=> '',
								'YES'					=> '',
								'NO'					=> '',
								'UPDATE_REBOOT_NOTE'	=> '',
								'ACCEPT_REBOOT'			=> '' );
		
		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		$html = <<HERE;
<form method=post action=server_updates.cgi enctype="application/x-www-form-urlencoded">
<table border=0 width=375 bgcolor=#eeeeee>
<tr>
	<td>
		<table border=0 width=100%>
		<tr>
			<td colspan=2>$_R{'UPDATE_REBOOT_NOTE'}</td>
		</tr>
		<tr>
			<td colspan=2 width=100%>&nbsp;</td>
		</tr>
		<tr>
			<td colspan=2>$_R{'ACCEPT_REBOOT'}</td>
		</tr>
		<tr>
			<td colspan=2 width=100%>&nbsp;</td>
		</tr>
		<tr>
			<td width=17% align=left><input type=submit class=clsButton value='$_R{'YES'}' name=reboot_yes></td>
			<td width=83% align=right><input type=submit class=clsButton value='$_R{'NO'}' name=reboot_no></td>
		</tr>
		</table>
	</td>
</tr>
</table>
</form>
HERE

		# Display the page.
		DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html,
									 'title' => $_R{'IDT:2401'}, 'result' => $Result, 'feedback' => $Feedback,
									 'root-link' => 'server_updates'} );
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
