package Authorize;

###############################################################################
#                                                                             #
# Filename: Authorize.pm                                                      #
#                                                                             #
# Description: Contains routines for determining the target user/domain and   #
# what privilege level (if any) the currently authenticated user has with     #
# regards to operating on the target.                                         #
#                                                                             #
# This module depends on rewriting rules in httpd.conf such as this:          #
#                                                                             #
# RewriteRule ^(.*)/([^/=]+)=([^/]+)/(.*) $1/$4 [N,E=NS_URL_${toupper:$2|NIL}:$3] #
# RewriteRule ^(/cgi-bin/.*)              $1    [PT]                          #
#                                                                             #
# User and domain scripts will be called using URLs such as:                  #
#                                                                             #
# http://server.com:5555/cgi-bin/user/simmons/user_setmail.cgi                #
# http://server.com:5555/cgi-bin/domain/test1.com/domain_settings.cgi         #
#                                                                             #
# The rewriting rules will extract the username or domain into an environment #
# variable which the routines in this module will read.                       #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK);
use LanguageResource;
use LanguageLogCGI;
use DisplayMain;
use Users;
use CGI;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 

###############################################################################
# Privilege levels
###############################################################################

use vars qw($PRIV_NONE $PRIV_USER $PRIV_DOMAIN $PRIV_SERVER);

$PRIV_NONE   = 0;
$PRIV_USER   = 1;
$PRIV_DOMAIN = 2;
$PRIV_SERVER = 3;

###############################################################################
# Authorize::checkAuthorization
#
# Return: 1 - authorized
#         0 - unauthorized
#
###############################################################################

sub checkAuthorization
{
	my ($LevelCGI, $cgi)	= @_;
	my $Domain;
	my $User;
	my $Level;
	my $Return				= 1;

	# CGI from user directory.
	($User, $Level) = getTargetUser();
	
	# Safe default.
	$LevelCGI = '' unless(defined($LevelCGI));

	# Based on the privilege level...
	unless ($LevelCGI eq lc('user'))
	{
		# CGI from domain directory.
		($Domain, $Level) = getTargetDomain();
	}

	# Display error message if accessing unauthorized script
	if ($Level == $PRIV_NONE)
	{
		my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
		my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'Authorize.lrc' ]);
		my $Log					= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'Authorize.lrc' ]);
		my $RemoteUser;
		my $Feedback;
		my $Result				= 1300000;

		# Create CGI if not available.
		$cgi = new CGI unless($cgi);

		# User information and feedback...
		$User		= $StringResource->_S('NOT_AVAILABLE') unless($User);
		$RemoteUser	= $cgi->remote_user();
		$Feedback	= $StringResource->_S($Result);

		# If the users are not the same...
		if ($User ne $RemoteUser)
		{
			# Log the security check.
			$Log->Append('security', 'information', 1300001, [ $RemoteUser, $User, $cgi->remote_host(), $0 ]);
		}
		else
		{
			# Log the security check.
			$Log->Append('security', 'information', 1300002, [ $RemoteUser, $cgi->remote_host(), $0 ]);
		}
		
		# Display the page.
		DisplayMain->printDocument({ 'q' => $cgi, 'result' => $Result, 'feedback' => $Feedback });
		
		# User is not authorized.
		$Return = 0; 
	}

	return $Return;
}

###############################################################################
# Authorize::getTargetDomain
#
# Return the target domain and privilege level as set
# by the Apache mod_rewrite rule.
#
###############################################################################

sub getTargetDomain
{
	my $TargetDomain;
	my $Level;

	# Determine the target domain which was set by the RewriteRule
	$TargetDomain = ($ENV{'NS_URL_DOMAIN'}) ? $ENV{'NS_URL_DOMAIN'} : undef;

	# If there is NOT a target domain...
	if (!defined($TargetDomain))
	{
		# There are no privileges.
		$Level = $Authorize::PRIV_NONE;
	}
	else
	{
		# Determine the privilege level.
		my @PasswordEntry	= getpwnam($ENV{REMOTE_USER});
		my $User			= $PasswordEntry[0];
		my $UserID			= $PasswordEntry[3];

		# Based on the user ID.
		if (10000 == $UserID)
		{
			# User is a Server Admin (GID is 10000).
			$Level = $Authorize::PRIV_SERVER;
		}
		elsif (10001 == $UserID)
		{
			# User is a Domain Admin. But are they the administrator of the target domain...
			if ($TargetDomain eq Users::getUserDomain($User))
			{
				# The user has Domain Admin privileges over this domain.
				$Level = $Authorize::PRIV_DOMAIN;
			}
			else
			{
				# This Domain Admin is trying to access another domain -- disallow.
				$Level = $Authorize::PRIV_NONE;
			}
		}
		else
		{
			# Regular users never have any privileges over domains.
			$Level = $Authorize::PRIV_NONE;
		}
	}

	return ($TargetDomain, $Level);
}

###############################################################################
# Authorize::getTargetUser
#
# Return the target user and privilege level as set
# by the Apache mod_rewrite rule.
#
###############################################################################

sub getTargetUser
{
	my ($TargetUser, $Level);

	# Determine the target user which was set by the RewriteRule
	$TargetUser = ($ENV{'NS_URL_USER'}) ? $ENV{'NS_URL_USER'} : undef;
	
	if (!defined($TargetUser))
	{
		# No target user was found, so there are no privileges.
		$Level = $Authorize::PRIV_NONE;
	}
	else
	{
		# Determine the privilege level
		my @PasswordEntry	= getpwnam($ENV{REMOTE_USER});
		my $User			= $PasswordEntry[0];
		my $UserID			= $PasswordEntry[3];

		# Based on the user ID.
		if (10000 == $UserID)
		{
			# User is a Server Admin (GID is 10000)
			$Level = $Authorize::PRIV_SERVER;
		}
		elsif (10001 == $UserID)
		{
			# User is a Domain Admin. But is he the admin of the target's domain...
			if (Users::getUserDomain($User) eq Users::getUserDomain($TargetUser))
			{
				# Yes, he has Domain Admin privileges over this user.
				$Level = $Authorize::PRIV_DOMAIN;
			}
			else
			{
				# This Domain Admin is trying to access a user in another domain -- disallow.
				$Level = $Authorize::PRIV_NONE;
			}
		}
		else
		{
			# User is just a regular User. Are they trying to change they own settings?
			if ($User eq $TargetUser)
			{
				# Yes, so they have User privileges over themselves.
				$Level = $Authorize::PRIV_USER;
			}
			else
			{
				# No, he is trying to change someone else -- disallow.
				$Level = $Authorize::PRIV_NONE;
			}
		}
	}

	return ($TargetUser, $Level);
}

1;

__END__
