package Chronos;

###############################################################################
#                                                                             #
# Filename: Chronos.pm                                                        #
#                                                                             #
# Description: Contains script needed to manage date/time data.               #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Daemon;
use Safe;

###############################################################################
# Chronos::new
###############################################################################

sub new
{
	my ($class, $PreferredLanguage, $RootDirectory) = @_;

	# Initialize the object.
	my $this =
	{
		'string-resource'	=> new LanguageResource($PreferredLanguage, $RootDirectory, [ 'Chronos.lrc' ]),
	};
	# Align the object with this package.
	bless($this, $class);
	
	# Return the object.
	return $this;
}

###############################################################################
# GetTimeZone
###############################################################################

sub GetTimeZone
{
	my $TimeZone = '';

	# Try to open the timezone file...
	if (open(TIMEZONE, '/etc/sysconfig/clock'))
	{
		my $Label;

		# Loop through the file looking for the timezone.
		while (<TIMEZONE>) 
		{
			# If this line has to the right label...
			if (/^ZONE/) 
			{ 
				# Get the timezone information and clean it up.
				($Label, $TimeZone) = split(/=/, $_); 
				chomp($TimeZone);
			}
		}

		# Close the file.
		close(TIMEZONE);
	}

	return $TimeZone;
}

###############################################################################
# Chronos::SetTimeZone
###############################################################################

sub SetTimeZone
{
	my ($this, $TimeZone)	= @_;
	my $Return				= 0;

	# Try to open the clock file...
	if (open(CLOCK, '> /etc/sysconfig/clock'))
	{
		# Try to lock the file...
		if (flock(CLOCK, 2))
		{
			# If a timezone is specified...
			if (defined($TimeZone))
			{
				# Update the timezone information.
				print CLOCK "ZONE=$TimeZone\n";
				print CLOCK "UTC=true\n";
			}
			else
			{
				print CLOCK "UTC=false\n";
			}
		}
		else
		{
			$Return = 210013;
		}

		# Close the file.
		close(CLOCK);
	}
	else
	{
		$Return = 210002;
    }
	
	# If things are still running smoothly...
	if (0 == $Return)
	{
		# If there is a time zone mapping...
		if ( -e '/etc/localtime' )
		{
			# Delete the mapping.
			$Return = (unlink('/etc/localtime')) ? 0 : 210000;
		}
		
		# If there is a specified time zone...
		if ((0 == $Return) &&
			(defined($TimeZone)))
		{
			# Create the new mapping...
			$Return = (symlink("/usr/share/zoneinfo/$TimeZone", '/etc/localtime')) ? 0 : 210001;
		}
	}

	return $Return;
}

###############################################################################
# Chronos::GetTime
###############################################################################

sub GetTime
{
	my ($this, $TimeDataRef) = @_;

	# If there is a time data array...
	if (defined($TimeDataRef))
	{
		my $TimeRaw;
		my $Key;

		# Get the raw date/time and clean it up.
		$TimeRaw = Safe::backtick('/bin/date +%H:%M:%S:%w:%m:%d:%Y');
		chomp($TimeRaw);

		# Break out the date and time.
		($TimeDataRef->{'hour'}, $TimeDataRef->{'minute'}, $TimeDataRef->{'second'}, $TimeDataRef->{'day'}, $TimeDataRef->{'month'}, $TimeDataRef->{'date'}, $TimeDataRef->{'year'}) = split(/:/, $TimeRaw);
		
        # Translate the data.
		foreach $Key (keys(%{ $TimeDataRef }))
		{
			# Strip the lead zero.
			$TimeDataRef->{$Key} += 0;
		}
	}
}

###############################################################################
# Chronos::SetTime
###############################################################################

sub SetTime
{
	my ($this, $TimeDataRef)	= @_;
	my $Return;
	
	# Safe defaults.
	$TimeDataRef->{'second'}	= 0 unless(defined($TimeDataRef->{'second'}));
	$TimeDataRef->{'minute'}	= 0 unless(defined($TimeDataRef->{'minute'}));
	$TimeDataRef->{'hour'}		= 0 unless(defined($TimeDataRef->{'hour'}));
	$TimeDataRef->{'date'}		= 1 unless(defined($TimeDataRef->{'date'}));
	$TimeDataRef->{'month'}		= 0 unless(defined($TimeDataRef->{'month'}));
	$TimeDataRef->{'year'}		= 1900 unless(defined($TimeDataRef->{'year'}));

	# Validate the input data...
	if (((0 <= $TimeDataRef->{'second'}) && (59 >= $TimeDataRef->{'second'})) &&
		((0 <= $TimeDataRef->{'minute'}) && (59 >= $TimeDataRef->{'minute'})) &&
		((0 <= $TimeDataRef->{'hour'})   && (23 >= $TimeDataRef->{'hour'})) &&
		((1 <= $TimeDataRef->{'date'})   && (31 >= $TimeDataRef->{'date'})) &&
		((1 <= $TimeDataRef->{'month'})  && (12 >= $TimeDataRef->{'month'})))
	{
		my $Parameter;
		my $Key;

        # Translate the data.
		foreach $Key (keys(%{ $TimeDataRef }))
		{
			# Strip the lead zero.
			$TimeDataRef->{$Key} += 0;
		}
		
		# Pad the values.
		$TimeDataRef->{'second'}	= '0'  . $TimeDataRef->{'second'} if($TimeDataRef->{'second'} <= 9);
		$TimeDataRef->{'minute'}	= '0'  . $TimeDataRef->{'minute'} if($TimeDataRef->{'minute'} <= 9);
		$TimeDataRef->{'hour'}		= '0'  . $TimeDataRef->{'hour'} if($TimeDataRef->{'hour'} <= 9);
		$TimeDataRef->{'date'}		= '0'  . $TimeDataRef->{'date'} if($TimeDataRef->{'date'} <= 9);
		$TimeDataRef->{'month'}		= '0'  . $TimeDataRef->{'month'} if($TimeDataRef->{'month'} <= 9);
		$TimeDataRef->{'year'}		= '0'  . $TimeDataRef->{'year'} if($TimeDataRef->{'year'} <= 9);
		$TimeDataRef->{'year'}		= '19' . $TimeDataRef->{'year'} if(4 > length($TimeDataRef->{'year'}));

		# Build the configuration parameter.
		$Parameter	= $TimeDataRef->{'month'} . 
					  $TimeDataRef->{'date'} . 
					  $TimeDataRef->{'hour'} . 
					  $TimeDataRef->{'minute'} .
					  $TimeDataRef->{'year'} . '.' .
					  $TimeDataRef->{'second'};
		
		# Disable NTP.
		$this->ControlNTP(0);

		# Set the date and time information.
		Safe::system("/bin/date $Parameter");
		Safe::system('/sbin/hwclock --systohc --utc');

		# Success.
		$Return	= 0;
	}
	else
	{
		$Return = 210005;
	}

	return $Return;
}

###############################################################################
# Chronos::GetStatusNTP
###############################################################################

sub GetStatusNTP
{
	my $XNTPDaemon	= new Daemon('xntpd');

	# Get the status.
	return $XNTPDaemon->GetStatus();
}

###############################################################################
# Chronos::ControlNTP
###############################################################################

sub ControlNTP
{
	my ($this, $Activity) = @_;
	my $Return;

	# Validate the input parameter(s)...
	if ((defined($Activity)) &&
		((2 >= $Activity) && (0 <= $Activity)))
	{
		my $XNTPDaemon	= new Daemon('xntpd');

		# Pass on the activity.
		$XNTPDaemon->Control(0);

		# If there is activity...
		if ($Activity) 
		{
            my @ServerList;

			# Get the current server list.
			if (0 == ($Return = $this->GetNTPServers(\@ServerList)))
			{			
				# Try to open the cron job...
				if (open (NTPCRON, "> /etc/cron.daily/ins_ntp_to_hwclock"))
				{
					# Write out the mini-script.
					print NTPCRON '#!/bin/sh' . "\n\n";
					print NTPCRON '/sbin/hwclock --systohc --utc';

					# Close the file.
					close (NTPCRON);

					# Set the permissions.
					Safe::system('/bin/chmod 750 /etc/cron.daily/ins_ntp_to_hwclock');

					# Synchronize with the server.
					if (0 == Safe::system("/usr/sbin/ntpdate -b -t 5 $ServerList[0]"))
					{
						# Set the hardware clock to system time & UTC 
						Safe::system('/sbin/hwclock --systohc --utc');
	
						# Pass on the activity.
						$XNTPDaemon->Control($Activity);
	
						# Success.
						$Return = 0;
					}
					else
					{
						# Delete the cron job.
						unlink('/etc/cron.daily/ins_ntp_to_hwclock');
						$Return = 210010;
					}
				}
				else
				{
					$Return = 210003;
				}
			}
		}
		else
		{
			# Delete the cron job.
			$Return = (unlink('/etc/cron.daily/ins_ntp_to_hwclock')) ? 0 : 210004;
		}
	}
	else
	{
		$Return = 210005;
	}

	return $Return;
}

###############################################################################
# Chronos::GetNTPServers
###############################################################################

sub GetNTPServers
{
	my ($this, $ServerListRef) = @_;
	my $Return;

	# Validate the imput parameter(s)...
	if (defined($ServerListRef))
	{
		# Try to open the configuration file...
        if (open (NTPCONF, '< /etc/ntp.conf'))
		{
			# Get the current configuration.
			my @NTPConfiguration = <NTPCONF>;
		
			# Close the file.
			close (NTPCONF);
	
			# Get the server list...
			@{ $ServerListRef } = grep (/server/, @NTPConfiguration);
	
			# Run through the server list...
			for (@{ $ServerListRef }) 
			{
				# Clean up the value.
				($_) = (split(/\s+/, $_))[1];
			}
			
			# Set the return...
			$Return	= (scalar(@{ $ServerListRef })) ? 0 : 210007;
		}
		else
		{
			$Return = 210006;
		}
	}
	else
	{
		$Return = 210005;
	}

	return $Return;
}

###############################################################################
# Chronos::SetNTPServers
###############################################################################

sub SetNTPServers
{
	my ($this, $ServerListRef, $Trusted) = @_;
	my $Return;

	# Validate the imput parameter(s)...
	if (defined($ServerListRef))
	{
		# If there is anything in the array...
		if ((defined($ServerListRef->[0])) &&
			('' ne $ServerListRef->[0]))
		{
			# Try to synch the time using the "preferred" server...
			if (($Trusted) ||
				(0 == Safe::system("/usr/sbin/ntpdate -q -b -t 5 $ServerListRef->[0]")))
			{
				# Try to open the configuration file...
				if (open (NTPCONF, '> /etc/ntp.conf'))
				{
					# Try to lock the file...
					if (flock(NTPCONF, 2))
					{
						my $HostName = Safe::backtick('/bin/hostname');

						# Set the preferred server...
						$ServerListRef->[0] .= " prefer\n";

						# Add the header.
						print NTPCONF "# NTP Configuration for $HostName\n\n";

						# Run through the server list...
						for (@{ $ServerListRef }) 
						{ 
							# Add the server.
							print NTPCONF "server     $_\n"; 
						}

						# Set the footer.
						print NTPCONF "server     127.127.1.0                      # local clock\n";
						print NTPCONF "fudge      127.127.1.0   stratum 10\n\n";
						print NTPCONF "driftfile  /etc/ntp/drift\n\n";
						print NTPCONF "authenticate     no\n";

						# Success.
						$Return	= 0;
					}
					else
					{
						$Return = 210009;
					}

					# Close the file.
					close (NTPCONF);
				}
				else
				{
					$Return = 210008;
				}
			}
			else
			{
				$Return = 210011;
			}
		}
		else
		{
			# Stop the service.
			$this->ControlNTP(0);

			if ( -e '/etc/ntp.conf' )
			{
				# Delete the cron job.
				$Return = (unlink('/etc/ntp.conf')) ? 0 : 210012;
			}
			else
			{
				# Success.
				$Return = 0;
			}
		}
	}
	else
	{
		$Return = 210005;
	}

	return $Return;
}

###############################################################################
# Chronos::GetErrorString
###############################################################################

sub GetErrorString
{
	my ($this, $ErrorCode) = @_;
	my $ErrorString;

	# Validate the input parameter(s)...
	if (defined($ErrorCode))
	{
		# Return the error string.
        $ErrorString = $this->{'string-resource'}->_S($ErrorCode);
	}

	return $ErrorString;
}

1;

__END__
