package ConfigurationData;

###############################################################################
#                                                                             #
# Filename: ConfigurationData.pm                                              #
#                                                                             #
# Description: Contains script needed to manage configuration data.           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;
use Errno qw(/.*/);
use FileHandle;

require Exporter;

###############################################################################
# ConfigurationData::import
###############################################################################

sub import
{
	my $Caller	= caller;

	eval
	{
		require Fcntl;

		# Try to export the fnctl symbols...
		Exporter::export('Fcntl', $Caller);
	};

	eval
	{
		require Errno;
	
		# Try to export the Errno symbols...
		Exporter::export('Errno', $Caller, qw(/.*/));
	};
}

###############################################################################
# ConfigurationData::new
###############################################################################

sub new
{
	my ($this) = @_;

	# Initialize the object.
	$this = 
	{
		'gdm'	=> { 'begin' => '<', 'end' => '>' },
	};

	# Align the object with this package.
	bless($this, 'ConfigurationData');
	
	# Return the object.
	return $this;
}

###############################################################################
# ConfigurationData::DESTROY
###############################################################################

sub DESTROY
{
	my $this = shift;

	# Detach from the file.
	$this->Detach(1);
}

###############################################################################
# ConfigurationData::Attach
###############################################################################

sub Attach
{
	my ($this, $Resource, $Options)	= @_;
	my $Return = 0;

	# Validate the input parameters...
	if (defined($Resource))
	{
		# Make sure the object is detached...
		if (0 == ($Return = $this->Detach(1)))
		{
			# Cache the file path.
			$this->{'file-path'} 	= $Resource;
			
			# Create a file handle object.
			$this->{'file-handle'}	= new FileHandle;

			# If there are options...
			if (defined($Options))
			{
				# Use the options provided.
				$this->{'file-mode'}		= $Options->{'mode'};
				$this->{'file-permissions'}	= $Options->{'permissions'};
				$this->{'auto-save'}		= $Options->{'auto-save'};
				$this->{'loose-match'}		= $Options->{'loose-match'};
			}

			# Safe defaults.
			$this->{'file-mode'}		= O_RDONLY	unless(defined($this->{'file-mode'}));
			$this->{'file-permissions'}	= 0400    	unless(defined($this->{'file-permissions'}));

			# Always append the creation bit.
			$this->{'file-mode'}		|= O_CREAT;
			
			# Try to open the file...
			if ($this->{'file-handle'}->open($this->{'file-path'}, $this->{'file-mode'}, $this->{'file-permissions'}))
			{
				my $FileHandle = $this->{'file-handle'};

				# Get the file data.
				$this->{'file-data'} = [ <$FileHandle> ];
			}
			else
			{
				$Return = EINVAL;
			}
		}
	}
	else
	{
		$Return = 1;
	}

	return $Return;
}

###############################################################################
# ConfigurationData::Reset
###############################################################################

sub Reset
{
	my $this = shift;

	# Undefine the object elements.
	undef($this->{'auto-save'});
	undef($this->{'loose-match'});
	undef($this->{'file-path'});
	undef($this->{'file-data'});
	undef($this->{'file-mode'});
	undef($this->{'file-handle'});
	undef($this->{'file-permissions'});
}

###############################################################################
# ConfigurationData::Detach
###############################################################################

sub Detach
{
	my ($this, $Forced)	= @_;
	my $Return			= 0;

	# Safe default.
	$Forced = 1 unless(defined($Forced));
	
	# If there is a file handle...
	if (defined($this->{'file-handle'}))
	{
		# If auto-save is 'on'...
		if ((defined($this->{'auto-save'})) &&
			(1 == $this->{'auto-save'}))
		{
			# Try to flush the data.
			$Return = $this->Flush();
		}

		# Check for a forced detach...
		$Return = 0 if ($Forced);

		# If things are running smoothly...
		if (0 == $Return)
		{
			# Close the file handle.
			$this->{'file-handle'}->close();

			# Reset the member data.
			$this->Reset();
		}
	}

	return $Return;
}

###############################################################################
# ConfigurationData::Flush
###############################################################################

sub Flush
{
	my $this 	= shift;
    my $Return	= 0;

	# If there is a file handle and it is open for writing...
	if ((defined($this->{'file-data'})) &&
		(defined($this->{'file-handle'})) &&
		((O_WRONLY | O_RDWR) & $this->{'file-mode'}))
	{
		# Close the file handle.
		$this->{'file-handle'}->close();

		# Try to re-open the file truncated...
		if ($this->{'file-handle'}->open($this->{'file-path'}, $this->{'file-mode'} | O_TRUNC, $this->{'file-permissions'}))
		{
			my $FileHandle = $this->{'file-handle'};

			# Set the file data.
			print $FileHandle @{ $this->{'file-data'} };
		}
		else
		{
			# Reset the member data.
			$this->Reset();

			$Return = EBADF;
		}
	}

	return $Return;
}

###############################################################################
# ConfigurationData::GetResourcePath
###############################################################################

sub GetResourcePath
{
	my $this = shift;

	# Return the requested data.
	return $this->{'file-path'};
}

###############################################################################
# ConfigurationData::GetResourceOptions
###############################################################################

sub GetResourceOptions
{
	my $this = shift;

	# Return the requested data.
	return ( 
			 'mode'			=> $this->{'file-mode'},
			 'permissions'	=> $this->{'file-permissions'},
			 'auto-save'	=> $this->{'auto-save'},
		   );
}

###############################################################################
# ConfigurationData::GetValueKeyed
###############################################################################

sub GetValueKeyed
{
	my ($this, $Key) = @_;
    my $Value;
	
	# If there is a file handle...
	if (defined($this->{'file-handle'}))
	{
		my $Line;
	
		# Run through the data...
		foreach $Line (@{ $this->{'file-data'} })
		{
			# If the line matches...
			if ($Line =~ /^\s*$Key\s*=/)
			{
				# Cache the line.
				$Value = $Line;
	
				# Get the value.
				$Value =~ s/^\s*$Key\s*=(.*)$/$1/;
	
				# Clean off any carrage return.
				chomp($Value);
	
				# Bail out.
				last;
			}
		}
	}

	return $Value;
}

###############################################################################
# ConfigurationData::SetValueKeyed
###############################################################################

sub SetValueKeyed
{
	my ($this, $Key, $Value) = @_;
	my $Return;
	
	# If there is a file handle...
	if (defined($this->{'file-handle'}))
	{
		my $Line;
	
		# Run through the data...
		foreach $Line (@{ $this->{'file-data'} })
		{
			# If the line matches...
			if ($Line =~ /^\s*$Key\s*=/)
			{
				# Set the value.
				$Line =~ s/^(\s*$Key\s*=)(.*)$/$1$Value/;
	
				# Success.
				$Return = 0;
	
				# Bail out.
				last;
			}
		}
	
		# If the data is not yet set...
		unless (defined($Return))
		{
			# Build the line.
			$Line = "$Key=$Value\n";
	
			# Add the line to the resource.
			push(@{ $this->{'file-data'} }, $Line);

			# Success.
			$Return = 0;
		}
	}
	else
	{
		$Return = ENODATA;
	}

	return $Return;
}

###############################################################################
# ConfigurationData::DeleteValueKeyed
###############################################################################

sub DeleteValueKeyed
{
	my ($this, $Key)	= @_;
    my $Return			= ENODATA;
	
	# If there is a file handle...
	if (defined($this->{'file-handle'}))
	{
		my $i;
	
		# Run through the data...
		for $i ( 0 .. $#{ $this->{'file-data'} } ) 
		{
			# If the line matches...
			if ($this->{'file-data'}[$i] =~ /^\s*$Key\s*=/)
			{
				# Delete the line.
				splice(@{ $this->{'file-data'} }, $i, 1);
	
				# Success.
				$Return = 0;

				# Bail out.
				last;
			}
		}
	}

	return $Return;
}

###############################################################################
# ConfigurationData::GetDirectiveList
###############################################################################

sub GetDirectiveList
{
	my ($this, $Directive, $DirectiveListRef, $GroupHashRef) = @_;
	my $Return;

	# If there is a file handle...
	if (defined($this->{'file-handle'}))
	{
		# Validate the input parameter(s)...
		if ((defined($Directive)) &&
			(defined($DirectiveListRef)))
		{
			my @GroupDirectives;
			my $GroupDirective;
			my $LevelLocked;
			my $Identity;
			my $Value;
			my $Line;

			# Safe default(s).
			$GroupHashRef->{'directive'}	= '' unless(defined($GroupHashRef->{'directive'}));
			$GroupHashRef->{'identity'}		= '' unless(defined($GroupHashRef->{'identity'}));

			# Set the base level lock.
			$LevelLocked = ($GroupHashRef->{'directive'}) ? 0 : 1;

			# Set the return code.
			$Return = ENODATA;

			# Clean out the existing list.
			undef(@{ $DirectiveListRef });

			# Run through the data...
			foreach $Line (@{ $this->{'file-data'} })
			{
				# Look for group directives...
				if ((scalar(@GroupDirectives)) &&
					($GroupDirective = $GroupDirectives[scalar(@GroupDirectives) - 1]->{'directive'}) &&
					($Line =~ /^\s*$this->{'gdm'}->{'begin'}\/$GroupDirective$this->{'gdm'}->{'end'}\s*$/))
				{
					# Pop the group directive.
					pop(@GroupDirectives);
					
					# If there is a group binding...
					if ($GroupHashRef->{'directive'})
					{
						# If not on the bottom level...
						if (scalar(@GroupDirectives))
						{
							# Set the directive and identity.
							$GroupDirective = $GroupDirectives[scalar(@GroupDirectives) - 1]->{'directive'};
							$Identity		= $GroupDirectives[scalar(@GroupDirectives) - 1]->{'identity'};

							# Set the level lock...
							$LevelLocked = (($GroupHashRef->{'directive'} eq $GroupDirective) && 
											($GroupHashRef->{'identity'}  eq $Identity)) ? 1 : 0;
						}
						else
						{
							# Set the level lock.
							$LevelLocked = 0;
						}
					}
					else
					{
						# Set the level lock...
						$LevelLocked = (0 == scalar(@GroupDirectives)) ? 1 : 0;
					}
					
					next;
				}
				elsif ($Line =~ /^\s*$this->{'gdm'}->{'begin'}.+$this->{'gdm'}->{'end'}\s*$/)
				{
					# Get the group directive.
					$GroupDirective = $Line;
					$GroupDirective =~ s/^\s*$this->{'gdm'}->{'begin'}(.+?)(\s+.*)*$this->{'gdm'}->{'end'}\s*$/$1/;
					
					# If there is an identity...
					if ($Identity = $2)
					{
						# Set the identity.
						$Identity =~ s/^\s*(.+)\s*$/$1/;
					}
					else
					{
						# Safe default.
						$Identity = '';
					}
                    
					# If this is a group directive starter...
					if ($GroupDirective !~ /^\/\S+/)
					{
						# Push the group directive onto the pile.
						push(@GroupDirectives, { 'directive' => $GroupDirective, 'identity' => $Identity });
						
						# Set the level lock...
						$LevelLocked = (($GroupHashRef->{'directive'} eq $GroupDirective) && 
										($GroupHashRef->{'identity'}  eq $Identity)) ? 1 : 0;
						next;
					}
				}

				# If the line matches...
				if (($LevelLocked) &&
					($Line =~ /^\s*$Directive\s+\S+/))
				{
					# Get the value.
					$Value = $Line;
					$Value =~ s/^\s*$Directive\s+(.*)$/$1/;

					# Clean off any carrage return.
					chomp($Value);

					# Add the directive to the list.
					push(@{ $DirectiveListRef }, $Value);

					# Return success.
					$Return = 0;
				}
			}
		}
		else
		{
			$Return = EINVAL;
		}
	}
	else
	{
		$Return = ENODATA;
	}

	return $Return;
}

###############################################################################
# ConfigurationData::SetDirectiveList
###############################################################################

sub SetDirectiveList
{
	my ($this, $Directive, $DirectiveListRef, $GroupHashRef) = @_;
	my $Return;

	# If there is a file handle...
	if (defined($this->{'file-handle'}))
	{
		# Validate the input parameter(s)...
		if ((defined($Directive)) &&
			(defined($DirectiveListRef)))
		{
			my @GroupDirectives;
			my $GroupDirective;
			my $LevelLocked;
			my $Identity;
			my $Value;
			my $Line;
			my $i;
			my $j;

			# Safe default(s).
			$GroupHashRef->{'directive'}	= '' unless(defined($GroupHashRef->{'directive'}));
			$GroupHashRef->{'identity'}		= '' unless(defined($GroupHashRef->{'identity'}));

			# Set the base level lock.
			$LevelLocked = ($GroupHashRef->{'directive'}) ? 0 : 1;

			# Set the return code.
			$Return = 0;

			# Run through the data...
			for (my $i = 0; $i < scalar(@{ $this->{'file-data'} }); ++$i) 
			{
				# Set the current line.
				$Line = $this->{'file-data'}[$i];

				# Look for group directives...
				if ((scalar(@GroupDirectives)) &&
					($GroupDirective = $GroupDirectives[scalar(@GroupDirectives) - 1]->{'directive'}) &&
					($Line =~ /^\s*$this->{'gdm'}->{'begin'}\/$GroupDirective$this->{'gdm'}->{'end'}\s*$/))
				{
					my $CurrentLevel = $LevelLocked;
										
					# Pop the group directive.
					pop(@GroupDirectives);
					
					# If there is a group binding...
					if ($GroupHashRef->{'directive'})
					{
						# If not on the bottom level...
						if (scalar(@GroupDirectives))
						{
							# Set the directive and identity.
							$GroupDirective = $GroupDirectives[scalar(@GroupDirectives) - 1]->{'directive'};
							$Identity		= $GroupDirectives[scalar(@GroupDirectives) - 1]->{'identity'};

							# Set the level lock...
							$LevelLocked = (($GroupHashRef->{'directive'} eq $GroupDirective) && 
											($GroupHashRef->{'identity'}  eq $Identity)) ? 1 : 0;
						}
						else
						{
							# Set the level lock.
							$LevelLocked = 0;
						}
					}
					else
					{
						# Set the level lock.
						$LevelLocked = (0 == scalar(@GroupDirectives)) ? 1 : 0;
					}
					
					# See if the end of the group is here...
					if (($CurrentLevel) &&
						(!$LevelLocked))
					{
						# Run through the data...
						for ($j = 0; $j < scalar(@{ $DirectiveListRef }); ++$j) 
						{
							# Build the line.
							$Line = "$Directive $DirectiveListRef->[$j]\n";

							# Add the line to the resource.
							splice(@{ $this->{'file-data'} }, $i, 0, $Line);

							# Increment the index.
							++$i;
							
							# Delete the item.
							splice(@{ $DirectiveListRef }, $j, 1);
                            
							# Decrement the index.
							--$j;
						}
					}
					next;
				}
				elsif ($Line =~ /^\s*$this->{'gdm'}->{'begin'}.+$this->{'gdm'}->{'end'}\s*$/)
				{
					# Get the group directive.
					$GroupDirective = $Line;
					$GroupDirective =~ s/^\s*$this->{'gdm'}->{'begin'}(.+?)(\s+.*)*$this->{'gdm'}->{'end'}\s*$/$1/;
					
					# If there is an identity...
					if ($Identity = $2)
					{
						# Set the identity.
						$Identity =~ s/^\s*(.+)\s*$/$1/;
					}
					else
					{
						# Safe default.
						$Identity = '';
					}

					# If this is a group directive starter...
					if ($GroupDirective !~ /^\/\S+/)
					{
						# Push the group directive onto the pile.
						push(@GroupDirectives, { 'directive' => $GroupDirective, 'identity' => $Identity });
						
						# Set the level lock...
						$LevelLocked = (($GroupHashRef->{'directive'} eq $GroupDirective) && 
										($GroupHashRef->{'identity'}  eq $Identity)) ? 1 : 0;
						next;
					}
				}

				# If the line matches...
				if (($LevelLocked) &&
					($Line =~ /^\s*$Directive\s+\S+/))
				{
					# Get the value.
					$Value = $Line;
					$Value =~ s/^\s*$Directive\s+(.*)$/$1/;

					# Clean off any carrage return.
					chomp($Value);

					# Run through the data...
					for $j ( 0 .. $#{ $DirectiveListRef } )  
					{
						my $ValueA;
						my $ValueB;

						# Cache the values.
						$ValueA =  $DirectiveListRef->[$j];
						$ValueB =  $Value;

						# Look for the need to match loosely...
						if ((defined($this->{'loose-match'})) &&
							(1   ==  $this->{'loose-match'}))
						{
							# Remove the white space.
							$ValueA =~ s/\s//g;
							$ValueB =~ s/\s//g;
						}

						# If the line matches...
						if ($ValueA eq $ValueB)
						{
							# Delete the item.
							splice(@{ $DirectiveListRef }, $j, 1);

							# Decrement the index.
							--$j;
						}
					}
				}
			}

			# If there is data...
			if (scalar(@{ $DirectiveListRef }))
			{
				# If there is a directive...
				if ($GroupHashRef->{'directive'})
				{
					# Build the line.
					$Line = "$this->{'gdm'}->{'begin'}$GroupHashRef->{'directive'}";
					$Line .= " $GroupHashRef->{'identity'}" if($GroupHashRef->{'identity'});
					$Line .= "$this->{'gdm'}->{'end'}\n";

					# Add the line to the resource.
					push(@{ $this->{'file-data'} }, $Line);
				}

				# Run through the data...
				for $j ( 0 .. $#{ $DirectiveListRef } ) 
				{
					# Build the line.
					$Line = "$Directive $DirectiveListRef->[$j]\n";
					
					# Add the line to the resource.
					push(@{ $this->{'file-data'} }, $Line);
				}
				
				# If there is a directive...
				if ($GroupHashRef->{'directive'})
				{
					# Build the line.
					$Line = "$this->{'gdm'}->{'begin'}/$GroupHashRef->{'directive'}$this->{'gdm'}->{'end'}\n";

					# Add the line to the resource.
					push(@{ $this->{'file-data'} }, $Line);
				}
			}
		}
		else
		{
			$Return = EINVAL;
		}
	}
	else
	{
		$Return = ENODATA;
	}

	return $Return;
}

###############################################################################
# ConfigurationData::DeleteDirectiveList
###############################################################################

sub DeleteDirectiveList
{
	my ($this, $Directive, $DirectiveListRef, $GroupHashRef) = @_;
	my $Return;

	# If there is a file handle...
	if (defined($this->{'file-handle'}))
	{
		# Validate the input parameter(s)...
		if (defined($Directive))
		{
			my @GroupDirectives;
			my $GroupDirective;
			my $LevelLocked;
			my $Identity;
			my $Value;
			my $Line;
			my $j;

			# Check for the proper stack alignment...
			if ((defined($DirectiveListRef)) &&
				('HASH' eq ref($DirectiveListRef)))
			{
				# Re-align the stack.
				$GroupHashRef = $DirectiveListRef;
				undef($DirectiveListRef);
			}

			# Safe default(s).
			$GroupHashRef->{'directive'}	= '' unless(defined($GroupHashRef->{'directive'}));
			$GroupHashRef->{'identity'}		= '' unless(defined($GroupHashRef->{'identity'}));

			# Set the base level lock.
			$LevelLocked = ($GroupHashRef->{'directive'}) ? 0 : 1;

			# Set the return code.
			$Return = 0;

			# Run through the data...
			for (my $i = 0; $i < scalar(@{ $this->{'file-data'} }); ++$i) 
			{
				# Set the current line.
				$Line = $this->{'file-data'}[$i];

				# Look for group directives...
				if ((scalar(@GroupDirectives)) &&
					($GroupDirective = $GroupDirectives[scalar(@GroupDirectives) - 1]->{'directive'}) &&
					($Line =~ /^\s*$this->{'gdm'}->{'begin'}\/$GroupDirective$this->{'gdm'}->{'end'}\s*$/))
				{
					# Pop the group directive.
					pop(@GroupDirectives);
					
					# If there is a group binding...
					if ($GroupHashRef->{'directive'})
					{
						# If not on the bottom level...
						if (scalar(@GroupDirectives))
						{
							# Set the directive and identity.
							$GroupDirective = $GroupDirectives[scalar(@GroupDirectives) - 1]->{'directive'};
							$Identity		= $GroupDirectives[scalar(@GroupDirectives) - 1]->{'identity'};

							# Set the level lock...
							$LevelLocked = (($GroupHashRef->{'directive'} eq $GroupDirective) && 
											($GroupHashRef->{'identity'}  eq $Identity)) ? 1 : 0;
						}
						else
						{
							# Set the level lock.
							$LevelLocked = 0;
						}
					}
					else
					{
						# Set the level lock...
						$LevelLocked = (0 == scalar(@GroupDirectives)) ? 1 : 0;
					}
					
					next;
				}
				elsif ($Line =~ /^\s*$this->{'gdm'}->{'begin'}.+$this->{'gdm'}->{'end'}\s*$/)
				{
					# Get the group directive.
					$GroupDirective = $Line;
					$GroupDirective =~ s/^\s*$this->{'gdm'}->{'begin'}(.+?)(\s+.*)*$this->{'gdm'}->{'end'}\s*$/$1/;
					
					# If there is an identity...
					if ($Identity = $2)
					{
						# Set the identity.
						$Identity =~ s/^\s*(.+)\s*$/$1/;
					}
					else
					{
						# Safe default.
						$Identity = '';
					}
                    
					# If this is a group directive starter...
					if ($GroupDirective !~ /^\/\S+/)
					{
						# Push the group directive onto the pile.
						push(@GroupDirectives, { 'directive' => $GroupDirective, 'identity' => $Identity });
						
						# Set the level lock...
						$LevelLocked = (($GroupHashRef->{'directive'} eq $GroupDirective) && 
										($GroupHashRef->{'identity'}  eq $Identity)) ? 1 : 0;
						next;
					}
				}

				# If the line matches...
				if (($LevelLocked) &&
					($Line =~ /^\s*$Directive\s+\S+/))
				{
					# Get the value.
					$Value = $Line;
					$Value =~ s/^\s*$Directive\s+(.*)$/$1/;

					# Clean off any carrage return.
					chomp($Value);

					# If there is a list...
					if (defined($DirectiveListRef))
					{
						# Run through the data...
						for $j ( 0 .. $#{ $DirectiveListRef } )  
						{
							my $ValueA;
							my $ValueB;

							# Cache the values.
							$ValueA =  $DirectiveListRef->[$j];
							$ValueB =  $Value;
							
							# Look for the need to match loosely...
							if ((defined($this->{'loose-match'})) &&
								(1   ==  $this->{'loose-match'}))
							{
								# Remove the white space.
								$ValueA =~ s/\s//g;
								$ValueB =~ s/\s//g;
							}

							# If the line matches...
							if ($ValueA eq $ValueB)
							{
								# Delete the line.
								splice(@{ $this->{'file-data'} }, $i, 1);

								# Move the counter back one.
								--$i;

								# Bail out.
								last;
							}
						}
					}
					else
					{
						# Delete the line.
						splice(@{ $this->{'file-data'} }, $i, 1);

						# Move the counter back one.
						--$i;
					}
				}
			}
		}
		else
		{
			$Return = EINVAL;
		}
	}
	else
	{
		$Return = ENODATA;
	}

	return $Return;
}

###############################################################################
# ConfigurationData::ValidateDirectiveValue
###############################################################################

sub ValidateDirectiveValue
{
	my ($this, $Directive, $DirectiveValue, $GroupHashRef)	= @_;
	my @DirectiveValues;
	my $Return = 0;

	# Get the directive list...
	if (0 == $this->GetDirectiveList($Directive, \@DirectiveValues, $GroupHashRef))
	{
		my $Item;

		# Run through the list...
		foreach $Item (@DirectiveValues)
		{
			# Look for a match...
			if ($Item eq $DirectiveValue)
			{
				# Signal a match.
				$Return = 1;
				last;
			}
		}
	}
	
	return $Return;
}

1;

__END__
