package CookieData;

###############################################################################
#                                                                             #
# Filename: CookieData.pm                                                     #
#                                                                             #
# Description: Contains script needed to manage cookie data.                  #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Safe;
use FileHandle;
use DB_File;
use Errno qw(/.*/);
use Cwd;

require Exporter;

###############################################################################
# CookieData::import
###############################################################################

sub import
{
	my $Caller	= caller;

	eval
	{
		require Fcntl;

		# Try to export the fnctl symbols...
		Exporter::export('Fcntl', $Caller);
	};

	eval
	{
		require Errno;
	
		# Try to export the Errno symbols...
		Exporter::export('Errno', $Caller, qw(/.*/));
	};
}

###############################################################################
# CookieData::new
###############################################################################

sub new
{
	my ($class, $LifeSpan) = @_;

	# Initialize the object.
	my $this = 
	{ 
		'life-span'	=> ((defined($LifeSpan)) && ('' ne $LifeSpan)) ? ($LifeSpan * 3600) : 3600,
	};

	# Align the object with this package.
	bless($this, $class);
	
	# Return the object.
	return $this;
}

###############################################################################
# CookieData::DESTROY
###############################################################################

sub DESTROY
{
	my $this = shift;

	# Clean up any objects in use.
	$this->Detach();
}

###############################################################################
# CookieData::Attach
###############################################################################

sub Attach
{
	my ($this, $CookiePath) = @_;
	my $Return;

	# Validate the input parameter(s)...
	if (defined($CookiePath))
	{
		# Detach from any previous bindings.
		$this->Detach();

		# Add on the current woking directory if necessary...
		$CookiePath = cwd() . '/'. $CookiePath if($CookiePath !~ /\//);

		# Tie the data...
		if (tie(%{ $this->{'cookie-data'} }, 'DB_File', $CookiePath))
		{
			# Set the new cookie path.
			$this->{'cookie-path'} = $CookiePath;

			# If there is an expiration...
			if ($this->{'life-span'})
			{
				my $CronRootPath	= '/etc/cron.hourly';
				my $CronJobFile 	= $CookiePath;

				$| = 1;

				# Build the cron job file name.
				$CronJobFile =~ s/^(.*\/)*(.*)$/$2/;
				
				# Try to open the cron job...
				if (open(CRONJOB, "> $CronRootPath/$CronJobFile"))
				{
					# Spew out some code.
					print CRONJOB <<HERE;
#!/usr/bin/perl

# If the cookie exists.
if ( -e '$CookiePath' )
{
	# If the cookie has expired...
	if (((stat('$CookiePath'))[8] + $this->{'life-span'}) < time())
	{
		# Delete the cookie.
		unlink('$CookiePath');

		# Delete the cron job.
		unlink('$CronRootPath/$CronJobFile');
	}
}
else
{
	# Delete the cron job.
	unlink('$CronRootPath/$CronJobFile');
}
HERE

					# Close the file.
					close(CRONJOB);
	
					# Set the permissions.
					Safe::system("/bin/chmod 750 $CronRootPath/$CronJobFile");
				}
			}

			$Return = 0;
		}
		else
		{
			$Return = EINVAL;
		}
	}
	else
	{
		$Return = EINVAL;
	}
	
	return $Return;
}

###############################################################################
# CookieData::Detach
###############################################################################

sub Detach
{
	my $this = shift;

	# Untie the data.
	untie(%{ $this->{'cookie-data'} });
	
	# Undefine the data.
	undef($this->{'cookie-data'});

	return 0;
}

###############################################################################
# CookieData::Delete
###############################################################################

sub Delete
{
	my $this = shift;

	# Detach from the file.
	$this->Detach();

	# If there is a cookie path...
	if (defined($this->{'cookie-path'}))
	{
		my $CronRootPath	= '/etc/cron.hourly';
		my $CronJobFile 	= $this->{'cookie-path'};
		
		# Build the cron job file name.
		$CronJobFile =~ s/^(.*\/)*(.*)$/$2/;
		$CronJobFile = $CronRootPath . '/'. $CronJobFile;

		# Delete the cron job and cookie file.
		unlink($CronJobFile);
		unlink($this->{'cookie-path'});

		# Undefine the path.
		undef($this->{'cookie-path'});
	}

	return 0;
}

###############################################################################
# CookieData::GetValueKeyed
###############################################################################

sub GetValueKeyed
{
	my ($this, $Key) = @_;

	# Validate the input parameter(s)...
	if (defined($Key))
	{
		# Return the data...
		return $this->{'cookie-data'}->{$Key};
	}

	return undef;
}

###############################################################################
# CookieData::SetValueKeyed
###############################################################################

sub SetValueKeyed
{
	my ($this, $Key, $Value) = @_;

	# Validate the input parameter(s)...
	if (defined($Key))
	{
		# Return the data...
		if (defined($Value))
		{
			# Set the value.
			$this->{'cookie-data'}->{$Key} = $Value;
		}
		else
		{
			# Delete the key.
			delete($this->{'cookie-data'}->{$Key});
		}
	}

	return 0;
}

1;

__END__
