#!/usr/bin/perl -- # vim:ts=4
package EmergencySync;

###############################################################################
#                                                                             #
# Filename: EmergencySync.pm                                                  #
#                                                                             #
# Description: Synchronize relevant passwords and network settings on the     #
#              emergency boot partition.                                      #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use IO::File;
use File::Copy;
use Safe;

######################################################################
# global settings
######################################################################

use vars qw($mount_point $lockfile_name);
$mount_point = "/emergency";
$lockfile_name = "/var/lock/emergency.lock";

use vars qw(@network_sync_files);
@network_sync_files = (	'/etc/sysconfig/clock',
						'/etc/sysconfig/network',
						'/etc/sysconfig/network-scripts/ifcfg-eth0',
						'/etc/sysconfig/network-scripts/ifcfg-eth1',
						'/etc/resolv.conf',
						'/etc/ssh/ssh_host_key',
						'/etc/ssh/ssh_host_key.pub',
						'/etc/ssh/ssh_host_dsa_key',
						'/etc/ssh/ssh_host_dsa_key.pub',
						'/etc/ssh/ssh_config',
						'/etc/ssh/sshd_config',
);

use vars qw($passwd_sync_users);
$passwd_sync_users = { 'root' => 0 };

######################################################################
# global variables
######################################################################

use vars qw($lockfile_handle);
$lockfile_handle = undef;

######################################################################
# test routine
######################################################################

#&testroutine4;

sub testroutine {

	print "mount();\n";
	EmergencySync::mount();
	print "sleep 5;\n";
	sleep(5);
	print "unmount();\n";
	EmergencySync::unmount();
	print "sleep 5;\n";
	sleep(5);

	print "mount();\n";
	EmergencySync::mount();
	print "sleep 5;\n";
	sleep(5);
	print "unmount();\n";
	EmergencySync::unmount();
	print "sleep 5;\n";
	sleep(5);

}

sub testroutine2 {
	EmergencySync::sync_network();
}

sub testroutine3 {
	EmergencySync::sync_passwd();
}

sub testroutine4 {
	EmergencySync::sync_all();
}

######################################################################
# interfaces
######################################################################

#
# &sync_network
#
# mounts the emergency partition, syncs the network files, and
# unmounts.
#

sub sync_network {
	# mount the emergency partition
	EmergencySync::mount();

	&sync_network_files();

	# unmount the emergency partition
	EmergencySync::unmount();
}

#
# &sync_passwd
#
# mounts the emergency partition, syncs the passwd and shadow files,
# and unmounts.
#

sub sync_passwd {
	# mount the emergency partition
	EmergencySync::mount();

	&sync_passwd_users();

	# unmount the emergency partition
	EmergencySync::unmount();
}

#
# &sync_all
#
# syncs everything
#

sub sync_all {
	# mount the emergency partition
	EmergencySync::mount();

	&sync_network_files();
	&sync_passwd_users();

	# unmount the emergency partition
	EmergencySync::unmount();
}

######################################################################
# supporting subroutines
######################################################################

#
# &sync_network_files
#
# copy network configuration files to the emergency partition
#
# (assumes the emergency partition has already been mounted.)
#

sub sync_network_files {
	my $i;
	foreach $i (@network_sync_files) {
		# declare the source and destination filenames
		my $source_file = $i;
		my $dest_file   = "$mount_point/$i";

		# Unsuccessful copies are ignored -- some of the sync files
		# (i.e. ssh keys) may not exist, and that is okay.
		File::Copy::copy($source_file,$dest_file);

		# If a successful copy was actually made, make sure the
		# resulting destination file has the same permissions
		# as the source file.
		if ( (-f $source_file) && (-f $dest_file) ) {
			# get the mode of the source file
			my $mode = (stat($source_file))[2];
			# strip off the "file type" bits
			$mode = $mode & 07777;
			# set the destination file to the correct mode
			chmod $mode, $dest_file;
		}
	}
}

#
# &sync_passwd_users
#
# Updates or adds users to the /etc/passwd and /etc/shadow files on
# the emergency partition to keep it in sync with the production
# partition.  This subroutine only syncs users which are either
# specified in the $passwd_sync_users hash, or have group id 10000
# designating them as a server admin.
#
# (assumes the emergency partition has already been mounted.)

sub sync_passwd_users {
	# hash of user information to be transfered
	my $relevant_users = {};

	# store a list of all passwd usernames and all shadow usernames.
	# this will be used to delete users from the emergency passwd/shadow
	# who no longer exist in the production passwd/shadow.
	my $all_passwd_users = {};
	my $all_shadow_users = {};

	############################################################
	# read in relevant data from the production passwd/shadow
	############################################################

	# open input files
	my $p_passwd_in = new IO::File "/etc/passwd";
	my $p_shadow_in = new IO::File "/etc/shadow";

	####################
	# /etc/passwd
	####################

	while (<$p_passwd_in>) {
		chomp;

		# determine the username of this entry.
		my $username = ((split(/:/))[0]);
		my $uid = ((split(/:/))[2]);
		my $gid = ((split(/:/))[3]);

		# add this user to the all_passwd_users list.
		$all_passwd_users->{$username} = {'uid' => $uid, 'gid' => $gid};

		# store this user's data if it is a sync user or a server admin
		if (	(defined $username
				&& exists ($passwd_sync_users->{$username}))
			||	(defined $gid
				&& ($gid == 10000)) ) {
			$relevant_users->{$username}->{passwd} = $_;
		}
	}

	####################
	# /etc/shadow
	####################

	# scan shadow
	while (<$p_shadow_in>) {
		chomp;

		# determine the username of this entry.
		my $username = ((split(/:/))[0]);

		# skip if this entry has no corresponding entry in passwd
		if (! exists $all_passwd_users->{$username}) {
			next;
		}

		# add this user to the all_shadow_users list.
		$all_shadow_users->{$username} = 0;

		# store this user's data if it is a sync user.
		if (	(defined $username
				&& exists ($passwd_sync_users->{$username}))
			||	(exists $relevant_users->{$username}
				&& exists $relevant_users->{$username}->{passwd})) {
			$relevant_users->{$username}->{shadow} = $_;
		}
	}

	# close files
	$p_passwd_in->close;
	$p_shadow_in->close;

	############################################################
	# write relevant data to the emergency passwd/shadow
	############################################################

	# determine temporary names
	my $temp_passwd_name = "$mount_point/etc/passwd.$$";
	my $temp_shadow_name = "$mount_point/etc/shadow.$$";

	# open input files
	my $e_passwd_in = new IO::File "$mount_point/etc/passwd";
	my $e_shadow_in = new IO::File "$mount_point/etc/shadow";

	# open output files
	my $e_passwd_out = new IO::File "> $temp_passwd_name";
	my $e_shadow_out = new IO::File "> $temp_shadow_name";

	####################
	# /etc/passwd
	####################

	while (<$e_passwd_in>) {
		chomp;

		# don't trust $_ -- store it.
		my $line = $_;

		# extract the username of this entry.
		my $username = ((split(/:/))[0]);

		# is this username in our list of relevant users to update?
		if (	defined $username
				&& exists $relevant_users->{$username}
				&& exists ($relevant_users->{$username}->{passwd})	) {
			# if so, replace this entry
			print $e_passwd_out $relevant_users->{$username}->{passwd},"\n";
			delete $relevant_users->{$username}->{passwd};
		} elsif (exists $all_passwd_users->{$username}) {
			# otherwise, keep the old entry, if it was in the production passwd,
			# and if it meets the criteria to be retained:
			#   1. the username exists in the defined passwd_sync_users list
			#   2. the user is a system user (i.e. uid less than 100)
			#   3. the user is a bona fide system admin (gid is 10000)
			if ( (exists $passwd_sync_users->{$username}) ||
				 ($all_passwd_users->{$username}->{'uid'} < 100) ||
				 ($all_passwd_users->{$username}->{'gid'} == 10000) ) {
				print $e_passwd_out $line,"\n";
			}
		}
	}

	# add users who were not in the original passwd file at all

	foreach my $i (keys %$relevant_users) {
		if (exists $relevant_users->{$i}->{passwd}) {
			print $e_passwd_out $relevant_users->{$i}->{passwd},"\n";
			delete $relevant_users->{$i}->{passwd};
		}
	}

	####################
	# /etc/shadow
	####################

	while (<$e_shadow_in>) {
		chomp;

		# don't trust $_ -- store it.
		my $line = $_;

		# extract the username of this entry.
		my $username = ((split(/:/))[0]);

		# is this username in our list of relevant users to update?
		if (	defined $username
				&& exists $relevant_users->{$username}
				&& exists ($relevant_users->{$username}->{shadow})	) {
			# if so, replace this entry
			print $e_shadow_out $relevant_users->{$username}->{shadow},"\n";
			delete $relevant_users->{$username}->{shadow};
		} elsif (exists $all_shadow_users->{$username}) {
			# otherwise, keep the old entry, if it was in the production passwd,
			# and if it meets the criteria to be retained:
			#   1. the username exists in the defined passwd_sync_users list
			#   2. the user is a system user (i.e. uid less than 100)
			#   3. the user is a bona fide system admin (gid is 10000)
			if ( (exists $passwd_sync_users->{$username}) ||
				 ($all_passwd_users->{$username}->{'uid'} < 100) ||
				 ($all_passwd_users->{$username}->{'gid'} == 10000) ) {
				print $e_shadow_out $line,"\n";
			}
		}
	}

	# add users who were not in the original shadow file at all

	foreach my $i (keys %$relevant_users) {
		if (exists $relevant_users->{$i}->{shadow}) {
			print $e_shadow_out $relevant_users->{$i}->{shadow},"\n";
			delete $relevant_users->{$i}->{shadow};
		}
	}

	# close files
	$e_passwd_in->close;
	$e_shadow_in->close;
	$e_passwd_out->close;
	$e_shadow_out->close;

	# make a backup copy of the originals
	File::Copy::copy("$mount_point/etc/passwd","$mount_point/etc/passwd-");
	File::Copy::copy("$mount_point/etc/shadow","$mount_point/etc/shadow-");
	# replace the original files with the new files
	rename($temp_passwd_name,"$mount_point/etc/passwd");
	rename($temp_shadow_name,"$mount_point/etc/shadow");
	# assure the shadow files have the right permissions
	chmod 0400, "$mount_point/etc/shadow", "$mount_point/etc/shadow-";

	############################################################
	# Make a backup of the production passwd/shadow to store in
	# a safe place on the emergency partition.
	############################################################

	File::Copy::copy("/etc/passwd","$mount_point/etc/passwd.full");
	File::Copy::copy("/etc/shadow","$mount_point/etc/shadow.full");
	# assure the shadow file has the right permissions
	chmod 0400, "$mount_point/etc/shadow.full";


}


#
# &mount
#
# mount the emergency partition
#

sub mount {
	EmergencySync::lock();

	## The emergency partition is always mounted, so
	## this line is not needed presently.
	##
	## Safe::system("/bin/mount $mount_point");
	##
}

#
# &unmount
#
# unmount the emergency partition
#

sub unmount {

	## The emergency partition is always mounted, so
	## this line is not needed presently.
	##
	## Safe::system("/bin/umount $mount_point");
	##

	EmergencySync::unlock();
}

#
# &lock
#
# Open a lock file for append (creating it if needed) and
# try to get a flock() on it.
#

sub lock {
	$lockfile_handle = new IO::File;
	$lockfile_handle->open(">> $lockfile_name");
	flock($lockfile_handle,2);
}

#
# &unlock
#
# Close the file handle to the lock file, thereby
# releasing the lock.  We DO NOT try to unlink the file,
# since that would raise concurrency issues.
#

sub unlock {
	$lockfile_handle->close;
	undef $lockfile_handle;
}

1;

