package LanguageResource;

###############################################################################
#                                                                             #
# Filename: LanguageResource.pm                                               #
#                                                                             #
# Description: Contains script needed to manage resource data.                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ConfigurationData;

###############################################################################
# LanguageResource::new
###############################################################################

sub new
{
	my ($class, $PreferredLanguage, $RootDirectory, $SuffixListRef) = @_;

	# Initialize the object.
	my $this = 
	{
		'preferred-language'	=> $PreferredLanguage,
		'root-directory' 		=> $RootDirectory,
		'suffix-list'			=> $SuffixListRef,
	};

	# Safe default(s).
	$this->{'preferred-language'}	= 'en' unless(defined($this->{'preferred-language'}));
	$this->{'root-directory'}		= './' unless(defined($this->{'root-directory'}));

	# Align the object with this package.
	bless($this, $class);
	
	# Distill the language code.
	$this->{'preferred-language'}	= $this->DistillLanguageCode($this->{'preferred-language'});

	# Polish the root directory.
	$this->{'root-directory'}		= $this->PolishDirectory($this->{'root-directory'});

	# Return the object.
	return $this;
}

###############################################################################
# LanguageResource::DESTROY
###############################################################################

sub DESTROY
{
	my $this = shift;

	# Clean up any objects in use.
	$this->Detach();
}

###############################################################################
# LanguageResource::Attach
###############################################################################

sub Attach
{
	my $this = shift;

	# Detach from any previous bindings.
	$this->Detach();

	# If there is a suffix list...
    if (defined($this->{'suffix-list'}))
	{
		my $ResourceSuffix;
		my $ResourcePath;

        # Run through each of the resource suffixes...
		foreach $ResourceSuffix (@{ $this->{'suffix-list'} })
		{
			# Acquire a resource path...
			if (defined($ResourcePath = $this->ResolveResourceFile($ResourceSuffix)))
			{
				my $Resource = new ConfigurationData;

				# Attach to the resource...
				if (0 == $Resource->Attach($ResourcePath))
				{
					# Add the object to the resource list.
					push(@{ $this->{'resource-list'} }, $Resource);
				}
			}
		}
	}
}

###############################################################################
# LanguageResource::Detach
###############################################################################

sub Detach
{
	my $this		= shift;
	my $Resource;
	
	# Run though the resource list...
	while ((defined($this->{'resource-list'})) &&
		   (defined($Resource = pop(@{ $this->{'resource-list'} }))))
	{
		# Detach the object.
		$Resource->Detach(1);

		# Delete the object.
		undef($Resource);
	}

	# Undefine the list...
	undef($this->{'resource-list'});
}

###############################################################################
# LanguageResource::GetPreferredLanguage
###############################################################################

sub GetPreferredLanguage
{
	my $this = shift;

	# Return the requested data.
	return $this->{'preferred-language'};
}

###############################################################################
# LanguageResource::SetPreferredLanguage
###############################################################################

sub SetPreferredLanguage
{
	my ($this, $PreferredLanguage) = @_;

	# Distill the language code.
	$PreferredLanguage = $this->DistillLanguageCode($PreferredLanguage);
    
    # Check to see if the preferred language is not already in use...
	if ((!defined($PreferredLanguage)) ||
		($PreferredLanguage ne $this->{'preferred-language'}))
	{
		# Detach from the current resource list.
		$this->Detach();
		
		# Assume the new value provided.
		$this->{'preferred-language'} = $PreferredLanguage;
	}
}

###############################################################################
# LanguageResource::GetRootDirectory
###############################################################################

sub GetRootDirectory
{
	my $this = shift;

	# Return the requested data.
	return $this->{'root-directory'};
}

###############################################################################
# LanguageResource::SetRootDirectory
###############################################################################

sub SetRootDirectory
{
	my ($this, $RootDirectory) = @_;
	
	# Polish the root directory.
	$RootDirectory = $this->PolishDirectory($RootDirectory);

	# Check to see if the preferred language is not already in use...
	if ((!defined($RootDirectory)) ||
		($RootDirectory ne $this->{'root-directory'}))
	{
		# Detach from the current resource list.
		$this->Detach();
		
		# Assume the new value provided.
        $this->{'root-directory'} = $RootDirectory;
	}
}

###############################################################################
# LanguageResource::GetSuffixList
###############################################################################

sub GetSuffixList
{
	my $this = shift;

	# Return the requested data.
	return @{ $this->{'suffix-list'} };
}

###############################################################################
# LanguageResource::SetSuffixList
###############################################################################

sub SetSuffixList
{
	my ($this, @SuffixList) = @_;

	# Detach from the current resource list.
	$this->Detach();

	# Assume the new value provided.
	$this->{'suffix-list'} = [ @SuffixList ];
}

###############################################################################
# LanguageResource::GetString
###############################################################################

sub GetString
{
	# Call the equivalent function.
	return _S(@_);
}

###############################################################################
# LanguageResource::GetStrings
###############################################################################

sub GetStrings
{
	my ($this, $StringHashRef, $Default) = @_;

	# Validate the input parameter(s)...
	if (defined($StringHashRef))
	{
		my $Key;

		# Run through the hash...
		foreach $Key (keys(%$StringHashRef))
		{
			# Try to get the string.
			$StringHashRef->{$Key} = $this->_S($Key, $Default);
		}
	}
}

###############################################################################
# LanguageResource::_S
###############################################################################

sub _S
{
	my ($this, $Key, $ParameterListRef, $Default) = @_;
	my $Value;

	# Validate the input parameter(s)...
	if (defined($Key))
	{
		# Safe default.
		$Default = '' unless(defined($Default));

		# Update the default...
		$Default = $ParameterListRef if((defined($ParameterListRef)) && ('SCALAR' eq ref($ParameterListRef)));
		
		# Make sure the object is primed...
		unless (defined($this->{'resource-list'}))
		{
			# Attach to the resource objects.
			$this->Attach();
		}

		# If there are any resource objects...
		if (defined($this->{'resource-list'}))
		{
			my $i;

			# Run through the resource list...
			for $i ( 0 .. $#{ $this->{'resource-list'} })
			{
				# Look for the value...
				if (defined($Value = $this->{'resource-list'}[$i]->GetValueKeyed($Key)))
				{
					# Bail out.
					last;
				}
			}
		}

		# Apply the default string if necessary...
		$Value = $Default unless(defined($Value));
			
		# If there is a parameter list...
		if ((defined($ParameterListRef)) &&
			('ARRAY' eq ref($ParameterListRef)))
		{
			my $Parameter;
			my $Key;
			my $i			= 1;
	
			# Run through all of the parameters.
			foreach $Parameter (@{ $ParameterListRef })
			{
				# Build the key.
				$Key = '%' . $i;
	
				# Make the substitution.
				$Value =~ s/$Key/$Parameter/;
	
				# Increment the counter.
				++$i;
			}
		}
	}

	return $Value;
}

###############################################################################
# LanguageResource::DistillLanguageCode
###############################################################################

sub DistillLanguageCode
{
	my ($this, $LanguageCode) = @_;

	# Validate the input parameter(s)...
	if (defined($LanguageCode))
	{
		# Get the purified language code.
		$LanguageCode = (split(/-/, $LanguageCode))[0];
	}

	return $LanguageCode;
}

###############################################################################
# LanguageResource::PolishDirectory
###############################################################################

sub PolishDirectory
{
	my ($this, $Directory) = @_;
	
	# Validate the input parameter(s)...
	if (defined($Directory))
	{
		# If the directory does not have an ending slash...
		unless ($Directory =~ /^.*\/$/)
		{
			# Append the missing slash.
			$Directory .= '/';
		}
	}

	return $Directory;
}

###############################################################################
# LanguageResource::ResolveResourceFile
###############################################################################

sub ResolveResourceFile
{
	my ($this, $ResourceSuffix) = @_;
	my $ResourceFile;

	# Make sure the object is defined...
    if ((defined($this->{'root-directory'})) &&
		(defined($this->{'preferred-language'})))
	{
		# If the preferred resource exists...
		if ( -e $this->{'root-directory'} . $this->{'preferred-language'} . $ResourceSuffix)
		{
			# Set the resource path.
			$ResourceFile = $this->{'root-directory'} . $this->{'preferred-language'} . $ResourceSuffix;
		}
		elsif ( -e $this->{'root-directory'} . 'en' . $ResourceSuffix)
		{
			# Set the resource path.
			$ResourceFile = $this->{'root-directory'} . 'en' . $ResourceSuffix;
		}
	}

	return $ResourceFile;
}

1;

__END__
