package MonSerializer;

###############################################################################
#                                                                             #
# Filename: MonSerializer.pm                                                  #
#                                                                             #
# Description: Strategy algorithmic class for a monitor serializer.           #
#                                                                             #
# Pattern(s): Strategy, Factory, Iterator                                     #
#                                                                             #
# Aggregates: MonConfiguration                                                #
#                                                                             #
# References: MonAlertFactory, MonAlertTolerance, MonHostGroup, MonMonitor,   #
#             MonPeriod, MonService, MonWatch                                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use MonAlertFactory;
use MonAlertTolerance;
use MonHostGroup;
use MonMonitor;
use MonPeriod;
use MonWatch;
use Daemon;

###############################################################################
# MonSerializer::new
###############################################################################

sub new
{
	my ($class, $ConfigurationFile) = @_;

	# Initialize the object.
	my $this = 
	{
		'configuration'			=> undef,
		'configuration-file'	=> undef,
		'configuration-data'	=> undef,
	};

	# Align the object with this package.
    bless($this, $class);

	# Set the configuration file.
	$this->setConfigurationFile($ConfigurationFile);

	# Return the object.
	return $this;
}

###############################################################################
# MonSerializer::getHost
###############################################################################

sub getHost
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'configuration'};
}

###############################################################################
# MonSerializer::setHost
###############################################################################

sub setHost
{
	my ($this, $Host) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($Host->isa('MonConfiguration'))
	{
		# Cache the object reference.
		$this->{'configuration'} = $Host;
	}
	else
	{
		$Return = 20004001;
	}

	return $Return;
}

###############################################################################
# MonSerializer::getConfigurationFile
###############################################################################

sub getConfigurationFile
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'configuration-file'};
}

###############################################################################
# MonSerializer::setConfigurationFile
###############################################################################

sub setConfigurationFile
{
	my ($this, $ConfigurationFile) = @_;
	
	# Set the data.
	$this->{'configuration-file'} = $ConfigurationFile;

	return 0;
}

###############################################################################
# MonSerializer::load
###############################################################################

sub load
{
	my $this	= shift;
	my $Return	= 0;

	# Try to open the file...
	if (open(CONFIGURATION, "< $this->{'configuration-file'}"))
	{
		# Try to lock the file...
		if (flock(CONFIGURATION, 2))
		{
			# Read in the file.
			$this->{'configuration-data'} = [ <CONFIGURATION> ];

			# Unlock the file.
			flock(CONFIGURATION, 8);

			# Build the object hierarchy.
			$Return = $this->build();
		}
		else
		{
			$Return = 20004002;
		}
		
		# Close the file.
		close(CONFIGURATION);
	}

	return $Return;
}

###############################################################################
# MonSerializer::save
###############################################################################

sub save
{
	my $this	= shift;
	my $Return	= 0;

	# Flatten the object hierarchy...
	if ((0 == ($Return = $this->flatten())) &&
		($this->{'configuration-data'}))
	{
		# Try to open the file...
		if (open(CONFIGURATION, "> $this->{'configuration-file'}"))
		{
			# Try to lock the file...
			if (flock(CONFIGURATION, 2))
			{
				my $DaemonMon	= new Daemon('mon');

				# Write out the file.
				print CONFIGURATION @{ $this->{'configuration-data'} };

				# Unlock the file.
				flock(CONFIGURATION, 8);

				# Restart the daemon.
				$DaemonMon->Control(2);
			}
			else
			{
				$Return = 20004003;
			}

			# Close the file.
			close(CONFIGURATION);
		}
		else
		{
			$Return = 20004004;
		}

		# Clean out the data...
		$this->{'configuration-data'} = undef if(0 == $Return);
	}

	return $Return;
}

###############################################################################
# MonSerializer::build
###############################################################################

sub build
{
	my $this	= shift;
	my $Return	= 0;

	# If there is a host configuration reference and 
	# any configuration data...
	if ((defined($this->{'configuration'})) &&
		(defined($this->{'configuration-data'})))
	{
		my %FunctionHash	= ( 'alertdir'				    => [ [ 'root' ], 'assignment',				\&parseALERTDIR				],
								'mondir'				    => [ [ 'root' ], 'assignment',				\&parseMONDIR				],
								'statedir'				    => [ [ 'root' ], 'assignment',				\&parseSTATEDIR				],
								'logdir'				    => [ [ 'root' ], 'assignment',				\&parseLOGDIR				],
								'basedir'				    => [ [ 'root' ], 'assignment',				\&parseBASEDIR				],
								'cfbasedir'				    => [ [ 'root' ], 'assignment',				\&parseCFBASEDIR			],
								'authfile'				    => [ [ 'root' ], 'assignment',				\&parseAUTHFILE				],
								'authtype'				    => [ [ 'root' ], 'assignment',				\&parseAUTHTYPE				],
								'userfile'				    => [ [ 'root' ], 'assignment',				\&parseUSERFILE				],
								'snmpport'				    => [ [ 'root' ], 'assignment',				\&parseSNMPPORT				],
								'serverbind'			    => [ [ 'root' ], 'assignment',				\&parseSERVERBIND			],
								'trapbind'				    => [ [ 'root' ], 'assignment',				\&parseTRAPBIND				],
								'snmp'					    => [ [ 'root' ], 'assignment',				\&parseSNMP					],
								'dtlogfile'				    => [ [ 'root' ], 'assignment',				\&parseDTLOGFILE			],
								'dtlogging'				    => [ [ 'root' ], 'assignment',				\&parseDTLOGGING			],
								'histlength'			    => [ [ 'root' ], 'assignment',				\&parseHISTLENGTH			],
								'historicfile'			    => [ [ 'root' ], 'assignment',				\&parseHISTORICFILE			],
								'historictime'			    => [ [ 'root' ], 'assignment',				\&parseHISTORICTIME			],
								'serverport'			    => [ [ 'root' ], 'assignment',				\&parseSERVERPORT			],
								'trapport'				    => [ [ 'root' ], 'assignment',				\&parseTRAPPORT				],
								'pidfile'				    => [ [ 'root' ], 'assignment',				\&parsePIDFILE				],
								'maxprocs'				    => [ [ 'root' ], 'assignment',				\&parseMAXPROCS				],
								'cltimeout'				    => [ [ 'root' ], 'assignment',				\&parseCLTIMEOUT			],
								'randstart'				    => [ [ 'root' ], 'assignment',				\&parseRANDSTART			],
								'dep_recur_limit'		    => [ [ 'root' ], 'assignment',				\&parseDEPRECURLIMIT		],
								'syslog_facility'		    => [ [ 'root' ], 'assignment',				\&parseSYSLOGFACILITY		],
								'startupalerts_on_reset'    => [ [ 'root' ], 'assignment',				\&parseSTARTUPALERTSONRESET	],
								'hostgroup'				    => [ [ 'root' ], 'directive',				\&parseHOSTGROUP			],
								'watch'					    => [ [ 'any' ], 'directive',				\&parseWATCH				],
								'service'				    => [ [ 'any' ], 'directive',				\&parseSERVICE				],
								'interval'				    => [ [ 'service' ], 'directive',			\&parseINTERVAL				],
								'traptimeout'			    => [ [ 'service' ], 'directive',			\&parseTRAPTIMEOUT			],
								'trapduration'			    => [ [ 'service' ], 'directive',			\&parseTRAPDURATION			],
								'randskew'				    => [ [ 'service' ], 'directive',			\&parseRANDSKEW				],
								'monitor'				    => [ [ 'service' ], 'directive',			\&parseMONITOR				],
								'allow_empty_group'		    => [ [ 'service' ], 'marker',				\&parseALLOWEMPTYGROUP		],
								'description'			    => [ [ 'service' ], 'directive',			\&parseDESCRIPTION			],
								'exclude_hosts'			    => [ [ 'service' ], 'directive',			\&parseEXCLUDEHOSTS			],
								'exclude_period'		    => [ [ 'service' ], 'directive',			\&parseEXCLUDEPERIOD		],
								'depend'				    => [ [ 'service' ], 'directive',			\&parseDEPEND				],
								'dep_behavior'			    => [ [ 'root', 'service' ], 'directive',	\&parseDEPBEHAVIOR			],
								'period'				    => [ [ 'any' ], 'directive',				\&parsePERIOD				],
								'alertevery'			    => [ [ 'period' ], 'directive', 			\&parseALERTEVERY			],
								'alertafter'			    => [ [ 'period' ], 'directive',				\&parseALERTAFTER			],
								'numalerts'				    => [ [ 'period' ], 'directive',				\&parseNUMALERTS			],
								'comp_alerts'			    => [ [ 'period' ], 'marker',				\&parseCOMPALERTS			],
								'alert'					    => [ [ 'period' ], 'directive',				\&parseALERT				],
								'upalert'				    => [ [ 'period' ], 'directive',				\&parseALERT				],
								'startupalert'			    => [ [ 'period' ], 'directive',				\&parseALERT				],
								'upalertafter'			    => [ [ 'period' ], 'directive',				\&parseUPALERTAFTER			] );
		my %ContextHash		= ( 'current'	=> 'root',
								'root'		=> $this->{'configuration'},
								'watch'		=> undef,
								'service'	=> undef,
								'period'	=> undef );
		my $Context			= 'root';
		my $Function;
        my $Line;
		my $Key,
		my $Value;
		my $Kind;

		foreach $Line (@{ $this->{'configuration-data'} })
		{
			# Cache the data.
			$Key = $Line;
			
			# Reset the values.
			undef($Value);
			undef($Kind);
			
			# Clean off the line.
			chomp($Key);

			# If the line has an assignment...
			if ($Key =~ /^\s*\w+\s*=\s*?(.*)$/)
			{
				# Parse out the assignment data.
				$Key	=~ s/^\s*(\w+)\s*=\s*?(.*)$/$1/;
				$Value	=  ($2 || '');
				$Kind	=  'assignment';
			}
			elsif ($Key =~ /^\s*\w+\s+?.*$/)
			{
				# Parse out the directive data.
				$Key	=~ s/^\s*(\w+)\s+?(.*)$/$1/;
				$Value	=  ($2 || '');
				$Kind	=  'directive';
			}
			elsif ($Key =~ /^\s*\w+\s*$/)
			{
				# Parse out the marker data.
				$Key	=~ s/^\s*(\w+)\s*$/$1/;
				$Value	=  '';
				$Kind	=  'marker';
			}

			# Clean off any whitespace.
			$Value =~ s/\s*(.*?)\s*$/$1/ if($Value);

			# If there is a value...
			if ((defined($Value)) &&
				($Kind eq $FunctionHash{$Key}->[1]) &&
				(0 == checkSyntax($FunctionHash{$Key}->[0], $ContextHash{'current'})))
			{
				# Get the function pointer.
				$Function = $FunctionHash{$Key}->[2];

				# Call the function.
				&$Function($this, \%ContextHash, $Key, $Value);
			}
		}
	}

	return 0;
}

###############################################################################
# MonSerializer::checkSyntax
###############################################################################

sub checkSyntax
{
	my ($ContextListRef, $Context)	= @_;
	my $Return						= 1;

	# Validate the input parameters...
	if ((defined($Context)) &&
		(defined($ContextListRef)) &&
		('ARRAY' eq ref($ContextListRef)))
	{
		my $i;

		# Run through the data...
		for $i ( 0 .. $#{ $ContextListRef } )  
		{
			# If the context is a match...
			if (($Context eq $ContextListRef->[$i]) ||
				('any'    eq $ContextListRef->[$i]))
			{
				$Return = 0;
				last;
			}
		}
	}

	return $Return;
}

###############################################################################
# MonSerializer::parseALERTDIR
###############################################################################

sub parseALERTDIR
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setAlertLibraryDirectory($Value);
}

###############################################################################
# MonSerializer::parseMONDIR
###############################################################################

sub parseMONDIR
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setMonitorLibraryDirectory($Value);
}

###############################################################################
# MonSerializer::parseSTATEDIR
###############################################################################

sub parseSTATEDIR
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setProcessStateDirectory($Value);
}

###############################################################################
# MonSerializer::parseLOGDIR
###############################################################################

sub parseLOGDIR
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setLogFileDirectory($Value);
}

###############################################################################
# MonSerializer::parseBASEDIR
###############################################################################

sub parseBASEDIR
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setCommonBaseDirectory($Value);
}

###############################################################################
# MonSerializer::parseCFBASEDIR
###############################################################################

sub parseCFBASEDIR
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setConfigurationDirectory($Value);
}

###############################################################################
# MonSerializer::parseAUTHFILE
###############################################################################

sub parseAUTHFILE
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setAuthorizationFile($Value);
}

###############################################################################
# MonSerializer::parseAUTHTYPE
###############################################################################

sub parseAUTHTYPE
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setAuthorizationType($Value);
}

###############################################################################
# MonSerializer::parseUSERFILE
###############################################################################

sub parseUSERFILE
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setUserFile($Value);
}

###############################################################################
# MonSerializer::parseSNMPPORT
###############################################################################

sub parseSNMPPORT
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setSNMPPort($Value);
}

###############################################################################
# MonSerializer::parseSERVERBIND
###############################################################################

sub parseSERVERBIND
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setServerBindAddress($Value);
}

###############################################################################
# MonSerializer::parseTRAPBIND
###############################################################################

sub parseTRAPBIND
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setTrapBindAddress($Value);
}

###############################################################################
# MonSerializer::parseSNMP
###############################################################################

sub parseSNMP
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setSNMPState(('yes' eq $Value) ? 1 : 0);
}

###############################################################################
# MonSerializer::parseDTLOGFILE
###############################################################################

sub parseDTLOGFILE
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setDowntimeLogFile($Value);
}

###############################################################################
# MonSerializer::parseDTLOGGING
###############################################################################

sub parseDTLOGGING
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setDowntimeLoggingState(('yes' eq $Value) ? 1 : 0);
}

###############################################################################
# MonSerializer::parseHISTLENGTH
###############################################################################

sub parseHISTLENGTH
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setHistoryLength($Value);
}

###############################################################################
# MonSerializer::parseHISTORICFILE
###############################################################################

sub parseHISTORICFILE
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setHistoryLogFile($Value);
}

###############################################################################
# MonSerializer::parseHISTORICTIME
###############################################################################

sub parseHISTORICTIME
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setHistoryTimeLimit($Value);
}

###############################################################################
# MonSerializer::parseSERVERPORT
###############################################################################

sub parseSERVERPORT
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setServerPort($Value);
}

###############################################################################
# MonSerializer::parseTRAPPORT
###############################################################################

sub parseTRAPPORT
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setTrapPort($Value);
}

###############################################################################
# MonSerializer::parsePIDFILE
###############################################################################

sub parsePIDFILE
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setProcessIDFile($Value);
}

###############################################################################
# MonSerializer::parseMAXPROCS
###############################################################################

sub parseMAXPROCS
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setChildProcessLimit($Value);
}

###############################################################################
# MonSerializer::parseCLTIMEOUT
###############################################################################

sub parseCLTIMEOUT
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setClientInactivityTimeout($Value);
}

###############################################################################
# MonSerializer::parseRANDSTART
###############################################################################

sub parseRANDSTART
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setRandomStartSkew($Value);
}

###############################################################################
# MonSerializer::parseDEPRECURLIMIT
###############################################################################

sub parseDEPRECURLIMIT
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setDependencyRecursionLimit($Value);
}

###############################################################################
# MonSerializer::parseSYSLOGFACILITY
###############################################################################

sub parseSYSLOGFACILITY
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setSyslogFacility($Value);
}

###############################################################################
# MonSerializer::parseSTARTUPALERTSONRESET
###############################################################################

sub parseSTARTUPALERTSONRESET
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'root'}->setStartupAlertsAtResetState(('yes' eq $Value) ? 1 : 0);
}

###############################################################################
# MonSerializer::parseHOSTGROUP
###############################################################################

sub parseHOSTGROUP
{
	my ($this, $ContextRef, $Key, $Value) = @_;
	my @HostList = split(/\s+/, $Value);
	
	# If there is at least one host...
	if (1 < scalar(@HostList))
	{
        my $HostGroup;

		# If the allocation was successful...
		if ($HostGroup = new MonHostGroup(shift(@HostList)))
		{
            my $Host;
			
			# Run through the host list...
			foreach $Host (@HostList)
			{
				# Add the host.
				$HostGroup->addHost($Host);
			}

			# Set the data.
			$ContextRef->{'root'}->addHostGroup($HostGroup);
		}
	}
}

###############################################################################
# MonSerializer::parseWATCH
###############################################################################

sub parseWATCH
{
	my ($this, $ContextRef, $Key, $Value) = @_;
	my $HostGroup;
	my $Iterator;

	# If there are any host groups...
	if ($Iterator = $ContextRef->{'root'}->iteratorHostGroups())
	{
		# Run through the available host groups...
		while ($Iterator->hasNext())
		{
			# Get the host group.
			$HostGroup = $Iterator->next();

			# If the host group name matches and the object can be created...
			if (($Value eq $HostGroup->getName()) &&
				($ContextRef->{'watch'} = new MonWatch($HostGroup)))
			{
				# Add the watch.
				$ContextRef->{'root'}->addWatch($ContextRef->{'watch'});

				# Set the current context.
				$ContextRef->{'current'}	= 'watch';

				# Clear the child contexts.
				$ContextRef->{'service'}	= undef;
				$ContextRef->{'period'}		= undef;
				
				# Done.
				last;
			}
			
			# Clean out the reference.
			undef($HostGroup);
		}
	}
	
	# If there isn't a host group, build a default...
	if ((!defined($HostGroup)) &&
		($HostGroup = new MonHostGroup($Value)) &&
		($ContextRef->{'watch'} = new MonWatch($HostGroup)))
	{
		# Add the watch.
		$ContextRef->{'root'}->addWatch($ContextRef->{'watch'});

		# Set the current context.
		$ContextRef->{'current'}	= 'watch';

		# Clear the child contexts.
		$ContextRef->{'service'}	= undef;
		$ContextRef->{'period'}		= undef;
	}
}

###############################################################################
# MonSerializer::parseSERVICE
###############################################################################

sub parseSERVICE
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Try to create the object...
	if ($ContextRef->{'service'} = new MonService($Value))
	{
		# Add the service.
		$ContextRef->{'watch'}->addService($ContextRef->{'service'});
		
		# Set the current context.
        $ContextRef->{'current'}	= 'service';

		# Clear the child contexts.
		$ContextRef->{'period'}		= undef;
	}
}

###############################################################################
# MonSerializer::parseINTERVAL
###############################################################################

sub parseINTERVAL
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'service'}->setMonitorInterval($Value);
}

###############################################################################
# MonSerializer::parseTRAPTIMEOUT
###############################################################################

sub parseTRAPTIMEOUT
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'service'}->setMonitorTrapTimeout($Value);
}

###############################################################################
# MonSerializer::parseTRAPDURATION
###############################################################################

sub parseTRAPDURATION
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'service'}->setMonitorTrapDuration($Value);
}

###############################################################################
# MonSerializer::parseRANDSKEW
###############################################################################

sub parseRANDSKEW
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'service'}->setMonitorIntervalSkew($Value);
}

###############################################################################
# MonSerializer::parseMONITOR
###############################################################################

sub parseMONITOR
{
	my ($this, $ContextRef, $Key, $Value) = @_;
	my $Name;
	my $Arguments;
	my $Monitor;

	# Break out the data.
	$Name		=  $Value;
	$Name		=~ s/^(\S+)\s+?(.*)$/$1/;
	$Arguments	=  $2;
	
	# If the object can be created...
	if ($Monitor = new MonMonitor($Name, $Arguments))
	{
		# Set the data.
		$ContextRef->{'service'}->setMonitor($Monitor);
	}
}

###############################################################################
# MonSerializer::parseALLOWEMPTYGROUP
###############################################################################

sub parseALLOWEMPTYGROUP
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'service'}->setEmptyGroupAllowed(1);
}

###############################################################################
# MonSerializer::parseDESCRIPTION
###############################################################################

sub parseDESCRIPTION
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'service'}->setDescription($Value);
}

###############################################################################
# MonSerializer::parseEXCLUDEHOSTS
###############################################################################

sub parseEXCLUDEHOSTS
{
	my ($this, $ContextRef, $Key, $Value) = @_;
	my @HostList = split(/\s+/, $Value);
	my $Host;
	
	# Run through the host list...
	foreach $Host (@HostList)
	{
		# Add the host.
		$ContextRef->{'service'}->addExcludedHost($Host);
	}
}

###############################################################################
# MonSerializer::parseEXCLUDEPERIOD
###############################################################################

sub parseEXCLUDEPERIOD
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'service'}->setExclusionPeriod($Value);
}

###############################################################################
# MonSerializer::parseDEPEND
###############################################################################

sub parseDEPEND
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'service'}->setDependency($Value);
}

###############################################################################
# MonSerializer::parseDEPBEHAVIOR
###############################################################################

sub parseDEPBEHAVIOR
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{$ContextRef->{'current'}}->setDependencyBehavior($Value);
}

###############################################################################
# MonSerializer::parsePERIOD
###############################################################################

sub parsePERIOD
{
	my ($this, $ContextRef, $Key, $Value) = @_;
	my $Period;
	my $Label;
	
	# If using the labeled form...
	if ($Value =~ /^\S+:\s*\S*$/)
	{
		# Get the label and period.
		$Label	=  $Value;
		$Label	=~ s/^(\S+):\s*(\S*)$/$1/;
		$Period =  ($2 || '');
	}
	else
	{
		# The value is the period.
		$Period = $Value;
		$Label	= '';
	}

	# Try to create the object...
	if ($ContextRef->{'period'} = new MonPeriod($Period, $Label))
	{
		# Add the service.
		$ContextRef->{'service'}->addPeriod($ContextRef->{'period'});
	}
	
	# Set the current context.
	$ContextRef->{'current'} = 'period';
}

###############################################################################
# MonSerializer::parseALERTEVERY
###############################################################################

sub parseALERTEVERY
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'period'}->setAlertInterval($Value);
}

###############################################################################
# MonSerializer::parseALERTAFTER
###############################################################################

sub parseALERTAFTER
{
	my ($this, $ContextRef, $Key, $Value) = @_;
	my $Tolerance;
	my $Interval;
	my $AlertTolerance;

	# Break out the data.
	$Tolerance	=  $Value;
	$Tolerance	=~ s/^(\S+)\s+?(.*)$/$1/;
	$Interval	=  ($2 || '');

	# If the object can be created...
	if ($AlertTolerance = new MonAlertTolerance($Tolerance, $Interval))
	{
		# Set the data.
		$ContextRef->{'period'}->setAlertTolerance($AlertTolerance);
	}
}

###############################################################################
# MonSerializer::parseNUMALERTS
###############################################################################

sub parseNUMALERTS
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'period'}->setAlertLimit($Value);
}

###############################################################################
# MonSerializer::parseCOMPALERTS
###############################################################################

sub parseCOMPALERTS
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'period'}->setAlertsPaired(1);
}

###############################################################################
# MonSerializer::parseALERT
###############################################################################

sub parseALERT
{
	my ($this, $ContextRef, $Key, $Value) = @_;
	my $Name;
	my $Arguments;
	my $Alert;
	my $Factory;

	# Break out the data.
	$Name		=  $Value;
	$Name		=~ s/^(\S+)\s+?(.*)$/$1/;
	$Arguments	=  ($2 || '');

	# Get an alert factory...
	if ($Factory = new MonAlertFactory($Name))
	{
		SWITCH :
		{
			# Based on the key...
			if ('alert' eq $Key)		{ $Alert = $Factory->createAlert(0, $Name, $Arguments); last SWITCH; };
			if ('upalert' eq $Key)		{ $Alert = $Factory->createAlert(1, $Name, $Arguments); last SWITCH; };
			if ('startupalert' eq $Key)	{ $Alert = $Factory->createAlert(2, $Name, $Arguments); last SWITCH; };
		}

		# If the object can be created...
		if ($Alert)
		{
			# Set the data.
			$ContextRef->{'period'}->addAlert($Alert);
		}
	}
}

###############################################################################
# MonSerializer::parseUPALERTAFTER
###############################################################################

sub parseUPALERTAFTER
{
	my ($this, $ContextRef, $Key, $Value) = @_;

	# Set the data.
    $ContextRef->{'period'}->setServiceDowntimeAlertLimit($Value);
}

###############################################################################
# MonSerializer::flatten
###############################################################################

sub flatten
{
	my $this	= shift;
	my $Return	= 0;

	# If there is a host configuration reference and 
	# any configuration data...
	if (defined($this->{'configuration'}))
	{
		my $Value;

		undef($this->{'configuration-data'});

		$Value = <<HERE;
###############################################################################
#
# NOTE: THIS FILE IS GENERATED AUTOMATICALLY. ANY COMMENTS ADDED TO THIS FILE
#       WILL BE LOST THE NEXT TIME IT IS ACCESSED PROGRAMMATICALLY.
#
# Version: This configuration file is compliant with mon version mon-0-38-18.
#
###############################################################################

###############################################################################
# Global options
###############################################################################

HERE

		push(@{ $this->{'configuration-data'} }, $Value);
		
		# Fill out the configuration...
		push(@{ $this->{'configuration-data'} }, "alertdir=$Value\n")				if(defined($Value = $this->{'configuration'}->getAlertLibraryDirectory()));
		push(@{ $this->{'configuration-data'} }, "mondir=$Value\n")					if(defined($Value = $this->{'configuration'}->getMonitorLibraryDirectory()));
		push(@{ $this->{'configuration-data'} }, "statedir=$Value\n")				if(defined($Value = $this->{'configuration'}->getProcessStateDirectory()));
		push(@{ $this->{'configuration-data'} }, "logdir=$Value\n")					if(defined($Value = $this->{'configuration'}->getLogFileDirectory()));
		push(@{ $this->{'configuration-data'} }, "basedir=$Value\n")				if(defined($Value = $this->{'configuration'}->getCommonBaseDirectory()));
		push(@{ $this->{'configuration-data'} }, "cfbasedir=$Value\n")				if(defined($Value = $this->{'configuration'}->getConfigurationDirectory()));
		push(@{ $this->{'configuration-data'} }, "authfile=$Value\n")				if(defined($Value = $this->{'configuration'}->getAuthorizationFile()));
		push(@{ $this->{'configuration-data'} }, "authtype=$Value\n")				if(defined($Value = $this->{'configuration'}->getAuthorizationType()));
		push(@{ $this->{'configuration-data'} }, "userfile=$Value\n")				if(defined($Value = $this->{'configuration'}->getUserFile()));
		push(@{ $this->{'configuration-data'} }, "snmpport=$Value\n")				if(defined($Value = $this->{'configuration'}->getSNMPPort()));
		push(@{ $this->{'configuration-data'} }, "serverbind=$Value\n")				if(defined($Value = $this->{'configuration'}->getServerBindAddress()));
		push(@{ $this->{'configuration-data'} }, "trapbind=$Value\n")				if(defined($Value = $this->{'configuration'}->getTrapBindAddress()));
		push(@{ $this->{'configuration-data'} }, "snmp=$Value\n")					if($Value = (($this->{'configuration'}->isSNMPEnabled()) ? 'yes' : 'no'));
		push(@{ $this->{'configuration-data'} }, "dtlogfile=$Value\n")				if(defined($Value = $this->{'configuration'}->getDowntimeLogFile()));
		push(@{ $this->{'configuration-data'} }, "dtlogging=$Value\n")				if($Value = (($this->{'configuration'}->isDowntimeLoggingEnabled()) ? 'yes' : 'no'));
		push(@{ $this->{'configuration-data'} }, "histlength=$Value\n")				if(defined($Value = $this->{'configuration'}->getHistoryLength()));
		push(@{ $this->{'configuration-data'} }, "historicfile=$Value\n")			if(defined($Value = $this->{'configuration'}->getHistoryLogFile()));
		push(@{ $this->{'configuration-data'} }, "historictime=$Value\n")			if(defined($Value = $this->{'configuration'}->getHistoryTimeLimit()));
		push(@{ $this->{'configuration-data'} }, "serverport=$Value\n")				if(defined($Value = $this->{'configuration'}->getServerPort()));
		push(@{ $this->{'configuration-data'} }, "trapport=$Value\n")				if(defined($Value = $this->{'configuration'}->getTrapPort()));
		push(@{ $this->{'configuration-data'} }, "pidfile=$Value\n")				if(defined($Value = $this->{'configuration'}->getProcessIDFile()));
		push(@{ $this->{'configuration-data'} }, "maxprocs=$Value\n")				if(defined($Value = $this->{'configuration'}->getChildProcessLimit()));
		push(@{ $this->{'configuration-data'} }, "cltimeout=$Value\n")				if(defined($Value = $this->{'configuration'}->getClientInactivityTimeout()));
		push(@{ $this->{'configuration-data'} }, "randstart=$Value\n")				if(defined($Value = $this->{'configuration'}->getRandomStartSkew()));
		push(@{ $this->{'configuration-data'} }, "dep_recur_limit=$Value\n")		if(defined($Value = $this->{'configuration'}->getDependencyRecursionLimit()));
		push(@{ $this->{'configuration-data'} }, "dep_behavior=$Value\n")			if(defined($Value = $this->{'configuration'}->getDependencyBehavior()));
		push(@{ $this->{'configuration-data'} }, "syslog_facility=$Value\n")		if(defined($Value = $this->{'configuration'}->getSyslogFacility()));
		push(@{ $this->{'configuration-data'} }, "startupalerts_on_reset=$Value\n")	if($Value = (($this->{'configuration'}->areStartupAlertsAtResetEnabled()) ? 'yes' : 'no'));

		# Flatten the host groups...
		$Value = <<HERE;

###############################################################################
# Group definitions (hostnames or IP addresses)
###############################################################################

HERE
		push(@{ $this->{'configuration-data'} }, $Value);
		$this->flattenHostGroups($Value)	if($Value = $this->{'configuration'}->iteratorHostGroups());
		
		# Flatten the watches...
		$Value = <<HERE;
###############################################################################
# Watch definitions
###############################################################################
HERE
		push(@{ $this->{'configuration-data'} }, $Value);
		$this->flattenWatches($Value)		if($Value = $this->{'configuration'}->iteratorWatches());
	}

	return 0;
}

###############################################################################
# MonSerializer::flattenHostGroups
###############################################################################

sub flattenHostGroups
{
	my ($this, $Iterator) = @_;

	# Validate the parameter...
	if (defined($Iterator))
	{
		my $IteratorHosts;
		my $HostGroup;
		my $HostGroupName;
		my $HostList;
		
		# Run through the host groups...
		while ($Iterator->hasNext())
		{
			# Get the host group.
			$HostGroup = $Iterator->next();

			# Get the name...
			$HostGroupName = $HostGroup->getName();

			# Clear the host list.
			$HostList = '';

			# If there are hosts...
			if ($IteratorHosts = $HostGroup->iteratorHosts())
			{
				# Run through the hosts...
				while ($IteratorHosts->hasNext())
				{
					# Add on the host.
					$HostList .= $IteratorHosts->next() . ' ';
				}
			}

			# Add on the host group definition.
			push(@{ $this->{'configuration-data'} }, "hostgroup $HostGroupName $HostList\n");
			push(@{ $this->{'configuration-data'} }, "\n");
		}
	}
}

###############################################################################
# MonSerializer::flattenWatches
###############################################################################

sub flattenWatches
{
	my ($this, $Iterator) = @_;

	# Validate the parameter...
	if (defined($Iterator))
	{
		my $IteratorServices;
		my $HostGroup;
		my $Watch;
		
		# Run through the host groups...
		while ($Iterator->hasNext())
		{
			# Get the host group.
			$Watch = $Iterator->next();

			# Get the host group...
			if ($HostGroup = $Watch->getHostGroup())
			{
				# Add on the watch definition.
				push(@{ $this->{'configuration-data'} }, "\n##############################\n");
				push(@{ $this->{'configuration-data'} }, "watch " . $HostGroup->getName() . "\n");
				
				# Flatten the services...
				$this->flattenServices($IteratorServices) if($IteratorServices = $Watch->iteratorServices());
			}
		}
	}
}

###############################################################################
# MonSerializer::flattenServices
###############################################################################

sub flattenServices
{
	my ($this, $Iterator) = @_;
	
	# Validate the parameter...
	if (defined($Iterator))
	{
		my $Service;
		my $Value;
		
		# Run through the host groups...
		while ($Iterator->hasNext())
		{
			# Get the host group.
			$Service = $Iterator->next();

			# Add on the service definition.
			push(@{ $this->{'configuration-data'} }, "\tservice " . $Service->getName() . "\n");
			
			# Fill out the directive configurations...
			push(@{ $this->{'configuration-data'} }, "\t\tdescription $Value\n")	if(defined($Value = $Service->getDescription()));
			push(@{ $this->{'configuration-data'} }, "\t\tinterval $Value\n")		if(defined($Value = $Service->getMonitorInterval()));
			push(@{ $this->{'configuration-data'} }, "\t\ttraptimeout $Value\n")	if(defined($Value = $Service->getMonitorTrapTimeout()));
			push(@{ $this->{'configuration-data'} }, "\t\ttrapduration $Value\n")	if(defined($Value = $Service->getMonitorTrapDuration()));
			push(@{ $this->{'configuration-data'} }, "\t\trandskew $Value\n")		if(defined($Value = $Service->getMonitorIntervalSkew()));
			push(@{ $this->{'configuration-data'} }, "\t\texclude_period $Value\n")	if(defined($Value = $Service->getExclusionPeriod()));
			push(@{ $this->{'configuration-data'} }, "\t\tdepend $Value\n")			if(defined($Value = $Service->getDependency()));
			push(@{ $this->{'configuration-data'} }, "\t\tdep_behavior $Value\n")	if(defined($Value = $Service->getDependencyBehavior()));
			
			# Fill out the marker configuration.
			push(@{ $this->{'configuration-data'} }, "\t\tallow_empty_group\n")		if($Service->isEmptyGroupAllowed());
			
			# Flatten out the monitor...
			$this->flattenMonitor($Value) if($Value = $Service->getMonitor());

			# If there are excluded hosts...
			if ($Value = $Service->iteratorExcludedHosts())
			{
				my $HostList;

				# Run through the host groups...
				while ($Value->hasNext())
				{
					# Add on the host.
					$HostList .= $Value->next() . ' ';
				}
				
				# Add on the excluded host(s) configuration.
				push(@{ $this->{'configuration-data'} }, "\t\texclude_hosts $HostList\n") if(defined($HostList));
			}

			# Flatten out the periods...
			$this->flattenPeriods($Value) if($Value = $Service->iteratorPeriods());
		}
	}
}

###############################################################################
# MonSerializer::flattenMonitor
###############################################################################

sub flattenMonitor
{
	my ($this, $Monitor) = @_;
	
	# Validate the parameter...
	if (defined($Monitor))
	{
		my $MonitorName			= $Monitor->getMonitor();
		my $MonitorArguments	= ($Monitor->getArguments() || '');

		# Add on the excluded host(s) configuration.
		push(@{ $this->{'configuration-data'} }, "\t\tmonitor $MonitorName $MonitorArguments\n") if(defined($MonitorName));
	}
}

###############################################################################
# MonSerializer::flattenPeriods
###############################################################################

sub flattenPeriods
{
	my ($this, $Iterator) = @_;
	
	# Validate the parameter...
	if (defined($Iterator))
	{
		my $Period;
		my $Value;
		
		# Run through the host groups...
		while ($Iterator->hasNext())
		{
			my $Interval;

			# Get the host group.
			$Period = $Iterator->next();
			
			# If there is a period...
			if (defined($Interval = $Period->getPeriod()))
			{
				my $Label = $Period->getLabel();

				# Build the label.
				$Label = (((defined($Label)) && ('' ne $Label)) ? "$Label: " : '') . $Interval;

				# Add on the period definition.
				push(@{ $this->{'configuration-data'} }, "\t\tperiod $Label\n");

				# Flatten out the alert tolerance...
				$this->flattenAlertTolerance($Value) if($Value = $Period->getAlertTolerance());

				# Fill out the directive configurations...
				push(@{ $this->{'configuration-data'} }, "\t\t\talertevery $Value\n")	if(defined($Value = $Period->getAlertInterval()));
				push(@{ $this->{'configuration-data'} }, "\t\t\tnumalerts $Value\n")	if(defined($Value = $Period->getAlertLimit()));

				# Fill out the marker configuration.
				push(@{ $this->{'configuration-data'} }, "\t\t\tcomp_alerts\n")		if($Period->areAlertsPaired());

				# Flatten out the alerts...
				$this->flattenAlerts($Value) if($Value = $Period->iteratorAlerts());
			}
		}
	}
}

###############################################################################
# MonSerializer::flattenAlerts
###############################################################################

sub flattenAlerts
{
	my ($this, $Iterator) = @_;

	# Validate the parameter...
	if (defined($Iterator))
	{
		my $Alert;
		my $AlertType;
		my $AlertName;
		my $AlertArguments;
		
		# Run through the host groups...
		while ($Iterator->hasNext())
		{
			# Get the host group.
			$Alert = $Iterator->next();

			# If the alert has a name...
			if (defined($AlertName = $Alert->getAlert()))
			{
				my $AlertArguments = $Alert->getArguments();

				# Safe default.
				$AlertArguments	= '' unless(defined($AlertArguments));
				
				# Get the name...
				$AlertType		= $Alert->getType();

				SWITCH :
				{
					if (0 == $AlertType) { push(@{ $this->{'configuration-data'} }, "\t\t\talert $AlertName $AlertArguments\n"); last SWITCH; }
					if (1 == $AlertType) { push(@{ $this->{'configuration-data'} }, "\t\t\tupalert $AlertName $AlertArguments\n"); last SWITCH; }
					if (2 == $AlertType) { push(@{ $this->{'configuration-data'} }, "\t\t\tstartupalert $AlertName $AlertArguments\n"); last SWITCH; }
				}
			}
		}
	}
}

###############################################################################
# MonSerializer::flattenAlertTolerance
###############################################################################

sub flattenAlertTolerance
{
	my ($this, $AlertTolerance) = @_;
	
	# Validate the parameter...
	if (defined($AlertTolerance))
	{
		my $Tolerance	= $AlertTolerance->getTolerance();
		my $Interval	= ($AlertTolerance->getInterval() || '');

		# Add on the excluded host(s) configuration.
		push(@{ $this->{'configuration-data'} }, "\t\t\talertafter $Tolerance $Interval\n") if(defined($Tolerance));
	}
}

1;

__END__
