package SSL;

###############################################################################
#                                                                             #
# Filename: SSL.pm                                                            #
#                                                                             #
# Description: Contains script needed to manage SSL data.                     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Safe;

###############################################################################
# SSL::new
###############################################################################

sub new
{
	my ($class, $RootDirectory, $Class, $NamePrivateKey, $NameCSRequest, $NameCertificate) = @_;

	# Initialize the object.
	my $this = 
	{
		'root-directory'	=> $RootDirectory,
		'class'				=> $Class,
		'name-private-key'	=> $NamePrivateKey,
		'name-csrequest'	=> $NameCSRequest,
		'name-certificate'	=> $NameCertificate,
	};

	# Safe default(s).
	$this->{'root-directory'}	= './' unless(defined($this->{'root-directory'}));
	$this->{'class'}		   	= '' unless(defined($this->{'class'}));
	$this->{'name-private-key'}	= '-private-key.pem' unless(defined($this->{'name-private-key'}));
	$this->{'name-csrequest'}	= '-csrequest.pem' unless(defined($this->{'name-csrequest'}));
	$this->{'name-certificate'}	= '-certificate.pem' unless(defined($this->{'name-certificate'}));

	# Align the object with this package.
	bless($this, $class);
	
	# Polish the root directory.
	$this->{'root-directory'}	= $this->PolishDirectory($this->{'root-directory'});

	# Make sure the directory is available...	
	mkdir($this->{'root-directory'}, 0660) unless( -d $this->{'root-directory'});

	# Return the object.
	return $this;
}

###############################################################################
# SSL::GetRootDirectory
###############################################################################

sub GetRootDirectory
{
	my $this = shift;

	# Return the requested data.
	return $this->{'root-directory'};
}

###############################################################################
# SSL::SetRootDirectory
###############################################################################

sub SetRootDirectory
{
	my ($this, $RootDirectory) = @_;

	# Safe default(s).
	$RootDirectory = './' unless(defined($RootDirectory));
	
	# Assume the new value provided.
	$this->{'root-directory'} = PolishDirectory($RootDirectory);

	# Make sure the directory is available...	
	mkdir($this->{'root-directory'}, 0660) unless( -d $this->{'root-directory'});
}

###############################################################################
# SSL::GetClass
###############################################################################

sub GetClass
{
	my $this = shift;

	# Return the requested data.
	return $this->{'class'};
}

###############################################################################
# SSL::SetClass
###############################################################################

sub SetClass
{
	my ($this, $Class) = @_;

	# If the value is different...
	if ($Class ne $this->{'class'})
	{
		# Assume the new value provided.
		$this->{'class'} = $Class;

		# Clear the data.
		$this->SetSubject();
	}
}

###############################################################################
# SSL::PolishDirectory
###############################################################################

sub PolishDirectory
{
	my ($this, $Directory) = @_;
	
	# Validate the input parameter(s)...
	if (defined($Directory))
	{
		# If the directory does not have an ending slash...
		unless ($Directory =~ /^.*\/$/)
		{
			# Append the missing slash.
			$Directory .= '/';
		}
	}

	return $Directory;
}

###############################################################################
# SSL::GetSubject
###############################################################################

sub GetSubject
{
    my ($this, $SubjectRef) = @_;

	# Validate the input parameter(s)...
	if (defined($SubjectRef))
	{
		# See if the subject is available...
		unless(defined($this->{'subject'}))
		{
			# Get the path to the CSR.
			my $CSRPath = $this->GetCertificateSigningRequestPath();
			my $CSR;

			# Try to get the certificate information...
			if (($CSRPath) &&
				($CSR = Safe::backtick("/usr/bin/openssl req -config /usr/lib/openssl.cnf -in $CSRPath -text")))
			{
				# Get the subject data.
				$this->{'subject'}->{'country'}				= $1 if($CSR =~ /Subject:\s*C=(\S+),\s*/g);
				$this->{'subject'}->{'province'}			= $1 if($CSR =~ /ST=(.+?),\s*/g);
				$this->{'subject'}->{'locality'}			= $1 if($CSR =~ /L=(.+?),\s*/g);
				$this->{'subject'}->{'organization'}		= $1 if($CSR =~ /O=(.+?),\s*/g);
				$this->{'subject'}->{'organizational_unit'} = $1 if($CSR =~ /OU=(.+?),\s*/g);
				$this->{'subject'}->{'common_name'}			= $1 if($CSR =~ /CN=(.+?)\//g);
				$this->{'subject'}->{'e_mail'}				= $1 if($CSR =~ /Email=(.+)/g);
			}
		}

		# If there is data...
		if (defined($this->{'subject'}))
		{
			my $Key;

			# Return the requested data.
			foreach $Key (keys(%{ $this->{'subject'} }))
			{
				# Copy the data...
				$SubjectRef->{$Key} = $this->{'subject'}->{$Key};
			}
		}
	}
}

###############################################################################
# SSL::SetSubject
###############################################################################

sub SetSubject
{
	my ($this, $SubjectRef) = @_;
	my $Return				= 0;

	# If there is a subect reference...
    if (defined($SubjectRef))
	{
		# Validate the data...
		if ($SubjectRef->{'country'} !~ /^[a-zA-Z]{2}$/)
		{
			# Send back an error.
			$Return = 200000;
		}
		elsif (($SubjectRef->{'province'} !~ /^\S+$/) ||
			   ($SubjectRef->{'province'} =~ /,/))
		{
			# Send back an error.
			$Return = 200001;
		}
		elsif (($SubjectRef->{'locality'} !~ /^\S+(\s*\S+)*$/) ||
			   ($SubjectRef->{'locality'} =~ /,/))
		{
			# Send back an error.
			$Return = 200002;
		}
		elsif (($SubjectRef->{'organization'} !~ /^\S+(\s*\S+)*$/) ||
			   ($SubjectRef->{'organization'} =~ /,/))
		{
			# Send back an error.
			$Return = 200003;
		}
		elsif (($SubjectRef->{'organizational_unit'} !~ /^\S+(\s*\S+)*$/) ||
			   ($SubjectRef->{'organizational_unit'} =~ /,/))
		{
			# Send back an error.
			$Return = 200004;
		}
		elsif ($SubjectRef->{'common_name'} !~ /^(\w|-|\.|_)+$/)
		{
			# Send back an error.
			$Return = 200005;
		}
		elsif ($SubjectRef->{'e_mail'} !~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
		{
			# Send back an error.
			$Return = 200006;
		}

		# If things are still running smoothly...
		unless ($Return)
		{
			# Set using the provided data.
			$this->{'subject'}->{'country'}				= $SubjectRef->{'country'};
			$this->{'subject'}->{'province'}			= $SubjectRef->{'province'};
			$this->{'subject'}->{'locality'}			= $SubjectRef->{'locality'};
			$this->{'subject'}->{'organization'}		= $SubjectRef->{'organization'};
			$this->{'subject'}->{'organizational_unit'} = $SubjectRef->{'organizational_unit'};
			$this->{'subject'}->{'common_name'}			= $SubjectRef->{'common_name'};
			$this->{'subject'}->{'e_mail'}				= $SubjectRef->{'e_mail'};
		}
	}
	else
	{
		# Clean out the subject.
		undef($this->{'subject'});
	}

	return $Return;
}

###############################################################################
# SSL::GetPrivateKey
###############################################################################

sub GetPrivateKey
{
	my ($this, $AutoGenerate)	= @_;
	my $PrivateKeyPath			= $this->GetPrivateKeyPath();
	my $PrivateKey;
    
	# Safe default.
	$AutoGenerate = 0 unless(defined($AutoGenerate));

	# If the private key does not exist...
	if (($AutoGenerate) &&
		(!( -e $PrivateKeyPath )))
	{
		# Try to generate a private key.
		Safe::system("/usr/bin/openssl genrsa -out $PrivateKeyPath 1024 2>/dev/null");
	}
	
	# Try to open the private key...
	if (open(PRIVATEKEY, "< $PrivateKeyPath"))
	{
		# Read in the private key.
		read(PRIVATEKEY, $PrivateKey, -s $PrivateKeyPath);
		
		# Close the file.
		close(PRIVATEKEY);
	}

	return $PrivateKey;
}

###############################################################################
# SSL::SetPrivateKey
###############################################################################

sub SetPrivateKey
{
	my ($this, $PrivateKey)	= @_;
	my $PrivateKeyPath		= $this->GetPrivateKeyPath();
	my $Return				= 0;

	# If there is a private key...
	if (defined($PrivateKey))
	{
        # Clean off any goo...
		$PrivateKey =~ s/\r//g;
		$PrivateKey =~ m#(-----BEGIN RSA PRIVATE KEY-----.*-----END RSA PRIVATE KEY-----)#s;
		
		# Make sure there is a private key...
		if (defined($PrivateKey = $1))
		{
			# Tack on the newline.
			$PrivateKey .= "\n";

			# If the file can be opened...
			if (open(PRIVATEKEY, "> $PrivateKeyPath"))
			{
				# Spew out the private key.
				print PRIVATEKEY $PrivateKey;

				# Close the file.
				close(PRIVATEKEY);
			}
			else
			{
				# Send back an error.
				$Return = 200007;
			}
		}
		else
		{
			# Send back an error.
			$Return = 200008;
		}
	}
	else
	{
		# Delete the private key.
		unlink($PrivateKeyPath);
	}

	return $Return;
}

###############################################################################
# SSL::GetPrivateKeyPath
###############################################################################

sub GetPrivateKeyPath
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'root-directory'} . $this->{'class'} . $this->{'name-private-key'};
}

###############################################################################
# SSL::GetPrivateKeyName
###############################################################################

sub GetPrivateKeyName
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'name-private-key'};
}

###############################################################################
# SSL::SetPrivateKeyName
###############################################################################

sub SetPrivateKeyName
{
	my ($this, $Name)	= @_;
	my $Return			= 0;
	
	# Validate the input parameter(s)...
	if (defined($Name))
	{
		# Set the name.
		$this->{'name-private-key'} = $Name;
	}
	else
	{
		$Return = 200010;
	}

	return $Return;
}

###############################################################################
# SSL::GetCertificateSigningRequest
###############################################################################

sub GetCertificateSigningRequest
{
	my ($this, $AutoGenerate)	= @_;
	my $CSRPath					= $this->GetCertificateSigningRequestPath();
	my $CSR;
    
	# Safe default.
	$AutoGenerate = 0 unless(defined($AutoGenerate));

	# If the private key does not exist...
	if (($AutoGenerate) &&
		(!( -e $CSRPath )))
	{
		my %Subject;

		# Get the subject.
		$this->GetSubject(\%Subject);

		# If there is a subject and private key...
		if ((defined($Subject{'country'})) &&
			(defined($this->GetPrivateKey(1))))
		{
			my $PrivateKeyPath	= $this->GetPrivateKeyPath();
			
			# Try to generate a certificate signing request...
			if (open(NEWCSR, "| /usr/bin/openssl req -new -config /usr/lib/openssl.cnf -key $PrivateKeyPath -days 365 -out $CSRPath 2>/dev/null"))
			{
				# Pipe out the subject information...
				print NEWCSR <<SUBJECT;
$Subject{'country'}
$Subject{'province'}
$Subject{'locality'}
$Subject{'organization'}
$Subject{'organizational_unit'}
$Subject{'common_name'}
$Subject{'e_mail'}
SUBJECT
				# Close the file.
				close(NEWCSR);
			}
		}
	}
	
	# Try to open the certificate signing request file...
	if (open(CSR, "< $CSRPath"))
	{
		# Read in the certificate signing request.
		read(CSR, $CSR, -s $CSRPath);
		
		# Close the file.
		close(CSR);
	}

	return $CSR;
}

###############################################################################
# SSL::SetCertificateSigningRequest
###############################################################################

sub SetCertificateSigningRequest
{
	my ($this, $CSR)	= @_;
	my $CSRPath			= $this->GetCertificateSigningRequestPath();
	my $Return			= 0;

	# If there is a certificate signing request...
	if (defined($CSR))
	{
        # Clean off any goo...
		$CSR =~ s/\r//g;
		$CSR =~ m#(-----BEGIN CERTIFICATE REQUEST-----.*-----END CERTIFICATE REQUEST-----)#s;
		
		# Make sure there is a certificate signing request...
		if (defined($CSR = $1))
		{
			# Tack on the newline.
			$CSR .= "\n";

			# If the file can be opened...
			if (open(CSR, "> $CSRPath"))
			{
				# Spew out the certificate signing request.
				print CSR $CSR;

				# Close the file.
				close(CSR);
			}
			else
			{
				# Send back an error.
				$Return = 200009;
			}
		}
		else
		{
			# Send back an error.
			$Return = 200008;
		}
	}
	else
	{
		# Delete the certificate signing request.
		unlink($CSRPath);
	}

	return $Return;
}

###############################################################################
# SSL::GetCertificateSigningRequestPath
###############################################################################

sub GetCertificateSigningRequestPath
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'root-directory'} . $this->{'class'} . $this->{'name-csrequest'};
}

###############################################################################
# SSL::GetCertificateSigningRequestName
###############################################################################

sub GetCertificateSigningRequestName
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'name-csrequest'};
}

###############################################################################
# SSL::SetCertificateSigningRequestName
###############################################################################

sub SetCertificateSigningRequestName
{
	my ($this, $Name)	= @_;
	my $Return			= 0;
	
	# Validate the input parameter(s)...
	if (defined($Name))
	{
		# Set the name.
		$this->{'name-csrequest'} = $Name;
	}
	else
	{
		$Return = 200010;
	}

	return $Return;
}

###############################################################################
# SSL::GetCertificate
###############################################################################

sub GetCertificate
{
	my ($this, $AutoGenerate)	= @_;
	my $CertificatePath			= $this->GetCertificatePath();
	my $Certificate;
    
	# Safe default.
	$AutoGenerate = 0 unless(defined($AutoGenerate));

	# If the private key does not exist...
	if (($AutoGenerate) &&
		(!( -e $CertificatePath )))
	{
		# If there is a private key and certificate signing request...
		if ((defined($this->GetPrivateKey(1))) &&
			(defined($this->GetCertificateSigningRequest(1))))
		{
			my $PrivateKeyPath	= $this->GetPrivateKeyPath();
			my $CSRPath			= $this->GetCertificateSigningRequestPath();

			# Try to generate a certificate.
			Safe::system("/usr/bin/openssl x509 -days 365 -req -signkey $PrivateKeyPath -in $CSRPath -out $CertificatePath 2>/dev/null");
		}
	}
	
	# Try to open the certificate file...
	if (open(CERTIFICATE, "< $CertificatePath"))
	{
		# Read in the certificate.
		read(CERTIFICATE, $Certificate, -s $CertificatePath);
		
		# Close the file.
		close(CERTIFICATE);
	}

	return $Certificate;
}

###############################################################################
# SSL::SetCertificate
###############################################################################

sub SetCertificate
{
	my ($this, $Certificate)	= @_;
	my $CertificatePath			= $this->GetCertificatePath();
	my $Return					= 0;

	# If there is a certificate...
	if (defined($Certificate))
	{
        # Clean off any goo...
		$Certificate =~ s/\r//g;
		$Certificate =~ m#(-----BEGIN CERTIFICATE-----.*-----END CERTIFICATE-----)#s;
		
		# Make sure there is a certificate...
		if ((defined($Certificate = $1)) &&
			(0 == $this->VerifyCertificate($Certificate)))
		{
			# Tack on the newline.
			$Certificate .= "\n";

			# If the file can be opened...
			if (open(CERTIFICATE, "> $CertificatePath"))
			{
				# Spew out the certificate.
				print CERTIFICATE $Certificate;

				# Close the file.
				close(CERTIFICATE);
			}
			else
			{
				# Send back an error.
				$Return = 200009;
			}
		}
		else
		{
			# Send back an error.
			$Return = 200008;
		}
	}
	else
	{
		# Delete the certificate.
		unlink($CertificatePath);
	}

	return $Return;
}

###############################################################################
# SSL::VerifyCertificate
###############################################################################

sub VerifyCertificate
{
	my ($this, $Certificate)	= @_;
	my $Return					= 1;
	
	# Open up the x509 routine for validation...
	if (open(X509, "| /usr/bin/openssl x509 2>/dev/null 1>/dev/null"))
	{
		# Validate the certificate.
		print X509 $Certificate;

		# Close the pipe.
		close(X509);

		# Set the return value...
		$Return = ($?) ? 1 : 0;
	}

	return $Return;
}

###############################################################################
# SSL::GetCertificatePath
###############################################################################

sub GetCertificatePath
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'root-directory'} . $this->{'class'} . $this->{'name-certificate'};
}

###############################################################################
# SSL::GetCertificateName
###############################################################################

sub GetCertificateName
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'name-certificate'};
}

###############################################################################
# SSL::SetCertificateName
###############################################################################

sub SetCertificateName
{
	my ($this, $Name)	= @_;
	my $Return			= 0;
	
	# Validate the input parameter(s)...
	if (defined($Name))
	{
		# Set the name.
		$this->{'name-certificate'} = $Name;
	}
	else
	{
		$Return = 200010;
	}

	return $Return;
}

1;

__END__
