package Safe;

###############################################################################
#                                                                             #
# Filename: Safe.pm                                                           #
#                                                                             #
# Description: Provide a suite of utilities for spawning new processes,       #
#              properly closing unneeded file descriptors beforehand.         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;
use POSIX;
use Socket;
use IO::Handle;
use IO::Socket;

######################################################################
# global enum
######################################################################

use vars qw($PIPE_READ $PIPE_WRITE $PIPE_BIDIRECTIONAL);
$PIPE_READ          = 0;
$PIPE_WRITE         = 1;
$PIPE_BIDIRECTIONAL = 2;

######################################################################
# test routine
######################################################################

# &testroutine;

sub testroutine {
	print "\n",'='x50,"\n";
	print "Test 01: Safe::system(\"uname\")\n";
	print '-'x50,"\n";
	Safe::system("uname");

	print "\n",'='x50,"\n";
	print "Test 02: Safe::system(\"uname -a\")\n";
	print '-'x50,"\n";
	Safe::system("uname -a");

	print "\n",'='x50,"\n";
	print "Test 03: Safe::system('uname','-a')\n";
	print '-'x50,"\n";
	Safe::system('uname','-a');

	print "\n",'='x50,"\n";
	print "Test 04: Safe::backtick('ls -Fal')\n";
	print '-'x50,"\n";
	print Safe::backtick('ls -Fal'),"\n";

	print "\n",'='x50,"\n";
	print "Test 05: Safe::backtick('ls','-Fal')\n";
	print '-'x50,"\n";
	print Safe::backtick('ls','-Fal'),"\n";

	{
		print "\n",'='x50,"\n";
		print "Test 06: Safe::pipe('ls','-Fal')\n";
		print '-'x50,"\n";
		my $ls = Safe::pipe('ls','-Fal');
		my $line;
		while ($line = <$ls>) {
			chomp $line;
			print "read: $line\n";
		}
		close($ls);
	}

	{
		print "\n",'='x50,"\n";
		print "Test 07: Safe::pipe_read('ls','-Fal')\n";
		print '-'x50,"\n";
		my $ls = Safe::pipe_read('ls','-Fal');
		my $line;
		while ($line = <$ls>) {
			chomp $line;
			print "read: $line\n";
		}
		close($ls);
	}

	print "\n",'='x50,"\n";
	print "Test 08: Safe::pipe_write('cat')\n";
	print '-'x50,"\n";
	my $cat = Safe::pipe_write('cat');
	print $cat "This line is being fed into cat.\n";
	sleep 2;
	print $cat "This is another line being fed into cat.\n";
	sleep 2;
	print $cat "This is yet another line being fed into cat.\n";
	sleep 2;
	close($cat);

	print "\n",'='x50,"\n";
	print "Test 09: Safe::pipe('bc','-l')\n";
	print '-'x50,"\n";
	my ($bc_r, $bc_w) = Safe::pipe('bc','-l');
	my $z;
	sleep 1;
	print "Pwrite: 2+4\n";  print $bc_w "2+4\n";
	$z = <$bc_r>; chomp($z);
	print "Pread: ->$z<-\n";
	sleep 1;
	print "Pwrite: 3+5\n"; print $bc_w "3+5\n";
	$z = <$bc_r>; chomp($z);
	print "Pread: ->$z<-\n";
	sleep 1;
	print "Pwrite: 4+6\n"; print $bc_w "4+6\n";
	$z = <$bc_r>; chomp($z);
	print "Pread: ->$z<-\n";
	sleep 1;
	print "Pwrite: 5+7\n"; print $bc_w "5+7\n";
	$z = <$bc_r>; chomp($z);
	print "Pread: ->$z<-\n";
	sleep 1;
	close($bc_r);
	close($bc_w);

	print "\n",'='x50,"\n";
	print "Test 10: Safe::spawn_daemon(\"touch working\")\n";
	print '-'x50,"\n";
	Safe::spawn_daemon("touch working");

	print "\n",'='x50,"\n";
	print "Test 11: Safe::system(\"sleep 10\")\n";
	print '-'x50,"\n";
	Safe::system("sleep 10");

	print "\n",'='x50,"\n";
	print "Test 12: Safe::exec('/bin/echo','That\'s all folks!')\n";
	print '-'x50,"\n";
	Safe::exec('/bin/echo','That\'s all folks!');
}


######################################################################
# interfaces
######################################################################

#
# &system
#
# emulate the ::system() function, but close file descriptors
#

sub system {
	my @args = @_;
	my $pid;

	if ($pid = fork) {
		####################
		# parent process
		####################

		# wait for the child process to exit
		my $waitval = wait;
		while (($waitval != -1) && ($waitval != $pid)) {
			$waitval = wait;
		}

		# return 255*256 if error (emulating ::system(), or
		# return the child's return value.
		if ($waitval == -1) {
			return (255*256);
		} else {
			return $?;
		}
	}

	####################
	# child process
	####################

	# close all file descriptors above STDERR (2).
	closefd();

	# exec the new process
	CORE::exec(@args) or exit;

}

#
# &exec
#
# emulate the CORE::exec() function, but close file descriptors
#

sub exec {
	my @args = @_;
	my $pid;

	# close all file descriptors above STDERR (2).
	closefd();

	# exec the new process
	CORE::exec(@args) or exit;

}

#
# &spawn_daemon
#
# fork off a new process, redirect its stdin/stdout/stderr
# to /dev/null, begin a new session, and exec the desired
# program.
#
# in other words, spawn a process that is completely detached
# from its parent.
#

sub spawn_daemon {
	my @args = @_;
	my $pid;

	if ($pid = fork) {
		####################
		# parent process
		####################

		# we must do this so zombie processes don't
		# accumuate.
		$SIG{CHLD} = sub { wait };

		return $pid;
	}

	####################
	# child process
	####################

	# initiate a new "session"
	POSIX::setsid;

	# close all file descriptors above STDERR (2).
	closefd();

	# reopen standard file descriptors to /dev/null
	open(STDIN,"< /dev/null");
	open(STDOUT,"> /dev/null");
	open(STDERR,">&STDOUT");

	# exec the new process
	CORE::exec(@args) or exit;

}

#
# &backtick
#
# emulate the backtick operation, but close file descriptors
#

sub backtick {
	my @args = @_;
	my $pid;

	# this bit from "man perlipc"
	do {
		my $attempt_count = 0;
		$pid = open(CHILD, "-|"); # fork
		unless (defined $pid) {
			die "cannot fork: $!" if $attempt_count++ >> 6;
			sleep 1;
		}
	} until defined $pid;

	if ($pid) {
		####################
		# parent process
		####################

		# The following commented-out code did not fully emulate
		# backticks because it did not operate properly in list context.

		#my $buffer;
		#{
		#	local $/;  # make a temp copy of $/ for use in this block only.
		#	undef $/;  # disable the input field separator
		#	$buffer = <CHILD>;   # suck in all of child's ouput
		#}
		#close(CHILD);
		#return $buffer;

		# This code works properly in list context.

		my @buffer;
		while (<CHILD>) {
			push(@buffer,$_);
		}
		close(CHILD);
		return wantarray ? @buffer : join('',@buffer);

	} else {
		####################
		# child process
		####################

		# close all file descriptors above STDERR (2).
		closefd();

		# exec the new process
		CORE::exec(@args) or exit;

	}

}

######################################################################
# pipe subroutines
######################################################################

# This global variable stores the PID associated with every
# pipe file handle.  This data is used in pipe_close() to properly
# reap children via waitpid().

use vars qw($pipe_hash);
$pipe_hash = {};

#
# &pipe
#
# open a bidirectional pipe to a forked and execed process.
# close file descriptors.
#

sub pipe {
	my ($r, $w) = pipe_generic($PIPE_BIDIRECTIONAL,@_);
	return ($r, $w);
}

#
# &pipe_read
#
# open a read-only pipe to a forked and execed process.
#

sub pipe_read {
	my ($r, $w) = pipe_generic($PIPE_READ,@_);
	close($w);
	return $r;
}

#
# &pipe_read
#
# open a write-only pipe to a forked and execed process.
#

sub pipe_write {
	my ($r, $w) = pipe_generic($PIPE_WRITE,@_);
	close($r);
	return $w;
}

#
# &pipe_generic($direction, @args);
#
# open a pipe to a forked and execed process.
# close file descriptors.
# the $direction argument indicates the direction of the pipe:
#   PIPE_READ          = 0;
#   PIPE_WRITE         = 1;
#   PIPE_BIDIRECTIONAL = 2;
#

sub pipe_generic {
	my $direction = shift @_;
	my @args = @_;

	# these will be our file descriptors
	my ($CHILD_W,$CHILD_R,$PARENT_W,$PARENT_R);
	$CHILD_W = IO::Handle->new();
	$CHILD_R = IO::Handle->new();
	$PARENT_W = IO::Handle->new();
	$PARENT_R = IO::Handle->new();

	# connect the file descriptors together with pipes
	CORE::pipe($PARENT_R,$CHILD_W);
	CORE::pipe($CHILD_R,$PARENT_W);

	# disable buffering on everything
	$CHILD_W->autoflush;
    $PARENT_W->autoflush;
	$CHILD_R->autoflush;
    $PARENT_R->autoflush;

	# fork!
	my $pid = fork;
	if (! defined $pid) {
		die "cannot fork: $!";
	} elsif ($pid) {
		####################
		# parent process
		####################

		# we don't need sockets to talk to ourself.
		close $CHILD_R;
		close $CHILD_W;

		# record the PID in the $pipe_hash, keyed by the file handles.
		$pipe_hash->{$PARENT_R} = $pid if ($PARENT_R);
		$pipe_hash->{$PARENT_W} = $pid if ($PARENT_W);

		# return the pipe handle
		return ($PARENT_R, $PARENT_W);

	} else {
		####################
		# child process
		####################


		# we don't need a socket to talk to ourself.
		close $PARENT_R;
		close $PARENT_W;

		# dup stdin if needed
		if (($direction == $PIPE_BIDIRECTIONAL) ||
			($direction == $PIPE_WRITE) ) {
			#close(STDIN);
			STDIN->fdopen(fileno($CHILD_R),"r");
		}

		# dup stdout if needed
		if (($direction == $PIPE_BIDIRECTIONAL) ||
			($direction == $PIPE_READ) ) {
			#close(STDOUT);
			STDOUT->fdopen(fileno($CHILD_W),"w");
		}

		# (leave STDERR going to the apache error log)

		# make sure that stdin an stdout are set to autoflush...
		STDIN->autoflush;
		STDOUT->autoflush;

		# close all file descriptors above STDERR (2).
		closefd(fileno($CHILD_R),fileno($CHILD_W));

		# exec the new process
		CORE::exec(@args) or exit;

	}

}

#
# &pipe_close($file_handle);
#
# perform a waitpid() to reap the child process,
# then close the file handle.  If the child process is
# not properly reaped, then it could stay behind in the
# process table as a zombie (defunct) process.
#

sub pipe_close {
	my ($fh) = @_;

	# If any entry exists for this file handle...
	if (exists $pipe_hash->{$fh}) {

		# get the PID...
		my $pid = $pipe_hash->{$fh};

		# remove this entry from the hash...
		delete $pipe_hash->{$fh};

		# remove any other entries pointing to this pid...
		foreach my $i (keys %$pipe_hash) {
			if ($pipe_hash->{$i} == $pid) {
				delete $pipe_hash->{$i};
			}
		}

		# wait for the child to exit and gather its return value.
		# this is necessary to properly reap it and prevent it from
		# becoming a zombie (defunct) process.
		waitpid($pid,0);
	}

	# close the file handle...
	close($fh);
}

######################################################################
# utility subroutines
######################################################################

#
# &closefd
#
# iterate over all file descriptors, closing each one, other than
# stdin, stdout, stderr, and any specified as arguments.
#

sub closefd {

	# make a hash of the exceptions specified in the arguments
	my %skip_hash;
	foreach my $i (@_) {
		$skip_hash{$i} = 0;
	}

	# iterate over all file descriptors above 2
	my $fd;
	for ($fd=3; $fd < POSIX::sysconf(&POSIX::_SC_OPEN_MAX); $fd++) {
		# close the file descriptor unless it is an exception
		POSIX::close($fd) if (! exists $skip_hash{$fd});
	}

}

1;

__END__
