package VirtualServ;

###############################################################################
#                                                                             #
# Filename: VirtualServ.pm                                                    #
#                                                                             #
# Description: This module provides configuration support for managing        #
#              virtual server configuration.                                  #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageLogCGI;
use VirtualDomain;
use MonSettings;
use NetConfig;
use Safe;

###############################################################################
# VirtualServ::LogMessage
###############################################################################

sub LogMessage
{
	my ($Code, $Category, $Severity, $ParameterRef, $Domain) = @_;
	my $Location	= ($Domain) ? "/home/domain/$Domain/logs/error.log" : '/usr/ins/intel/gui/logs/server.log';
	my $Log			= new LanguageLogCGI($Location, '/usr/ins/intel/gui/locale', [ 'VirtualServ.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
}

###############################################################################
# VirtualServ::getSystemDomain
###############################################################################

sub getSystemDomain
{
	my $Domain = '';

	# Try to open the resolve file...
	if (open(RESOLVE, '/etc/resolv.conf'))
	{
		# Read the data.
		my @ResolveData	= <RESOLVE>; 
		
		# Get the 'domain' entry.
		$Domain = (grep(/domain/, @ResolveData))[0];
		$Domain = (split(/\s+/, $Domain))[1];

		# Close the file.
		close(RESOLVE);
	}

	return $Domain;
}

################################################################################
#  VirtualServ::getMappingsAdapter
################################################################################

sub getMappingsAdapter
{
	my ($HostName)	= @_;
	my @AdapterList;
	my @NICList		= NetConfig::listNetInterfaces();
	my $Domain		= getSystemDomain();
	my $Interface;
	my $NIC;

	# Add localhost entry.
	push(@AdapterList, [ '127.0.0.1' , 'localhost.localdomain', 'localhost' ]);
	
	# Run through the NIC interfaces...
	foreach $NIC (@NICList)
	{ 
		# Looking for 'eth0' otherwise 'eth1'...
		if ('eth0' eq $NIC->{'fullnic'})
		{
			# Set the preferred NIC and bail out.
			$Interface = $NIC->{'ipaddress'};
			last;
		}
		elsif ('eth1' eq $NIC->{'fullnic'})
		{
			# Set the preferred NIC but DO NOT break.
			$Interface = $NIC->{'ipaddress'};
		}
	}

	# If an interface was found.
	if ($Interface)
	{
		# Add the preferred interface to the list.
		push(@AdapterList, [ $Interface, "$HostName\.$Domain", $HostName ]);
	}

	return @AdapterList;
}

###############################################################################
# VirtualServ::getMappingsVirtualDomain
###############################################################################

sub getMappingsVirtualDomain
{
	my ($HostName)	= @_;
	my @AddressList;
	my @Domains		= VirtualDomain::list_vws();
	my $Domain		= getSystemDomain();

	# Build the FQDN.
	$HostName .= ('.' . $Domain) if($Domain);
	
	# Run through all of the domains...
	foreach $Domain (@Domains)
	{ 
		# If the name is not that the of local machine...
		unless ($Domain->{'vdn'} =~ $HostName)
		{
			# Add the preferred interface to the list.
			push(@AddressList, [ $Domain->{'vdip'}, $Domain->{'vdn'} ]);
		}
	}

	return @AddressList;
}

###############################################################################
# VirtualServ::setHostFile
###############################################################################

sub setHostFile
{
	my @OriginalMaps;
	my @AddressList;
	my @Mappings;
	my $HostName	= Safe::backtick('/bin/hostname');
	my $HostsFile	= '/etc/hosts';
	my $Warning;
	my $Item;
	my $Return		= 0;

	# Clean up the host name.
	chomp($HostName);

	# Get the adapter address map.
	@AddressList = getMappingsAdapter($HostName);
	
	# Run through the list...
	foreach $Item (@AddressList)
	{
		# Add the entries to the list.
		push(@Mappings, (join("\t", @{ $Item }) . "\n"));
	}

	# 'localhost' or Default Domain *MUST* be the FIRST entry or there will be 
	# problems starting Sendmail. Then add all virtual hosts to array.

	# Get the adapter address map.
	@AddressList = getMappingsVirtualDomain($HostName);
	
	# Run through the list...
	foreach $Item (@AddressList)
	{
		# Add the entries to the list.
		push(@Mappings, (join("\t", @{ $Item }) . "\n"));
	}
	
	# Try to open the file...
	if (open(HOSTS, $HostsFile))
	{		 
		# Try to lock the file...
		if (flock(HOSTS, 2))
		{			 
			# Run through the file...
			while (<HOSTS>)
			{
				# Match from the warning to the end...
				if (/^\#\#\#\#\#/ ... eof(HOSTS))
				{
					# Cache the original mapping.
					push(@OriginalMaps, $_);
				}
			}

			# Close the file.
			close(HOSTS);
		}
		else
		{
			# Log the error.
			LogMessage(1700000, 'error', 'warning');
		}
	}
	else
	{
		# Log the error.
		LogMessage(1700001, 'error', 'warning');
	}

	# Set the warning.
	$Warning = <<HERE;
#######################################################################
## Warning!  The lines above may be automatically edited!
#######################################################################
HERE

	# Add the warning if necessary...
	push(@OriginalMaps, $Warning) if(0 == grep($Warning, @OriginalMaps));

	# Add the original mappings back in.
	push(@Mappings, "\n", @OriginalMaps);

	# Try to open the file...
	if (open(HOSTS, "> $HostsFile"))
    {
		# Try to lock the file...
		if (flock(HOSTS, 2))
		{			 
			# Spew out the file.
			print HOSTS @Mappings;

			# Close the file.
			close(HOSTS);
		}
		else
		{
			# Log the error.
			LogMessage($Return = 1700002, 'error', 'critical');
		}
	}
	else
	{
		# Log the error.
		LogMessage($Return = 1700003, 'error', 'critical');
	}
	
	return $Return;
}

################################################################################
# SName: list_dns() - Creates a hashtable containing keys nameserver and domain.
# Arguments: None
# Return Value: Hash Table of nameserver and domain 
sub list_dns
{
my ($dom, %dom);
open(CONF, "/etc/resolv.conf");
flock(CONF, 2);
while(<CONF>) {
	s/\r|\n//g;
	if (/nameserver\s+(.*)/) {
		push(@{$dom->{ 'name' }}, $1);
		}
	elsif (/domain\s+(\S+)/) {
		$dom{'domain'} =  $1;
		$dom->{'domain'} = [ $1 ];
		}
		
	}
close(CONF);
return $dom;
}

###############################################################################
# SName: save_dns() - Modify the configuration or create the configuration in resolv.conf and file
# Arguments: Hash table generated by the list_dns and changed value
# Return Value:
#
sub save_dns
{
	open(DNS, "/etc/resolv.conf");
	my @dns_conf = <DNS>;
	close(DNS);
	open(DNS, ">/etc/resolv.conf");
	flock(NTP,2);
	if ($_[0]->{'domain'}) {
		if ($_[0]->{'domain'}-[1]) {
			print DNS "domain $_[0]->{'domain'}->[0]\n";
	
		}
	}
	foreach (@{$_[0]->{'name'}}) {
		print DNS "nameserver $_\n" if(defined($_) && ('' ne $_));
	}
	foreach (@dns_conf) {
		print DNS $_ if (!/^\s*(nameserver|domain)\s+/);
	}
	close(DNS);
}

################################################################################
# SName: save_host() - Save Hostname
# Arguments: hostname of the System
# Return Value:
#
sub save_host
{
	my ($newhost,$newdomain,$oldhost,$olddomain) = @_;
	my $Monitor		= new MonSettings();
	my $HostName = '/etc/HOSTNAME';
	my @data;
	$| = 1;
	Safe::system("/bin/hostname $_[0] >/dev/null 2>&1");
	open(HOST, ">$HostName");
	flock(HOST,2);	
	print HOST "$newhost\n";
	close(HOST);

        # Also save the host name in /etc/sysconfig/network...
        my $net = '/etc/sysconfig/network';
        open(NET,$net);
        while(<NET>) { push(@data,$_); }
        close(NET);

        open(NET,">$net");
        flock(NET,2);
	        foreach my $it (@data) {
                        if ($it =~ s/(HOSTNAME)=(\S+)/$1=$newhost/  ) {
                        }
                print NET "$it";
        }
        close(NET);

	# Copy Info to /etc/hosts

	my ($netaddr);
        my @netlist = NetConfig->listNetInterfaces();

        for my $nets (@netlist) {
                $netaddr = $nets->{ipaddress} if $nets->{fullnic} eq "eth0";
        }
	chomp($oldhost);
	
	# Open the configuration.
	if (0 == $Monitor->open())
	{
		# Update the domain name.
		$Monitor->modifyDomainName($newhost.'.'.$newdomain);

		# Close the configuration.
		$Monitor->close();
	}
	
	setHostFile();
}

###############################################################################
# VirtualServ::ftp_conf
###############################################################################

sub ftp_conf
{
	my ($Domain) = @_;
	my @FTPData;
	my $FTPAccess;
	
	# Set up the configuration file...
	$FTPAccess = ($Domain) ? "/usr/ins/intel/wu-ftpd/conf/$Domain/ftpaccess" : '/usr/ins/intel/wu-ftpd/conf/ftpaccess';
	
	# Try to open the configuration file for reading...
	if (open(CONFIGURATION, $FTPAccess))
	{
		my @FTPDataRaw;
		my $Line;

		# Run through the file...
		while (<CONFIGURATION>)
		{
			# Clean the line.
			s/\r|\n//g;

			# Cache the line. 
			push(@FTPDataRaw, $_);	
		}
		
		# Close the file.
		close(CONFIGURATION);

		# Read each line of virtual configuration and generate a hash array...
		foreach $Line (@FTPDataRaw)
		{
			# If there is a login failure limit...
			if ($Line =~ /loginfails (\S+)/)
			{
				# Cache the login failure limit.
				push(@FTPData, $1);
			}	
			
			# If there is a inactivity timeout...
			if ($Line =~ /timeout idle (\S+)/)
			{
				# Cache the inactivity timeout.
				push(@FTPData, $1);
			}	
			
			# If there is a byte limit...
			if ($Line =~ /byte-limit (\S+)/)
			{ 
				# Cache the byte limit.
				push(@FTPData, $1);
			}	
			
			# If there is a user limit...
			if ($Line =~ /limit \S+ (\S+)/)
			{
				# Cache the user limit.
				push(@FTPData, $1);
			}	
		}
	}
	else
	{
		# Log the error.
		LogMessage(1700004, 'error', 'warning', [ $FTPAccess ], $Domain);
	}
	
	return @FTPData;
}

###############################################################################
# VirtualServ::ftp_conf
###############################################################################

sub ftp_msg
{
	my ($Domain) = @_;
	my @FTPData;
	my $FTPWelcome;
	
	# Set up the configuration file...
	$FTPWelcome = ($Domain) ? "/usr/ins/intel/wu-ftpd/conf/$Domain/welcome.msg" : '/usr/ins/intel/wu-ftpd/conf/welcome.msg';
	
	# Try to open the configuration file for reading...
	if (open(MESSAGE, $FTPWelcome))
	{
		my $Line;

		# Run through the file...
		while (<MESSAGE>)
		{
			# Clean the line.
			s/\r|\n//g;

			# Cache the line. 
			push(@FTPData, $_);	
		}
		
		# Close the file.
		close(MESSAGE);
	}
	
	return @FTPData;
}

###############################################################################
# VirtualServ::ftp_anmsg
###############################################################################

sub ftp_anmsg
{
	my ($Domain) = @_;
	my @FTPData;
	my $FTPWelcome;
	
	# Set up the configuration file...
	$FTPWelcome = ($Domain) ? "/home/domain/$Domain/ftp/welcom.msg" : '/home/domain/domain/ftp/welcom.msg';
	
	# Try to open the configuration file for reading...
	if (open(MESSAGE, $FTPWelcome))
	{
		my $Line;

		# Run through the file...
		while (<MESSAGE>)
		{
			# Clean the line.
			s/\r|\n//g;

			# Cache the line. 
			push(@FTPData, $_);	
		}
		
		# Close the file.
		close(MESSAGE);
	}
	
	return @FTPData;
}

###############################################################################
# VirtualServ::checkAnon
###############################################################################

sub checkAnon 
{
	my ($Domain) = @_;
	my $path_for_conf = "/usr/ins/intel/wu-ftpd/conf/$Domain/ftpaccess";
	my $home = "/home/domain";
	my ($flag,$flag1);
	open(CONF,$path_for_conf);
	while(<CONF>) {
		$flag = 'yes' if (/root\s+$home/);
		$flag = 'yes' if (/anonymous-root\s+$home/);
	}
	my $path_for_def = "/usr/ins/intel/wu-ftpd/conf/ftpaccess";
        open(CONF,$path_for_def);
        while(<CONF>) {
                $flag1 = 'yes' if (/root\s+$home/);
		$flag1 = 'yes' if (/anonymous-root\s+$home/);
	}

	return ($flag,$flag1);
}

###############################################################################
# VirtualServ::setftp
###############################################################################

sub setftp
{
	my ($MaxLoginFailure, $IdleTimeout, $MaxSessionData, $MaxUserCount, $MessageAnonymous, $MessageUser, $Anonymous) = @_;
	my $ConfigurationDir	= '/usr/ins/intel/wu-ftpd/conf';
	my $AnonymousDir		= '/home/domain/domain/ftp';
	my $Return				= 0;
	
	# Try to open the anonymous message file...
	if (open(MESSAGEANON, '> /usr/ins/intel/wu-ftpd/conf/welcom.msg'))
	{
		# Try to open the user message file...
		if (open(MESSAGEUSER, "> $ConfigurationDir/welcome.msg"))
		{
			# Try to open the access configuration file...
			if (open(CONFIGURATION, "> $ConfigurationDir/ftpaccess"))
			{
				my $Configuration;
		
				# Safe defaults.
				$MaxLoginFailure	= 5    if((!defined($MaxLoginFailure)) || ('' eq $MaxLoginFailure));
				$MaxUserCount		= 2    if((!defined($MaxUserCount)) || ('' eq $MaxUserCount));
				$MaxSessionData		= 1100 if((!defined($MaxSessionData)) || ('' eq $MaxSessionData));
				$IdleTimeout		= 900  if((!defined($IdleTimeout)) || ('' eq $IdleTimeout));
				$MessageAnonymous	= ''   unless(defined($MessageAnonymous));
				$MessageUser		= ''   unless(defined($MessageUser));
				$Anonymous			= ''   unless(defined($Anonymous));

				# Lock the files.
				flock(MESSAGEANON, 2);
				flock(MESSAGEUSER, 2);
				
				# Print the messages.
				print MESSAGEANON "$MessageAnonymous";
				print MESSAGEUSER "$MessageUser";
		
				# Convert the session data.
				$MaxSessionData = $MaxSessionData * 1048576;
		
				# If anonymous login supported...
				if ('yes' eq $Anonymous)
				{
					# Set up the confuration block.
					$Anonymous = <<HERE;
anonymous-root /home/domain/domain/ftp
root /home/domain/domain/ftp
HERE
				}
				else
				{
					$Anonymous = '';
				}
		
				$Configuration = <<HERE;
greeting brief
$Anonymous
class   all   real,guest,anonymous  *
guestgroup %10001-65535
loginfails $MaxLoginFailure
timeout idle $IdleTimeout
data-limit total $MaxSessionData all 
limit all $MaxUserCount Any $ConfigurationDir/toomany.msg
email root\@localhost
readme  README*    login
readme  README*    cwd=*

banner $ConfigurationDir/welcome.msg login
message .message  cwd=*
noretrieve /etc/passwd /etc/group
# This is default for all
compress        yes all 
tar             yes all
chmod           no  anonymous
delete          no  anonymous
overwrite       no  anonymous
rename          no  anonymous
log transfers   real,anonymous,guest  inbound,outbound
shutdown /etc/shutmsg
passwd-check rfc822 warn
chmod           yes real,guest
delete          yes real,guest
overwrite       yes real,guest
rename          yes guest,real
logfile /var/log 
HERE
				
				# Lock the file.
				flock(CONFIGURATION, 2);
				
				# Set the configuration.
				print CONFIGURATION "$Configuration";
				
				# Close the file.
				close(CONFIGURATION);
			}
			else
			{
				# Log the error.
				LogMessage($Return = 1700005, 'error', 'critical', [ "$ConfigurationDir/ftpaccess" ]);
			}
			
			# Close the file.
			close(MESSAGEUSER);
		}
		else
		{
			# Log the error.
			LogMessage($Return = 1700005, 'error', 'critical', [ "$ConfigurationDir/welcome.msg" ]);
		}
		
		# Close the file.
		close(MESSAGEANON);
	}
	else
	{
		# Log the error.
		LogMessage($Return = 1700005, 'error', 'critical', [ '/usr/ins/intel/wu-ftpd/conf/welcom.msg' ]);
	}

	return $Return;
}

###############################################################################
# VirtualServ::set_virtual_ftp
###############################################################################

sub set_virtual_ftp
{
	my ($DomainDir, $DomainID, $DomainName, $DomainAdministrator, $MaxLoginFailure, $IdleTimeout, $MaxSessionData, $MaxUserCount, $MessageUser, $Anonymous) = @_;
	my $ConfigurationDir	= '/usr/ins/intel/wu-ftpd/conf';
	my $AnonymousDir		= "$DomainDir/$DomainID/ftp";
	my $Return				= 0;
	
	# Try to open the anonymous message file...
	if (open(MESSAGEANON, "> $AnonymousDir/welcom.msg"))
	{
		# Try to open the user message file...
		if (open(MESSAGEUSER, "> $ConfigurationDir/$DomainID/welcome.msg"))
		{
			# Try to open the access configuration file...
			if (open(CONFIGURATION, "> $ConfigurationDir/$DomainID/ftpaccess"))
			{
				my $MessageAnonymous;
				my $Configuration;
				my $LogFile				= "$DomainDir/$DomainID/logs/";
		
				# Safe defaults.
				$MaxLoginFailure	= 5   if((!defined($MaxLoginFailure)) || ('' eq $MaxLoginFailure));
				$MaxUserCount		= 100 if((!defined($MaxUserCount)) || ('' eq $MaxUserCount));
				$MaxSessionData		= 10  if((!defined($MaxSessionData)) || ('' eq $MaxSessionData));
				$IdleTimeout		= 600 if((!defined($IdleTimeout)) || ('' eq $IdleTimeout));
				$MessageAnonymous	= ''  unless(defined($MessageAnonymous));
				$MessageUser		= ''  unless(defined($MessageUser));
				$Anonymous			= ''  unless(defined($Anonymous));

				# Lock the files.
				flock(MESSAGEANON, 2);
				flock(MESSAGEUSER, 2);
				
				# Print the messages.
				print MESSAGEANON "$MessageAnonymous";
				print MESSAGEUSER "$MessageUser";
		
				# Convert the session data.
				$MaxSessionData = $MaxSessionData * 1048576;
		
				# If anonymous login supported...
				if ('yes' eq $Anonymous)
				{
					# Set up the confuration block.
					$Anonymous = <<HERE;
anonymous-root $AnonymousDir
root $AnonymousDir
HERE
				}
				else
				{
					$Anonymous = '';
				}
		
				$Configuration = <<HERE;
greeting brief
$Anonymous
class   all   real,guest,anonymous  *
guestgroup %10001-65535
loginfails $MaxLoginFailure
timeout idle $IdleTimeout
data-limit total $MaxSessionData all
limit all $MaxUserCount Any $ConfigurationDir\/toomany.msg
email $DomainAdministrator\@$DomainName
readme  README*    login
readme  README*    cwd=*
banner $ConfigurationDir\/$DomainID\/welcome.msg 
message .message  cwd=*
noretrieve /etc/passwd /etc/group
# This is default for all
compress        yes all
tar             yes all 
chmod           no anonymous
delete          no anonymous
overwrite       no anonymous
rename          no anonymous
log transfers   real,anonymous,guest  inbound,outbound
shutdown /etc/shutmsg
passwd-check rfc822 warn
chmod           yes real,guest
delete          yes real,guest
overwrite       yes real,guest
rename          yes guest,real
logfile $LogFile
HERE
				
				# Lock the file.
				flock(CONFIGURATION, 2);
				
				# Set the configuration.
				print CONFIGURATION "$Configuration";
				
				# Close the file.
				close(CONFIGURATION);
			}
			else
			{
				# Log the error.
				LogMessage($Return = 1700005, 'error', 'critical', [ "$ConfigurationDir/$DomainID/ftpaccess" ], $DomainID);
			}
			
			# Close the file.
			close(MESSAGEUSER);
		}
		else
		{
			# Log the error.
			LogMessage($Return = 1700005, 'error', 'critical', [ "$ConfigurationDir/$DomainID/welcome.msg" ], $DomainID);
		}
		
		# Close the file.
		close(MESSAGEANON);
	}
	else
	{
		# Log the error.
		LogMessage($Return = 1700005, 'error', 'critical', [ "$AnonymousDir/welcom.msg" ], $DomainID);
	}

	return $Return;
}

1;

__END__
